@@ -72,7 +72,7 @@ public class PIMAttachments implements Serializable {
 
     // Special flag that can be set to force this object to differ from other PIMAttachments (to enforce that a SyncState is written, since
     // normal equals only checks for the timestamp, not for the contained PIMAttachment objects)
-    private transient final boolean _forceModified;
+    private transient boolean _forceModified;
 
     public PIMAttachments(long timestamp, PIMAttachment... attachments) {
         this(timestamp, false, attachments);
@@ -114,6 +114,10 @@ public class PIMAttachments implements Serializable {
         return _forceModified;
     }
 
+    public void setForceModified(boolean forceModified) {
+        _forceModified = forceModified;
+    }
+
     @Override
     public int hashCode() {
         final int prime = 31;
