@@ -192,6 +192,18 @@ public interface Session {
      */
     Object setCustomProperty(String key, Object value);
 
+    /**
+     * This method behaves as setCustomProperty(String, Object), except the value will be automatically removed after the given timeout.
+     * This can be used for cache-like accesses, data that gets invalid after some time or for large data that should not be stored for too
+     * long.
+     * 
+     * @param key
+     * @param value
+     * @param timeout timeout in milliseconds after which the stored value should be removed
+     * @return previously stored Object
+     */
+    Object setCustomProperty(String key, Object value, long timeout);
+
     /**
      * Used for requests which need date range as parameter, some ContentTypes require a date range to be specified for their requests to
      * the OX server. Default value is 0l. <br>
@@ -243,24 +255,35 @@ public interface Session {
      * @return
      */
     int getMailLimit();
-    
-    /**
-     * Sets a limit for the number of objects that will be reported when syncing a folder. If on the server more objects than the specified
-     * limit are stored in a folder, only the newest ones (upto the limit) will be used for synchronization, older one will be treated as
-     * not present. A value <= 0 specifies that no limit should be used and all objects should be synchronized.
-     * 
-     * @param limit
-     */
-    void setObjectsLimit(int limit);
 
     /**
-     * Retrieves the currently set limit for objects in a folder.
+     * Retrieves the configured limit for synchronized objects per folder. If on the server more objects than the specified limit are stored
+     * in a folder, only the newest ones (upto the limit) will be used for synchronization, older one will be treated as not present. A
+     * value <= 0 specifies that no limit should be used and all objects should be synchronized.
      * 
      * @return
      */
     int getObjectsLimit();
-    
-   
+
+    /**
+     * Returns the configured limit for the number of synchronized PIM attachments per object. If on the server more files are attached to 1
+     * object, only the first ones (default: lowest OX ID) will be synchronized, all others will be treated as not present. A value < 0
+     * specifies that no limit should be used and all PIM attachments should be synchronized. Limit is applied independently of
+     * getPIMAttachmentsSizeLimit()
+     * 
+     * @return
+     */
+    int getPIMAttachmentsCountLimit();
+
+    /**
+     * Returns the configured size limit for the synchronized PIM attachments per object. If on the server the size of all attachments to 1
+     * object exceeds this limit, only the first ones (default: lowest OX ID) will be synchronized, all others will be treated as not
+     * present. A value < 0 specifies that no size limit should be used and all PIM attachments should be synchronized. Limit is applied
+     * independently of getPIMAttachmentsCountLimit()
+     * 
+     * @return
+     */
+    long getPIMAttachmentsSizeLimit();
 
     // -- Listening / waiting for changes on the server --
     /**
@@ -905,8 +928,6 @@ public interface Session {
 
     Map<String, String> getXRequestHeaders();
 
-    void setXRequestHeaders(Map<String, String> xHeaders);
-
     /**
      * Gets the newest known timestamp for the folder.
      * 
