@@ -50,10 +50,7 @@
 package com.openexchange.usm.clt;
 
 import java.io.IOException;
-import java.io.UnsupportedEncodingException;
-import java.net.URLDecoder;
 import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
@@ -64,7 +61,6 @@ import javax.management.MBeanServerConnection;
 import javax.management.ReflectionException;
 import javax.management.remote.JMXConnector;
 import javax.management.remote.JMXConnectorFactory;
-import javax.management.remote.JMXConnectorServer;
 import javax.management.remote.JMXServiceURL;
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.CommandLineParser;
@@ -74,6 +70,8 @@ import org.apache.commons.cli.OptionGroup;
 import org.apache.commons.cli.Options;
 import org.apache.commons.cli.ParseException;
 import org.apache.commons.cli.PosixParser;
+import com.openexchange.usm.clt.eas12.EASBase64Query;
+import com.openexchange.usm.clt.eas12.EASBase64RequestParser;
 
 /**
  * {@link USMSessionTool} Tool that displays information about USM sessions and allows enabling/disabling of the EAS debug logging.
@@ -128,39 +126,10 @@ public class USMSessionTool {
             "usmsessions [USM-Request-URL-parameters or USM-Session-filter]+",
             "",
             toolOptions,
-            "Options -d,-e,-r are mutually exclusive, as are options -q,-v.\nArguments may be given as EAS-POST-URL-parameters, OLOX2-PUT-URL-parameters, or an USM-session-filter.\nAn USM-session-filter consists of 5 parts, separated by colon(\":\"):\nContext-ID:OX-User-ID:USM-Session-ID:USM-Protocol:Device-ID\nEach part may be left empty to match any value.\nCurrently available USM-protocols are: EAS, JSON.\nExamples (from webserver-log):\nEAS: /Microsoft-Server-ActiveSync?Cmd=Sync&User=user&DeviceId=SomeDeviceId&DeviceType=Android\nOLOX2:\n/usm-json/syncUpdate?id=SomeId");
+            "Options -d,-e,-r are mutually exclusive, as are options -q,-v.\nArguments may be given as EAS-POST-URL-parameters, OLOX2-PUT-URL-parameters, or an USM-session-filter.\nAn USM-session-filter consists of 5 parts, separated by colon(\":\"):\nContext-ID:OX-User-ID:USM-Session-ID:USM-Protocol:Device-ID\nEach part may be left empty to match any value.\nEAS-Request-URLs are only specific to a device (so multiple sessions may be reported/affected for different users using that device).\nCurrently available USM-protocols are: EAS, JSON.\nExamples (from webserver-log):\nEAS: /Microsoft-Server-ActiveSync?Cmd=Sync&User=user&DeviceId=SomeDeviceId&DeviceType=Android\nEAS12: /Microsoft-Server-ActiveSync?eQAHBBCDDrY4vSg/QBtTxnnNJ1zwBGBI6k0LV2luZG93c01haWw=\nOLOX2:\n/usm-json/syncUpdate?id=SomeId");
         System.exit(0);
     }
 
-    private static String getOptionStringValue(CommandLine cmd, char opt) throws ParseException {
-        String val = cmd.getOptionValue(opt);
-        if (!provided(val))
-            throw new ParseException("Option \"" + opt + "\" requires a non-empty string value");
-        return val;
-    }
-
-    private static int getOptionIntValue(CommandLine cmd, char opt, int minValue, int maxValue) throws ParseException {
-        String val = cmd.getOptionValue(opt);
-        if (val != null) {
-            try {
-                int intVal = Integer.parseInt(val);
-                if (intVal >= minValue && intVal <= maxValue)
-                    return intVal;
-            } catch (NumberFormatException ignored) {
-                // fall through
-            }
-        }
-        throw new ParseException("Option \"" + opt + "\" requires an integer value between " + minValue + " and " + maxValue);
-    }
-
-    private static int parseIntegerValue(String part) {
-        return provided(part) ? Integer.parseInt(part) : 0;
-    }
-
-    private static boolean provided(String s) {
-        return s != null && s.length() > 0;
-    }
-
     public static void main(String[] args) {
         new USMSessionTool(args).execute();
     }
@@ -194,9 +163,9 @@ public class USMSessionTool {
                 printHelpAndExit();
             }
             if (cmd.hasOption('l'))
-                _jmxLogin = getOptionStringValue(cmd, 'l');
+                _jmxLogin = CLToolsUtility.getOptionStringValue(cmd, 'l');
             if (cmd.hasOption('p'))
-                _jmxPort = getOptionIntValue(cmd, 'p', 1, 65535);
+                _jmxPort = CLToolsUtility.getOptionIntValue(cmd, 'p', 1, 65535);
             if (cmd.hasOption('s'))
                 _jmxPassword = cmd.getOptionValue('s');
             if (cmd.hasOption('e'))
@@ -213,9 +182,10 @@ public class USMSessionTool {
             if (data == null || data.length == 0) {
                 System.err.println("No arguments specified");
                 printHelpAndExit();
+            } else {
+                for (String entry : data)
+                    _data.add(entry);
             }
-            for (String entry : data)
-                _data.add(entry);
         } catch (ParseException e) {
             System.err.println("Unable to parse command line: " + e.getMessage());
             printHelpAndExit();
@@ -226,13 +196,15 @@ public class USMSessionTool {
         try {
             final JMXServiceURL url = new JMXServiceURL(
                 new StringBuilder("service:jmx:rmi:///jndi/rmi://localhost:").append(_jmxPort).append("/server").toString());
-            final JMXConnector jmxConnector = JMXConnectorFactory.connect(url, createJMXEnvironment());
+            final JMXConnector jmxConnector = JMXConnectorFactory.connect(url, CLToolsUtility.createJMXEnvironment(_jmxLogin, _jmxPassword));
             try {
                 _connection = jmxConnector.getMBeanServerConnection();
                 for (String data : _data) {
                     if (handleCustomData(data))
                         continue;
-                    Map<String, String> urlParameters = extractURLParameters(data);
+                    Map<String, String> urlParameters = CLToolsUtility.extractURLParameters(data);
+                    if (handleBase64ActiveSyncData(data))
+                        continue;
                     if (handleActiveSyncData(urlParameters))
                         continue;
                     if (handleOlox2Data(urlParameters))
@@ -254,9 +226,9 @@ public class USMSessionTool {
         String[] parts = data.split(":", -1);
         if (parts.length == 5) {
             try {
-                int cid = parseIntegerValue(parts[0]);
-                int id = parseIntegerValue(parts[1]);
-                int usmSessionId = parseIntegerValue(parts[2]);
+                int cid = CLToolsUtility.parseIntegerValue(parts[0]);
+                int id = CLToolsUtility.parseIntegerValue(parts[1]);
+                int usmSessionId = CLToolsUtility.parseIntegerValue(parts[2]);
                 workOnUSMSession(cid, id, usmSessionId, parts[3], parts[4]);
                 return true;
             } catch (NumberFormatException nfe) {
@@ -267,9 +239,28 @@ public class USMSessionTool {
     }
 
     private boolean handleActiveSyncData(Map<String, String> urlParameters) throws InstanceNotFoundException, MBeanException, ReflectionException, IOException {
-        String user = urlParameters.get(_EAS_USER_PARAMETER);
+        // String user = urlParameters.get(_EAS_USER_PARAMETER);
         String deviceId = urlParameters.get(_EAS_DEVICE_ID_PARAMETER);
-        if (!provided(user) || !provided(deviceId))
+        return handleCommonActiveSyncData(deviceId);
+    }
+
+    private boolean handleBase64ActiveSyncData(String data) throws InstanceNotFoundException, MBeanException, ReflectionException, IOException {
+        int split = data.lastIndexOf('?');
+        if (split >= 0)
+            data = data.substring(split + 1);
+        EASBase64Query query;
+        try {
+            query = new EASBase64Query(new EASBase64RequestParser(data));
+        } catch (Exception e) {
+            return false;
+        }
+        // String user = query.getStringParameter(Parameter.RoundTripId);
+        String deviceId = query.getDeviceID();
+        return handleCommonActiveSyncData(deviceId);
+    }
+
+    protected boolean handleCommonActiveSyncData(String deviceId) throws InstanceNotFoundException, MBeanException, ReflectionException, IOException {
+        if (!CLToolsUtility.provided(deviceId))
             return false;
         workOnUSMSession(0, 0, 0, _USM_PROTOCOL_EAS, deviceId);
         return true;
@@ -319,7 +310,7 @@ public class USMSessionTool {
                     System.out.println(session);
                     reportEASDebugLogging(session);
                 }
-                if (provided(_enableEASDebugLogging)) {
+                if (CLToolsUtility.provided(_enableEASDebugLogging)) {
                     if (session.getProtocol().equals(_USM_PROTOCOL_EAS)) {
                         setSessionProperty(session, _USM_FIELD_EAS_DEBUG_LOG, _enableEASDebugLogging);
                         if (_verbose)
@@ -346,8 +337,8 @@ public class USMSessionTool {
         if (session.getProtocol().equals(_USM_PROTOCOL_EAS)) {
             String easLogging = session.getFields().get(_USM_FIELD_EAS_DEBUG_LOG);
             if (_quiet)
-                System.out.println(provided(easLogging) ? easLogging : "");
-            else if (provided(easLogging))
+                System.out.println(CLToolsUtility.provided(easLogging) ? easLogging : "");
+            else if (CLToolsUtility.provided(easLogging))
                 System.out.println("   EAS Debug Logging is activated with keyword \"" + easLogging + "\"");
             else
                 System.out.println("   EAS Debug Logging is not activated");
@@ -379,41 +370,4 @@ public class USMSessionTool {
         }
     }
 
-    private static Map<String, String> extractURLParameters(String data) {
-        Map<String, String> result = new HashMap<String, String>();
-        String params = cropToParameters(data);
-        for (String entry : params.split("&")) {
-            String[] parts = entry.split("=", 2);
-            if (parts.length >= 2) {
-                try {
-                    result.put(parts[0], URLDecoder.decode(parts[1], "UTF-8"));
-                } catch (UnsupportedEncodingException e) {
-                    throw new IllegalStateException("Required encoding UTF-8 is not supported", e);
-                }
-            } else if (parts.length >= 1) {
-                result.put(parts[0], "");
-            }
-        }
-        return result;
-    }
-
-    private static String cropToParameters(String data) {
-        int paramStart = data.indexOf('?');
-        if (paramStart >= 0)
-            data = data.substring(paramStart + 1);
-        int paramEnd1 = data.indexOf('#');
-        int paramEnd2 = data.indexOf(' ');
-        if (paramEnd1 >= 0 && paramEnd2 >= paramEnd1)
-            return data.substring(0, paramEnd1);
-        if (paramEnd2 < 0)
-            return data;
-        return data.substring(0, paramEnd2);
-    }
-
-    private Map<String, Object> createJMXEnvironment() {
-        Map<String, Object> environment = new HashMap<String, Object>();
-        if (_jmxLogin != null && _jmxPassword != null)
-            environment.put(JMXConnectorServer.AUTHENTICATOR, new JMXAuthenticatorImpl(_jmxLogin, _jmxPassword));
-        return environment;
-    }
 }
