@@ -58,6 +58,8 @@ package com.openexchange.usm.configuration;
  */
 public class ConfigurationProperties {
 
+    // --- USM ---
+    
     /**
      * Read USM access rights from configuration.
      */
@@ -116,17 +118,6 @@ public class ConfigurationProperties {
      */
     public static final String OX_ACCESS_URL_DEFAULT = "http://localhost/ajax/";
 
-    /**
-     * Used by the OXAjaxAccess bundle to determine pattern for results of post requests to OX server. The action will be determined using
-     * the key "**action**", the result will be determined using the key "**json**".
-     */
-    public static final String OX_ACCESS_POST_RESULT_PATTERN_PROPERTY = "com.openexchange.usm.ox.post_result_pattern";
-
-    /**
-     * Default value for the pattern for results of post requests to the OX server.
-     */
-    public static final String OX_ACCESS_POST_RESULT_PATTERN_DEFAULT = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\"\"http://www.w3.org/TR/html4/strict.dtd\"><html><head><META http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"> <script type=\"text/javascript\"> function callback(arg) { parent.callback_**action**(arg); }; callback(**json**);</script></head></html>";
-
     /**
      * Specifies the overall maximum number of connections the USM may use for requests to the OX server.
      */
@@ -156,6 +147,16 @@ public class ConfigurationProperties {
      * Default value for stale checking of Connections before using them (increases security, lowers performance).
      */
     public static final boolean OX_ACCESS_PERFORM_STALE_CHECKING_DEFAULT = true;
+    
+    /**
+     * If this property is set to true, usm sets the host header in the login request to the one received from client. To be able to do this, a system property sun.net.http.allowRestrictedHeaders is also set to true
+     */
+    public static final String OX_ACCESS_PASS_HOST_HEADER_FROM_CLIENT = "com.openexchange.usm.pass_client_host_header";
+
+    /**
+     * Default value for com.openexchange.usm.pass_client_host_header
+     */
+    public static final boolean OX_ACCESS_PASS_HOST_HEADER_FROM_CLIENT_DEFAULT = true;
 
     /**
      * Used by the slow and incremental syncer to limit the number of retries if a concurrent modification is detected while performing a
@@ -254,7 +255,30 @@ public class ConfigurationProperties {
      */
     public static final int SYNC_CACHE_TIME_LIMIT_INCOMPLETE_DEFAULT = 1200000; // 20 minutes
 
-    // Database connection properties
+    public static final String USM_FORCE_NEW_SYNC_KEY_ON_EMPTY_SYNC = "com.openexchange.usm.force_new_sync_key_on_empty_sync";
+
+    public static final boolean USM_FORCE_NEW_SYNC_KEY_ON_EMPTY_SYNC_DEFAULT = false;
+
+    public static final String OBJECTS_IN_FOLDER_SYNC_LIMIT_PROPERTY = "com.openexchange.usm.max_objects_per_folder";
+
+    public static final int OBJECTS_IN_FOLDER_SYNC_LIMIT_DEFAULT = 65535;
+
+    public static final String ATTACHMENT_COUNT_LIMIT_PER_OBJECT_PROPERTY = "com.openexchange.usm.max_attachment_count_per_object";
+
+    public static final int ATTACHMENT_COUNT_LIMIT_PER_OBJECT_DEFAULT = 100;
+
+    public static final String ATTACHMENT_SIZE_LIMIT_PER_OBJECT_PROPERTY = "com.openexchange.usm.max_attachment_size_per_object";
+
+    public static final long ATTACHMENT_SIZE_LIMIT_PER_OBJECT_DEFAULT = 50L * 1024L * 1024L;
+
+    /**
+     * Defines a timeout after which unused temporary files uploaded by a protocol will be automatically removed
+     */
+    public static final String TEMP_FILES_TIMEOUT_PROPERTY = "com.openexchange.usm.temp_files.timeout";
+
+    public static final int TEMP_FILES_TIMEOUT_DEFAULT = 7200000; // 2 hours
+
+    // Database connection properties (only used in stand-alone mode)
 
     public static final String MYSQL_DATABASE_HOST_PROPERTY = "com.openexchange.usm.database.mysql.host";
 
@@ -287,6 +311,8 @@ public class ConfigurationProperties {
     public static final String HSQL_PASSWORD_PROPERTY = "com.openexchange.usm.database.hsql.password";
 
     public static final String HSQL_PASSWORD_DEFAULT = "";
+    
+    // --- JSON/OLOX2 ---
 
     // TODO Create properties for this value ! Why this strange default value ?
     public static final long START_DATE_MINUS_DEFAULT = 311040000L;// 36000*24*360
@@ -304,7 +330,7 @@ public class ConfigurationProperties {
 
     public static final String JSON_SERVLET_PING_MAX_INTERVAL = "com.openexchange.usm.json.ping.max_interval";
 
-    public static final int JSON_SERVLET_PING_MAX_INTERVAL_DEFAULT = Integer.MAX_VALUE;
+    public static final int JSON_SERVLET_PING_MAX_INTERVAL_DEFAULT = 360000;
 
     public static final String JSON_SERVLET_PING_MAX_REQUEST_PER_SECOND = "com.openexchange.usm.json.ping.max_requests_per_second";
 
@@ -330,6 +356,10 @@ public class ConfigurationProperties {
 
     public static final int JSON_SERVLET_MAX_EMAIL_SIZE_DEFAULT = 50 * 1024 * 1024; // 50 MByte
 
+    public static final String JSON_SERVLET_MAX_INLINE_ATTACHMENT_SIZE_PROPERTY = "com.openexchange.usm.json.max_inline_attachment_size";
+
+    public static final int JSON_SERVLET_MAX_INLINE_ATTACHMENT_SIZE_DEFAULT = 1024 * 1024; // 1 MByte
+
     /**
      * Defines the lower time filter limit in days. Appointments and mails which are in the past beyond this limit will not be synchronised
      * from/to the client.
@@ -363,8 +393,10 @@ public class ConfigurationProperties {
      */
     public static final String JSON_SERVLET_INBOX_POLLING_INTERVAL_PROPERTY = "com.openexchange.usm.json.sync.inbox_polling_interval";
 
-    public static final int JSON_SERVLET_INBOX_POLLING_INTERVAL_DEFAULT = 60000; // 5 min
-
+    public static final int JSON_SERVLET_INBOX_POLLING_INTERVAL_DEFAULT = 600000; // 10 min
+    
+    // --- SyncML ---
+    
     public static final String SYNCML_SERVLET_ALIAS_PROPERTY = "com.openexchange.usm.syncml.alias";
 
     public static final String SYNCML_SERVLET_ALIAS_DEFAULT = "/usm-syncml";
@@ -392,19 +424,4 @@ public class ConfigurationProperties {
     public static final String SYNCML_DEBUG_LOG_PROPERTY = "com.openexchange.usm.syncml.debug_log";
 
     public static final String SYNCML_DEBUG_LOG_DEFAULT = null;
-
-    /**
-     * Temorary fix bug 20085: property to ignore client data at slow sync. TODO: remove when the bug is detected and fixed!!!
-     */
-    public static final String USM_IGNORE_CLIENT_DATA_AT_SLOW_SYNC = "com.openexchange.usm.ignore_client_data_at_slow_sync";
-
-    public static final boolean USM_IGNORE_CLIENT_DATA_AT_SLOW_SYNC_DEFAULT = false; // false
-
-    public static final String USM_FORCE_NEW_SYNC_KEY_ON_EMPTY_SYNC = "com.openexchange.usm.force_new_sync_key_on_empty_sync";
-
-    public static final boolean USM_FORCE_NEW_SYNC_KEY_ON_EMPTY_SYNC_DEFAULT = false;
-
-    public static final String OBJECTS_IN_FOLDER_SYNC_LIMIT_PROPERTY = "com.openexchange.usm.max_objects_per_folder";
-
-    public static final int OBJECTS_IN_FOLDER_SYNC_LIMIT_DEFAULT = 65535;
 }
