@@ -49,15 +49,11 @@
 package com.openexchange.usm.contenttypes.calendar.impl;
 
 import static com.openexchange.usm.api.datatypes.DataTypes.*;
-
 import java.util.BitSet;
-
 import com.openexchange.usm.api.contenttypes.AppointmentContentType;
 import com.openexchange.usm.api.contenttypes.ContentTypeField;
 import com.openexchange.usm.api.contenttypes.ContentTypeTransferHandler;
 import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
-import com.openexchange.usm.api.datatypes.PIMAttachment;
-import com.openexchange.usm.api.datatypes.PIMAttachments;
 import com.openexchange.usm.api.datatypes.PIMAttachmentsDataType;
 import com.openexchange.usm.api.exceptions.AuthenticationFailedException;
 import com.openexchange.usm.api.exceptions.OXCommunicationException;
@@ -187,26 +183,6 @@ public class AppointmentContentTypeImpl extends AbstractContentType implements A
 		return DefaultContentTypes.CALENDAR_CODE;
 	}
 
-	public long createNewAttachment(DataObject object, PIMAttachment attachment) throws USMException {
-		return _transferHandler.createNewAttachment(object, attachment);
-	}
-
-	public long deleteAttachments(DataObject object, PIMAttachment... attachmentsToDelete) throws USMException {
-		return _transferHandler.deleteAttachments(object, attachmentsToDelete);
-	}
-
-	public PIMAttachments getAllAttachments(DataObject object, String sort, String order) throws USMException {
-		return _transferHandler.getAllAttachments(object, sort, order);
-	}
-
-	public PIMAttachments getAllAttachments(DataObject object) throws USMException {
-		return _transferHandler.getAllAttachments(object);
-	}
-
-	public byte[] getAttachmentData(DataObject object, int attachmentId) throws USMException {
-		return _transferHandler.getAttachmentData(object, attachmentId);
-	}
-
 	public boolean supportsPIMAttachments() {
 		return true;
 	}
