@@ -56,16 +56,14 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.TimeZone;
-
 import org.apache.commons.logging.Log;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
-
 import com.openexchange.usm.api.contenttypes.AppointmentContentType;
 import com.openexchange.usm.api.contenttypes.ContentTypeField;
 import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
-import com.openexchange.usm.api.contenttypes.UnsupportedContentOperationException;
+import com.openexchange.usm.api.contenttypes.OXFolderContent;
 import com.openexchange.usm.api.datatypes.DataTypes;
 import com.openexchange.usm.api.exceptions.InternalUSMException;
 import com.openexchange.usm.api.exceptions.OXCommunicationException;
@@ -81,6 +79,7 @@ import com.openexchange.usm.datatypes.tasks.calendar.UserParticipantObject;
 import com.openexchange.usm.ox_json.JSONResult;
 import com.openexchange.usm.ox_json.JSONResultType;
 import com.openexchange.usm.ox_json.OXJSONAccess;
+import com.openexchange.usm.session.dataobject.DataObjectUtil;
 
 /**
  * Transfer Handler for the module "Calendar"
@@ -91,74 +90,91 @@ public class AppointmentContentTypeTransferHandler extends AbstractTransferHandl
 
 	private static final long ONE_DAY_IN_MILLISECONDS = 86400000L;
 	private static final long MAXIMUM_EXPECTED_EXCEPTION_MOVEMENT = ONE_DAY_IN_MILLISECONDS * 365L;
-
-	private final BitSet DELETE_EXCEPTION_FIELDS = new BitSet();
+    
+    private final BitSet _DELETE_EXCEPTION_FIELDS = new BitSet();
 
 	public AppointmentContentTypeTransferHandler(AppointmentContentType appointmentContentType, Log log,
 			OXJSONAccess ajaxAccess) {
 		super(appointmentContentType, log, ajaxAccess);
-		addToDeleteExceptionsBitMask(appointmentContentType.getFields(), UtilConstants.FIELD_ID,
+		addToFieldBitMask(_DELETE_EXCEPTION_FIELDS, appointmentContentType.getFields(), UtilConstants.FIELD_ID,
 				UtilConstants.FOLDER_ID, CommonCalendarTasksFieldNames.START_DATE,
 				CommonCalendarTasksFieldNames.RECURRENCE_POSITION,
 				CommonCalendarTasksFieldNames.RECURRENCE_DATE_POSITION, CalendarConstants.RECURRENCE_ID,
 				CalendarConstants.TIMEZONE);
 	}
 
-	private void addToDeleteExceptionsBitMask(ContentTypeField[] fields, String... requiredFields) {
+	private void addToFieldBitMask(BitSet mask, ContentTypeField[] fields, String... requiredFields) {
 		for (String f : requiredFields)
-			addToDeleteExceptionsBitMask(f, fields);
+			addToFieldBitMask(mask, f, fields);
 	}
 
-	private void addToDeleteExceptionsBitMask(String f, ContentTypeField[] fields) {
+	private void addToFieldBitMask(BitSet mask, String f, ContentTypeField[] fields) {
 		for (int i = 0; i < fields.length; i++) {
 			if (f.equals(fields[i].getFieldName())) {
-				DELETE_EXCEPTION_FIELDS.set(i);
+				mask.set(i);
 				return;
 			}
 		}
 		throw new IllegalStateException("Unknown field <" + f + "> in ContentType <" + _contentType + ">");
 	}
 
-	@Override
-	public DataObject[] readFolderContent(Folder folder, BitSet requestedFields) throws USMException {
-		if (_journal.isDebugEnabled())
-			_journal.debug(folder.getSession() + " Read all " + _contentType.getID() + " in folder " + folder.getID());
-		Map<String, String> parameters = new HashMap<String, String>();
-		parameters.put(UtilConstants.FOLDER, folder.getID());
-		parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
-		parameters.put(UtilConstants.RECURRENCE_MASTER, Boolean.TRUE.toString());
-		Session session = folder.getSession();
-		parameters.put(UtilConstants.START, String.valueOf(session.getStartDate()));
-		parameters.put(UtilConstants.END, String.valueOf(session.getEndDate()));
-		if(folder.getSession().getObjectsLimit() > 0) {
-		    addObjectsLimitParameters(folder.getSession().getObjectsLimit(), parameters);
-		}
-		DataObject[] result = performActionAndCheckRights(UtilConstants.ACTION_ALL, folder, requestedFields, parameters);
+    @Override
+    public OXFolderContent readFolderContent(Folder folder, int limit) throws USMException {
+        OXFolderContent result = super.readFolderContent(folder, limit);
+        resetRecurrenceTypeBasedFields(result.getObjects());
+        return result;
+    }
 
-		resetRecurrenceTypeBasedFields(result);
-		return result;
-	}
-	
+    @Override
+    public DataObject[] readFolderContent(Folder folder, BitSet requestedFields) throws USMException {
+        if (_journal.isDebugEnabled())
+            _journal.debug(folder.getSession() + " Read all " + _contentType.getID() + " in folder " + folder.getID());
+        Map<String, String> parameters = new HashMap<String, String>();
+        parameters.put(UtilConstants.FOLDER, folder.getID());
+        parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
+        parameters.put(UtilConstants.RECURRENCE_MASTER, Boolean.TRUE.toString());
+        Session session = folder.getSession();
+        parameters.put(UtilConstants.START, String.valueOf(session.getStartDate()));
+        parameters.put(UtilConstants.END, String.valueOf(session.getEndDate()));
+        addObjectsLimitParameters(folder.getSession(), parameters);
+        DataObject[] result = performActionAndCheckRights(UtilConstants.ACTION_ALL, folder, requestedFields, parameters);
 
-	public DataObject[] getAllAppointments(Session session, BitSet requestedFields) throws USMException {
-		if (_journal.isDebugEnabled())
-			_journal.debug(session + " Getting new appointments for date range " + session.getStartDate() + " - "
-					+ session.getEndDate());
-		Map<String, String> parameters = new HashMap<String, String>();
+        resetRecurrenceTypeBasedFields(result);
+        return result;
+    }
 
-		parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
-		parameters.put(UtilConstants.START, String.valueOf(session.getStartDate()));
-		parameters.put(UtilConstants.END, String.valueOf(session.getEndDate()));
-		parameters.put(UtilConstants.RECURRENCE_MASTER, Boolean.TRUE.toString());
-		
-		if(session.getObjectsLimit() > 0) {
-            addObjectsLimitParameters(session.getObjectsLimit(), parameters);
+    @Override
+    protected void addExtraTimestampListParameters(Map<String, String> parameters) {
+        parameters.put(UtilConstants.RECURRENCE_MASTER, Boolean.TRUE.toString());
+    }
+
+    @Override
+    protected void finishDataObjectFromOptimizedRead(DataObject o) {
+        int recurrenceType = DataObjectUtil.getIntValue(o, CommonCalendarTasksFieldNames.RECURRENCE_TYPE);
+        if (recurrenceType != 0) {
+            int seriesId = DataObjectUtil.getIntValue(o, CalendarConstants.RECURRENCE_ID);
+            if (String.valueOf(seriesId).equals(o.getID())) {
+                o.setFieldContent(CommonCalendarTasksFieldNames.RECURRENCE_POSITION, null);
+                o.setFieldContent(CommonCalendarTasksFieldNames.RECURRENCE_DATE_POSITION, null);
+            }
         }
+    }
+	
+    public DataObject[] getAllAppointments(Session session, BitSet requestedFields) throws USMException {
+        if (_journal.isDebugEnabled())
+            _journal.debug(session + " Getting new appointments for date range " + session.getStartDate() + " - " + session.getEndDate());
+        Map<String, String> parameters = new HashMap<String, String>();
 
-		JSONResult jsonResult = _ajaxAccess.doGet(getOXAjaxAccessPath(), UtilConstants.ACTION_ALL, session, parameters);
-		DataObject[] result = buildResultArray(session, requestedFields, jsonResult);
-		return result;
-	}
+        parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
+        parameters.put(UtilConstants.START, String.valueOf(session.getStartDate()));
+        parameters.put(UtilConstants.END, String.valueOf(session.getEndDate()));
+        addExtraTimestampListParameters(parameters);
+        addObjectsLimitParameters(session, parameters);
+
+        JSONResult jsonResult = _ajaxAccess.doGet(getOXAjaxAccessPath(), UtilConstants.ACTION_ALL, session, parameters);
+        DataObject[] result = buildResultArray(session, requestedFields, jsonResult);
+        return result;
+    }
 
 	public DataObject[] getAllRecurrencesAsAppointments(Session session, BitSet requestedFields, DataObject seriesObject)
 			throws USMException {
@@ -166,52 +182,47 @@ public class AppointmentContentTypeTransferHandler extends AbstractTransferHandl
 				session.getEndDate());
 	}
 
-	public DataObject[] getAllRecurrencesAsAppointments(Session session, BitSet requestedFields,
-			DataObject seriesObject, long startDate, long endDate) throws USMException {
-		if (_journal.isDebugEnabled())
-			_journal.debug(session + " Getting all appointments as recurrences");
-		Map<String, String> parameters = new HashMap<String, String>();
+    public DataObject[] getAllRecurrencesAsAppointments(Session session, BitSet requestedFields, DataObject seriesObject, long startDate, long endDate) throws USMException {
+        if (_journal.isDebugEnabled())
+            _journal.debug(session + " Getting all appointments as recurrences");
+        Map<String, String> parameters = new HashMap<String, String>();
 
-		parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
-		parameters.put(UtilConstants.START, String.valueOf(startDate));
-		parameters.put(UtilConstants.END, String.valueOf(endDate));
-		String folder = seriesObject != null ? seriesObject.getParentFolderID() : null;
-		if (folder != null)
-			parameters.put(UtilConstants.FOLDER, folder);
-		parameters.put(UtilConstants.RECURRENCE_MASTER, Boolean.FALSE.toString());
-		
-		if(session.getObjectsLimit() > 0) {
-            addObjectsLimitParameters(session.getObjectsLimit(), parameters);
-        }
+        parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
+        parameters.put(UtilConstants.START, String.valueOf(startDate));
+        parameters.put(UtilConstants.END, String.valueOf(endDate));
+        String folder = seriesObject != null ? seriesObject.getParentFolderID() : null;
+        if (folder != null)
+            parameters.put(UtilConstants.FOLDER, folder);
+        parameters.put(UtilConstants.RECURRENCE_MASTER, Boolean.FALSE.toString());
+        addObjectsLimitParameters(session, parameters);
 
-		JSONResult jsonResult = _ajaxAccess.doGet(getOXAjaxAccessPath(), UtilConstants.ACTION_ALL, session, parameters);
-		JSONArray jArray = extractArrayResult(jsonResult);
-		List<DataObject> resultList = new ArrayList<DataObject>();
-		for (int i = 0; i < jArray.length(); i++) {
-			JSONArray columnsArray;
-			DataObject destination = null;
-			try {
-				columnsArray = jArray.getJSONArray(i);
-				destination = _contentType.newDataObject(session);
-				updateDataObjectFromJSONArray(destination, requestedFields, columnsArray);
-				readOptionalTimestamp(jsonResult.getJSONObject(), destination);
-				if (seriesObject != null) {
-					if (destination.getID().equals(seriesObject.getID())
-							|| seriesObject.getID().equals(
-									String.valueOf(destination.getFieldContent(CalendarConstants.RECURRENCE_ID))))
-						resultList.add(destination);
-				} else {
-					resultList.add(destination);
-				}
-			} catch (JSONException e) {
+        JSONResult jsonResult = _ajaxAccess.doGet(getOXAjaxAccessPath(), UtilConstants.ACTION_ALL, session, parameters);
+        JSONArray jArray = extractArrayResult(jsonResult);
+        List<DataObject> resultList = new ArrayList<DataObject>();
+        for (int i = 0; i < jArray.length(); i++) {
+            JSONArray columnsArray;
+            DataObject destination = null;
+            try {
+                columnsArray = jArray.getJSONArray(i);
+                destination = _contentType.newDataObject(session);
+                updateDataObjectFromJSONArray(destination, requestedFields, columnsArray);
+                readOptionalTimestamp(jsonResult.getJSONObject(), destination);
+                if (seriesObject != null) {
+                    if (destination.getID().equals(seriesObject.getID()) || seriesObject.getID().equals(
+                        String.valueOf(destination.getFieldContent(CalendarConstants.RECURRENCE_ID))))
+                        resultList.add(destination);
+                } else {
+                    resultList.add(destination);
+                }
+            } catch (JSONException e) {
 				throw generateException(USMContentTypesUtilErrorCodes.DATA_NOT_PRESENT_NUMBER1, destination,
 						"get new appointmets:data not present", e);
-			}
-		}
-		DataObject[] result = resultList.toArray(new DataObject[resultList.size()]);
-		resetRecurrenceTypeBasedFields(result);
-		return result;
-	}
+            }
+        }
+        DataObject[] result = resultList.toArray(new DataObject[resultList.size()]);
+        resetRecurrenceTypeBasedFields(result);
+        return result;
+    }
 	
     public DataObject[] getAllChangeExceptions(Session session, BitSet requestedFields, DataObject seriesObject) throws USMException {
         if (_journal.isDebugEnabled())
@@ -240,7 +251,7 @@ public class AppointmentContentTypeTransferHandler extends AbstractTransferHandl
                 resultList.add(destination);
             } catch (JSONException e) {
                 throw generateException(USMContentTypesUtilErrorCodes.DATA_NOT_PRESENT_NUMBER1, destination,
-                        "get new appointmets:data not present", e);
+                        "get new appointments:data not present", e);
             }
         }
         DataObject[] result = resultList.toArray(new DataObject[resultList.size()]);
@@ -352,7 +363,7 @@ public class AppointmentContentTypeTransferHandler extends AbstractTransferHandl
 		if (deleteExceptionSet.isEmpty())
 			return;
 		DataObject[] allOcurrences = getAllRecurrencesAsAppointments(seriesObject.getSession(),
-				DELETE_EXCEPTION_FIELDS, seriesObject, intervalStart, intervalEnd);
+				_DELETE_EXCEPTION_FIELDS, seriesObject, intervalStart, intervalEnd);
 		long timestamp = 0L;
 		for (int j = 0; j < allOcurrences.length; j++) {
 			DataObject occ = allOcurrences[j];
