@@ -49,13 +49,19 @@
 package com.openexchange.usm.contenttypes.contacts.impl;
 
 import static com.openexchange.usm.api.datatypes.DataTypes.*;
-
-import java.io.Serializable;
-import java.util.*;
-
-import com.openexchange.usm.api.contenttypes.*;
-import com.openexchange.usm.api.datatypes.*;
-import com.openexchange.usm.api.exceptions.*;
+import java.util.BitSet;
+import java.util.List;
+import java.util.Map;
+import com.openexchange.usm.api.contenttypes.ContactContentType;
+import com.openexchange.usm.api.contenttypes.ContentTypeField;
+import com.openexchange.usm.api.contenttypes.ContentTypeTransferHandler;
+import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
+import com.openexchange.usm.api.contenttypes.PictureConversionException;
+import com.openexchange.usm.api.contenttypes.UnsupportedContentOperationException;
+import com.openexchange.usm.api.datatypes.PIMAttachmentsDataType;
+import com.openexchange.usm.api.exceptions.AuthenticationFailedException;
+import com.openexchange.usm.api.exceptions.InternalUSMException;
+import com.openexchange.usm.api.exceptions.OXCommunicationException;
 import com.openexchange.usm.api.session.DataObject;
 import com.openexchange.usm.api.session.Session;
 import com.openexchange.usm.contenttypes.util.AbstractContentType;
@@ -262,26 +268,6 @@ public class ContactContentTypeImpl extends AbstractContentType implements Conta
 		return DefaultContentTypes.CONTACTS_CODE;
 	}
 
-	public long createNewAttachment(DataObject object, PIMAttachment attachment) throws USMException {
-		return _transferHandler.createNewAttachment(object, attachment);
-	}
-
-	public long deleteAttachments(DataObject object, PIMAttachment... attachmentsToDelete) throws USMException {
-		return _transferHandler.deleteAttachments(object, attachmentsToDelete);
-	}
-
-	public PIMAttachments getAllAttachments(DataObject object, String sort, String order) throws USMException {
-		return _transferHandler.getAllAttachments(object, sort, order);
-	}
-
-	public PIMAttachments getAllAttachments(DataObject object) throws USMException {
-		return _transferHandler.getAllAttachments(object);
-	}
-
-	public byte[] getAttachmentData(DataObject object, int attachmentId) throws USMException {
-		return _transferHandler.getAttachmentData(object, attachmentId);
-	}
-
 	public boolean supportsPIMAttachments() {
 		return true;
 	}
