@@ -467,4 +467,17 @@ public class ContactContentTypeTransferHandler extends AbstractTransferHandler {
                 result.toString());
         }
     }
+
+    /**
+     * Special behavior of contacts regarding sorting: If no limit and no sorting is specified, sort explicitly by ID to avoid complex
+     * automatic sorting within OX.
+     */
+    @Override
+    protected void addObjectsLimitParameters(Session session, Map<String, String> parameters) {
+        int limit = session.getObjectsLimit();
+        if (limit > 0)
+            addObjectsLimitParameters(limit, _LAST_MODIFIED_COLUMN, parameters); // sort by last modified
+        else if (!parameters.containsKey(UtilConstants.SORT))
+            parameters.put(UtilConstants.SORT, "1");
+    }
 }
