@@ -49,12 +49,14 @@
 package com.openexchange.usm.contenttypes.folder.impl;
 
 import static com.openexchange.usm.api.datatypes.DataTypes.*;
-
-import com.openexchange.usm.api.contenttypes.*;
-import com.openexchange.usm.api.datatypes.PIMAttachment;
-import com.openexchange.usm.api.datatypes.PIMAttachments;
-import com.openexchange.usm.api.exceptions.USMException;
-import com.openexchange.usm.api.session.*;
+import com.openexchange.usm.api.contenttypes.ContentTypeField;
+import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
+import com.openexchange.usm.api.contenttypes.FolderConstants;
+import com.openexchange.usm.api.contenttypes.FolderContentType;
+import com.openexchange.usm.api.contenttypes.FolderContentTypeTransferHandler;
+import com.openexchange.usm.api.session.DataObject;
+import com.openexchange.usm.api.session.Folder;
+import com.openexchange.usm.api.session.Session;
 import com.openexchange.usm.contenttypes.util.AbstractContentType;
 import com.openexchange.usm.contenttypes.util.UtilConstants;
 import com.openexchange.usm.datatypes.folder.FolderPermissionDataType;
@@ -155,31 +157,6 @@ public class FolderContentTypeImpl extends AbstractContentType implements Folder
 		return DefaultContentTypes.FOLDER_CODE;
 	}
 
-	public long createNewAttachment(DataObject object, PIMAttachment attachment) throws USMException {
-		// do nothing, attachments are not supported
-		return 0L;
-	}
-
-	public long deleteAttachments(DataObject object, PIMAttachment... attachmentsToDelete) throws USMException {
-		// do nothing, attachments are not supported
-		return 0L;
-	}
-
-	public PIMAttachments getAllAttachments(DataObject object, String sort, String order) throws USMException {
-		// do nothing, attachments are not supported
-		return null;
-	}
-
-	public PIMAttachments getAllAttachments(DataObject object) throws USMException {
-		// do nothing, attachments are not supported
-		return null;
-	}
-
-	public byte[] getAttachmentData(DataObject object, int attachmentId) throws USMException {
-		// do nothing, attachments are not supported
-		return null;
-	}
-
 	public boolean supportsPIMAttachments() {
 		return false;
 	}
