@@ -51,9 +51,12 @@ package com.openexchange.usm.contenttypes.folder.impl;
 
 import java.util.BitSet;
 import org.apache.commons.logging.Log;
-import com.openexchange.usm.api.contenttypes.OXFolderContent;
 import com.openexchange.usm.api.contenttypes.FolderContentType;
 import com.openexchange.usm.api.contenttypes.FolderContentTypeTransferHandler;
+import com.openexchange.usm.api.contenttypes.OXFolderContent;
+import com.openexchange.usm.api.contenttypes.ResourceInputStream;
+import com.openexchange.usm.api.datatypes.PIMAttachment;
+import com.openexchange.usm.api.datatypes.PIMAttachments;
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.session.DataObject;
 import com.openexchange.usm.api.session.Folder;
@@ -132,6 +135,26 @@ public class FolderContentTypeTransferHandlerDelegator implements FolderContentT
         getTransferHandlerImplementation(object).writeUpdatedDataObject(object, timestamp);
     }
 
+    public PIMAttachments getAllAttachments(DataObject object) throws USMException {
+        return getTransferHandlerImplementation(object).getAllAttachments(object);
+    }
+
+    public long createNewAttachment(DataObject object, PIMAttachment attachment) throws USMException {
+        return getTransferHandlerImplementation(object).createNewAttachment(object, attachment);
+    }
+
+    public long deleteAttachments(DataObject object, PIMAttachment... attachmentsToDelete) throws USMException {
+        return getTransferHandlerImplementation(object).deleteAttachments(object, attachmentsToDelete);
+    }
+
+    public byte[] getAttachmentData(DataObject object, int attachmentId) throws USMException {
+        return getTransferHandlerImplementation(object).getAttachmentData(object, attachmentId);
+    }
+
+    public ResourceInputStream getAttachmentDataStream(DataObject object, int attachmentId, long offset, long length) throws USMException {
+        return getTransferHandlerImplementation(object).getAttachmentDataStream(object, attachmentId, offset, length);
+    }
+
     private FolderContentTypeTransferHandler getTransferHandlerImplementation(DataObject object) {
         return getTransferHandlerImplementation(object.getSession());
     }
@@ -143,5 +166,4 @@ public class FolderContentTypeTransferHandlerDelegator implements FolderContentT
         }
         return _folderContentTypeTransferHandlerImpl;
     }
-
 }
