@@ -87,10 +87,6 @@ import com.openexchange.usm.ox_json.OXJSONAccess;
  */
 public class FolderContentTypeTransferHandlerImpl extends AbstractTransferHandler implements FolderContentTypeTransferHandler {
 
-    private static final String ONE = "1";
-
-    private static final String IGNORE_TRANSLATION = "ignoreTranslation";
-
     private static final int FOLDER_ID_LENGTH_LIMIT = 255;
 
     private static final String ALLOWED_MODULES = "allowed_modules";
@@ -283,7 +279,6 @@ public class FolderContentTypeTransferHandlerImpl extends AbstractTransferHandle
         Map<String, String> parameters = new HashMap<String, String>();
         addTreeParameter(session, parameters);
         parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
-        parameters.put(IGNORE_TRANSLATION, ONE);
         return parameters;
     }
 
