@@ -62,13 +62,16 @@ import org.json.JSONObject;
 import com.openexchange.usm.api.contenttypes.ContentType;
 import com.openexchange.usm.api.contenttypes.ContentTypeField;
 import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
-import com.openexchange.usm.api.contenttypes.OXFolderContent;
 import com.openexchange.usm.api.contenttypes.MailContentType;
+import com.openexchange.usm.api.contenttypes.OXFolderContent;
+import com.openexchange.usm.api.contenttypes.ResourceInputStream;
 import com.openexchange.usm.api.contenttypes.UnsupportedContentOperationException;
+import com.openexchange.usm.api.exceptions.InternalUSMException;
 import com.openexchange.usm.api.exceptions.OXCommunicationException;
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.exceptions.USMIllegalArgumentException;
 import com.openexchange.usm.api.exceptions.USMStartupException;
+import com.openexchange.usm.api.io.InputStreamProvider;
 import com.openexchange.usm.api.session.DataObject;
 import com.openexchange.usm.api.session.Folder;
 import com.openexchange.usm.api.session.Session;
@@ -76,10 +79,12 @@ import com.openexchange.usm.api.session.SyncResult;
 import com.openexchange.usm.contenttypes.util.AbstractTransferHandler;
 import com.openexchange.usm.contenttypes.util.USMContentTypesUtilErrorCodes;
 import com.openexchange.usm.contenttypes.util.UtilConstants;
+import com.openexchange.usm.datatypes.mail.Vacation;
 import com.openexchange.usm.ox_json.JSONResult;
 import com.openexchange.usm.ox_json.JSONResultType;
 import com.openexchange.usm.ox_json.OXJSONAccess;
 import com.openexchange.usm.ox_json.OXResource;
+import com.openexchange.usm.util.Toolkit;
 
 public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler {
 
@@ -115,6 +120,10 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 
     private static final String SAVE = "save";
 
+    private static final String FLAG = "flag";
+
+    private static final String MAILFILTER_AJAX_PATH = "mailfilter";
+
     private final BitSet _getAllBitSet = new BitSet();
 
     private final int _idIndex;
@@ -147,26 +156,49 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
         checkResult(result, JSONResultType.JSONObject);
     }
 
+    public void sendMail(InputStreamProvider streamProvider, Session session) throws USMException {
+        Map<String, String> parameters = new HashMap<String, String>();
+        parameters.put(SRC, ONE);
+        JSONResult result = _ajaxAccess.doPut(UtilConstants.MAIL_PATH, UtilConstants.ACTION_NEW, session, parameters, streamProvider);
+        checkResult(result, JSONResultType.JSONObject);
+    }
+
     public Object getMailAttachment(Session session, String folderId, String mailId, String attachmentId) throws USMException {
+        Map<String, String> parameters = createMailAttachmentParameters(folderId, mailId, attachmentId);
+        OXResource result = _ajaxAccess.getResource(session, UtilConstants.MAIL_PATH, parameters);
+        return result;
+    }
+
+    public ResourceInputStream getMailAttachmentAsStream(Session session, String folderId, String mailId, String attachmentId) throws USMException {
+        Map<String, String> parameters = createMailAttachmentParameters(folderId, mailId, attachmentId);
+        ResourceInputStream result = _ajaxAccess.getResourceAsStream(session, UtilConstants.MAIL_PATH, parameters);
+        return result;
+    }
+
+    private static Map<String, String> createMailAttachmentParameters(String folderId, String mailId, String attachmentId) {
         Map<String, String> parameters = new HashMap<String, String>();
         parameters.put(UtilConstants.FOLDER, folderId);
         parameters.put(UtilConstants.ID, mailId);
         parameters.put(MailConstants.ATTACHMENT, attachmentId);
         parameters.put(SAVE, ONE);
         parameters.put(UtilConstants.ACTION, MailConstants.ATTACHMENT);
-
-        OXResource result = _ajaxAccess.getResource(session, UtilConstants.MAIL_PATH, parameters);
-        return result;
+        return parameters;
     }
 
-    public String createNewMail(String folderId, int flag, String mailMIMEdataBlock, Session session) throws USMException {
+    public String createNewMail(String folderId, int flags, String mailMIMEdataBlock, Session session) throws USMException {
         // TODO: use import instead of new (see svn history!)
-        Map<String, String> parameters = new HashMap<String, String>();
-        parameters.put(SRC, ONE);
-        parameters.put(UtilConstants.FOLDER, folderId);
-        parameters.put(FLAGS, String.valueOf(flag));
-        parameters.put(FORCE, ONE);
+        Map<String, String> parameters = buildNewMailParameters(folderId, flags);
         JSONResult result = _ajaxAccess.doPut(UtilConstants.MAIL_PATH, UtilConstants.ACTION_NEW, session, parameters, mailMIMEdataBlock);
+        return verifyNewMailResult(result);
+    }
+    
+    public String createNewMailFromStream(String folderId, int flags, InputStreamProvider streamProvider, Session session) throws USMException {
+        Map<String, String> parameters = buildNewMailParameters(folderId, flags);
+        JSONResult result = _ajaxAccess.doPut(UtilConstants.MAIL_PATH, UtilConstants.ACTION_NEW, session, parameters, streamProvider);
+        return verifyNewMailResult(result);
+    }
+
+    private String verifyNewMailResult(JSONResult result) throws OXCommunicationException, InternalUSMException {
         checkResult(result, JSONResultType.JSONObject);
         JSONObject o = result.getJSONObject();
         try {
@@ -180,6 +212,15 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
         }
     }
 
+    private static Map<String, String> buildNewMailParameters(String folderId, int flag) {
+        Map<String, String> parameters = new HashMap<String, String>();
+        parameters.put(SRC, ONE);
+        parameters.put(UtilConstants.FOLDER, folderId);
+        parameters.put(FLAGS, String.valueOf(flag));
+        parameters.put(FORCE, ONE);
+        return parameters;
+    }
+
     public DataObject getReplyForward(Session session, String messageID, String folderID, String action, String view, boolean dropPrefix) throws USMException {
         DataObject mail = _contentType.newDataObject(session);
         mail.setID(messageID);
@@ -347,7 +388,9 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
         readDataObject(object, requestedFields, parameters);
     }
 
-    public JSONObject readStructuredMail(DataObject object, BitSet requestedFields) throws USMException {
+    public JSONObject readStructuredMail(DataObject object, int maxAttachmentSize) throws USMException {
+        if (object == null || Toolkit.notProvided(object.getID()) || Toolkit.notProvided(object.getParentFolderID()))
+            return new JSONObject();
         if (_journal.isDebugEnabled())
             _journal.debug(object.getSession() + " Read structured mail with id " + object.getID());
         checkTypeToWrite(object, _contentType.getID());
@@ -355,6 +398,8 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
         parameters.put(UNSEEN, ONE);
         parameters.put(UtilConstants.ID, object.getID());
         parameters.put(UtilConstants.FOLDER, object.getParentFolderID());
+        if (maxAttachmentSize >= 0)
+            parameters.put(MailConstants.MAX_SIZE, Integer.toString(maxAttachmentSize));
         JSONResult result = _ajaxAccess.doGet(getOXAjaxAccessPath(), UtilConstants.ACTION_GET_STRUCTURE, object.getSession(), parameters);
         checkResult(result, JSONResultType.JSONObject);
         JSONObject resultObject = null;
@@ -638,6 +683,137 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
         return conversionObject;
     }
 
+    public List<DataObject> searchMails(JSONObject searchCriteria, String folderId, Session session, BitSet requestedFields) throws USMException {
+        Map<String, String> parameters = new HashMap<String, String>();
+        parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
+        parameters.put(UtilConstants.FOLDER, folderId);
+        JSONResult jResult = _ajaxAccess.doPut(getOXAjaxAccessPath(), UtilConstants.ACTION_ALL, session, parameters, searchCriteria);
+        return buildSearchResultList(session, requestedFields, jResult);
+    }
+
+    public List<DataObject> searchMails(String searchText, String folderId, Session session, BitSet requestedFields) throws USMException {
+        Map<String, String> parameters = new HashMap<String, String>();
+        parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
+        parameters.put(UtilConstants.FOLDER, folderId);
+        JSONObject searchCriteria = new JSONObject();
+        JSONArray searchArray = new JSONArray();
+        try {
+            searchCriteria.put("col", -1);
+            searchCriteria.put("pattern", searchText);
+            searchArray.put(searchCriteria);
+        } catch (JSONException ignored) {
+            // can not happen
+        }
+        JSONResult jResult = _ajaxAccess.doPut(getOXAjaxAccessPath(), UtilConstants.ACTION_SEARCH, session, parameters, searchArray);
+        return buildSearchResultList(session, requestedFields, jResult);
+    }
+
+    protected List<DataObject> buildSearchResultList(Session session, BitSet requestedFields, JSONResult jResult) throws OXCommunicationException, InternalUSMException {
+        JSONArray jsonar;
+        try {
+            checkResult(jResult, JSONResultType.JSONObject);
+            jsonar = jResult.getJSONObject().getJSONArray(UtilConstants.RESULT_DATA);
+        } catch (JSONException e1) {
+            _journal.error(session + " Can not read the result data array for search mails", e1);
+            return Collections.emptyList();
+        }
+        int length = jsonar.length();
+        if (length == 0)
+            return Collections.emptyList();
+        List<DataObject> resultMails = new ArrayList<DataObject>();
+        for (int j = 0; j < length; j++) {
+            JSONArray columnsArray;
+            try {
+                columnsArray = jsonar.getJSONArray(j);
+                DataObject newDataObject = _contentType.newDataObject(session);
+                updateDataObjectFromJSONArray(newDataObject, requestedFields, columnsArray);
+                resultMails.add(newDataObject);
+            } catch (JSONException e) {
+                _journal.error(session + " Can not read the result data for search mails", e);
+            }
+        }
+        return resultMails;
+    }
+
+    public JSONObject getVacationRule(Session session) throws USMException {
+        // GET /ajax/mailfilter?action=list&flag=vacation&session=4108cc5c4bee4d309e10b013ad280a90 HTTP/1.1\r\n
+        // RESULT:
+        // {"data":[{"active":true,"id":0,"test":{"id":"true"},"rulename":"Abwesenheit","position":0,"actioncmds":[{"id":"vacation","days":"7","text":"This is a test Out Of Office email.","addresses":["user5@ox-p5.microdoc.com","user5@ox.
+        Map<String, String> parameters = new HashMap<String, String>();
+        parameters.put(FLAG, Vacation.RULE_ID);
+        JSONResult jResult = _ajaxAccess.doGet(MAILFILTER_AJAX_PATH, UtilConstants.ACTION_LIST, session, parameters);
+        checkResult(jResult, JSONResultType.JSONObject);
+        try {
+            JSONArray jsonar = jResult.getJSONObject().getJSONArray(UtilConstants.RESULT_DATA);
+            Vacation vacation = new Vacation(jsonar.getJSONObject(0));
+            return vacation.toJSONObject();
+        } catch (JSONException e) {
+            _journal.error(session + " Can not read the vacation rule", e);
+        }
+        return null;
+    }
+
+    public void setVacationRule(Session session, JSONObject rule) throws USMException {
+        // PUT /ajax/mailfilter?action=new&session=ae231c985599470bb7d55bc393ba3779 HTTP/1.1\r\n
+        // {"rulename":"Abwesenheit","active":true,"flags":["vacation"],"test":{"id":"true"},"actioncmds":[{"id":"vacation","days":10,"addresses":["user4@ox-p5.microdoc.com"],"text":"This is a test","subject":"Test Out of Office"}]}
+        // RESULT : {"data":0}
+        Map<String, String> parameters = new HashMap<String, String>();
+        JSONObject requestBody = createVacationRequestBody(rule);
+        JSONResult jResult = _ajaxAccess.doPut(MAILFILTER_AJAX_PATH, UtilConstants.ACTION_NEW, session, parameters, requestBody);
+        checkResult(jResult, JSONResultType.JSONObject);
+    }
+
+    public void updateVacationRule(Session session, JSONObject rule) throws USMException {
+        // PUT /ajax/mailfilter?action=update&session=ae231c985599470bb7d55bc393ba3779 HTTP/1.1\r\n
+        // [truncated]
+        // {"rulename":"Abwesenheit","active":true,"flags":["vacation"],"test":{"id":"true"},"actioncmds":[{"id":"vacation","days":10,"addresses":["user4@ox-p5.microdoc.com","user4@ox.microdoc.de"],"text":"This is a test","subject":"Test
+        // RESULT: {"data":null}
+        Map<String, String> parameters = new HashMap<String, String>();
+        JSONObject requestBody = createVacationRequestBody(rule);
+        JSONResult jResult = _ajaxAccess.doPut(MAILFILTER_AJAX_PATH, UtilConstants.ACTION_UPDATE, session, parameters, requestBody);
+        checkResult(jResult, JSONResultType.JSONObject);
+    }
+
+    public void deleteVacationRule(Session session, JSONObject rule) throws USMException {
+        // PUT /ajax/mailfilter?action=delete
+        // Request body: An object with the field id.
+
+        Map<String, String> parameters = new HashMap<String, String>();
+        JSONObject requestBody = new JSONObject();
+        try {
+            requestBody.put(Vacation.ID, (new Vacation(rule).getId()));
+            JSONResult jResult = _ajaxAccess.doPut(MAILFILTER_AJAX_PATH, UtilConstants.ACTION_DELETE, session, parameters, requestBody);
+            checkResult(jResult, JSONResultType.JSONObject);
+        } catch (JSONException e) {
+            throw generateException(
+                USMContentTypesMailErrorCodes.ERROR_ON_VACATION_REQUEST_CREATE_1,
+                null,
+                "delete Vacation Rule: can not create request Body",
+                e);
+        }
+    }
+
+    private JSONObject createVacationRequestBody(JSONObject rule) throws OXCommunicationException {
+        Vacation vacation = new Vacation(rule);
+        JSONObject requestBody = new JSONObject();
+        try {
+            requestBody.put("rulename", Vacation.RULENAME);
+            requestBody.put(Vacation.ACTIVE, vacation.isActive());
+            requestBody.put("flags", (new JSONArray()).put(Vacation.RULE_ID));
+            requestBody.put("id", vacation.getId());
+            requestBody.put("test", new JSONObject("{\"id\":\"true\"}"));
+            requestBody.put("actioncmds", (new JSONArray()).put(vacation.toActionCmdObject()));
+        } catch (JSONException e) {
+            throw generateException(
+                USMContentTypesMailErrorCodes.ERROR_ON_VACATION_REQUEST_CREATE_2,
+                null,
+                "Vacation Rule: can not create request Body",
+                e);
+        }
+
+        return requestBody;
+    }
+
     @Override
     protected void addObjectsLimitParameters(Session session, Map<String, String> parameters) {
         int mailLimit = session.getMailLimit();
