@@ -48,6 +48,7 @@
  */
 package com.openexchange.usm.datatypes.mail;
 
+import java.io.Serializable;
 import org.json.*;
 
 import com.openexchange.usm.api.datatypes.AbstractDataType;
@@ -81,4 +82,9 @@ public class NestedMailDataType extends AbstractDataType<NestedMail> {
 	public void storeInJSONObject(Session session, JSONObject parent, String field, Object value) throws JSONException {
 		parent.put(field, ((NestedMail) value).toJSONObject(session));
 	}
+	
+	@Override
+	public Serializable serialize(Object value) {
+	    return null;
+	}
 }
