@@ -49,14 +49,10 @@
 package com.openexchange.usm.contenttypes.resources.impl;
 
 import static com.openexchange.usm.api.datatypes.DataTypes.*;
-
 import com.openexchange.usm.api.contenttypes.ContentTypeField;
 import com.openexchange.usm.api.contenttypes.ContentTypeTransferHandler;
 import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
 import com.openexchange.usm.api.contenttypes.ResourceContentType;
-import com.openexchange.usm.api.datatypes.PIMAttachment;
-import com.openexchange.usm.api.datatypes.PIMAttachments;
-import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.session.DataObject;
 import com.openexchange.usm.contenttypes.util.AbstractContentType;
 import com.openexchange.usm.contenttypes.util.UtilConstants;
@@ -75,26 +71,6 @@ public class ResourcesContentTypeImpl extends AbstractContentType implements Res
 
 	private ResourcesContentTypeTransferHandler _transferHandler;
 
-	public long createNewAttachment(DataObject object, PIMAttachment attachment) throws USMException {
-		return 0;
-	}
-
-	public long deleteAttachments(DataObject object, PIMAttachment... attachmentsToDelete) throws USMException {
-		return 0;
-	}
-
-	public PIMAttachments getAllAttachments(DataObject object, String sort, String order) throws USMException {
-		return null;
-	}
-
-	public PIMAttachments getAllAttachments(DataObject object) throws USMException {
-		return null;
-	}
-
-	public byte[] getAttachmentData(DataObject object, int attachmentId) throws USMException {
-		return null;
-	}
-
 	public int getCode() {
 		return DefaultContentTypes.RESOURCES_CODE;
 	}
