@@ -105,6 +105,11 @@ public class ResourcesContentTypeTransferHandler extends AbstractTransferHandler
 				"Updating a folder content is not supported for resources");
 	}
 
+    @Override
+    public OXFolderContent readFolderContent(Folder folder, int limit) throws USMException {
+        return new OXFolderContent(true, readFolderContent(folder, folder.getSession().getFieldFilter(folder.getElementsContentType())));
+    }
+
 	@Override
 	public DataObject[] readFolderContent(Folder folder, BitSet requestedFields) throws USMException {
 		Session session = folder.getSession();
