@@ -150,7 +150,7 @@ public abstract class AbstractContentType implements ContentType {
 		int i = 0;
 		try {
 			for (; i < limit; i++)
-				fieldContent[i] = fields[i].getFieldType().createCopy(data[i + offset]);
+				fieldContent[i] = fields[i].getFieldType().deserialize(data[i + offset]);
 		} catch (Exception e) {
 			throw new DeserializationFailedException(
 					USMContentTypesUtilErrorCodes.DESERIALIZATION_FAILED_INVALID_FIELD_CONTENT,
