@@ -64,6 +64,7 @@ import com.openexchange.usm.api.contenttypes.ContentTypeField;
 import com.openexchange.usm.api.contenttypes.ContentTypeTransferHandler;
 import com.openexchange.usm.api.contenttypes.FolderContentType;
 import com.openexchange.usm.api.contenttypes.OXFolderContent;
+import com.openexchange.usm.api.contenttypes.ResourceInputStream;
 import com.openexchange.usm.api.contenttypes.UnsupportedContentOperationException;
 import com.openexchange.usm.api.datatypes.DataType;
 import com.openexchange.usm.api.datatypes.PIMAttachment;
@@ -184,7 +185,7 @@ public abstract class AbstractTransferHandler implements ContentTypeTransferHand
         try {
             requestBody.put("confirmation", response);
             requestBody.put("confirmmessage", confirmMessage);
-            requestBody.put("id", computeFolderOwnerID(object));
+            requestBody.put(UtilConstants.ID, computeFolderOwnerID(object));
         } catch (JSONException e) {
 			throw generateException(USMContentTypesUtilErrorCodes.CONFIRM_ERROR_NUMBER1, object,
 					"can not create request body for confirming appoinment or task", e);
@@ -880,11 +881,8 @@ public abstract class AbstractTransferHandler implements ContentTypeTransferHand
             if (recurrenceType <= 2) {
                 data.setFieldContent("day_in_month", null);
             }
-
-            if (recurrenceType == 4) {
-                if (recurrenceType > 0)
-                    data.setFieldContent("interval", 1);
-            } else {
+            if (recurrenceType != 4) {
+                // Do not explicitly set interval to 1 for yearly series since other intervals are now also allowed
                 data.setFieldContent("month", null);
             }
         }
@@ -920,25 +918,35 @@ public abstract class AbstractTransferHandler implements ContentTypeTransferHand
     protected abstract String getOXAjaxAccessPath();
 
     public long createNewAttachment(DataObject object, PIMAttachment attachment) throws USMException {
+        if (!_contentType.supportsPIMAttachments())
+            return 0L;
         return _pimAttachmentsTransferHandler.createNewAttachment(object, attachment);
     }
 
-    public PIMAttachments getAllAttachments(DataObject object, String sort, String order) throws USMException {
-        return _pimAttachmentsTransferHandler.getAllAttachments(object, sort, order);
-    }
-
     public PIMAttachments getAllAttachments(DataObject object) throws USMException {
+        if (!_contentType.supportsPIMAttachments())
+            return null;
         return _pimAttachmentsTransferHandler.getAllAttachments(object);
     }
 
-    public long deleteAttachments(DataObject object, PIMAttachment[] attachmentsToDelete) throws USMException {
+    public long deleteAttachments(DataObject object, PIMAttachment... attachmentsToDelete) throws USMException {
+        if (!_contentType.supportsPIMAttachments())
+            return 0L;
         return _pimAttachmentsTransferHandler.deleteAttachments(object, attachmentsToDelete);
     }
 
     public byte[] getAttachmentData(DataObject object, int attachmentId) throws USMException {
+        if (!_contentType.supportsPIMAttachments())
+            return null;
         return _pimAttachmentsTransferHandler.getAttachmentData(object, attachmentId);
     }
 
+    public ResourceInputStream getAttachmentDataStream(DataObject object, int attachmentId, long offset, long length) throws USMException {
+        if (!_contentType.supportsPIMAttachments())
+            return null;
+        return _pimAttachmentsTransferHandler.getAttachmentDataStream(object, attachmentId, offset, length);
+    }
+
     protected void addObjectsLimitParameters(Session session, Map<String, String> parameters) {
         int limit = session.getObjectsLimit();
         if (limit > 0)
