@@ -73,10 +73,10 @@ public class TableDefinition {
 	// SQL statements to create tables 
 	final static String[] SQL_STATEMENTS = {
 			"CREATE TABLE USMSession (cid INT4 UNSIGNED NOT NULL, id INT4 UNSIGNED NOT NULL, device VARCHAR(64) NOT NULL, protocol VARCHAR(32) NOT NULL, usmSessionId INT4 UNSIGNED NOT NULL, PRIMARY KEY (cid, usmSessionId), UNIQUE (cid, id, device, protocol)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin",
-			"CREATE TABLE usmIdMapping(cid INT4 UNSIGNED NOT NULL, usmSessionId INT4 UNSIGNED NOT NULL, longId VARCHAR(255) NOT NULL, shortId INT4 UNSIGNED NOT NULL, UNIQUE usmIdMappingUIdx (cid, usmSessionId, shortId), FOREIGN KEY(cid, usmSessionId) REFERENCES USMSession(cid, usmSessionId)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin",
-			"CREATE TABLE easUIDMapping(cid INT4 UNSIGNED NOT NULL, usmSessionId INT4 UNSIGNED NOT NULL, folderId VARCHAR(64) NOT NULL, longId VARCHAR(255) NOT NULL, shortId INT4 UNSIGNED NOT NULL, UNIQUE easUIDMappingUIDX (cid, usmSessionId, folderId, shortId), FOREIGN KEY(cid, usmSessionId) REFERENCES USMSession(cid, usmSessionId)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin",
-			"CREATE TABLE USMDataStorage (cid INT4 UNSIGNED NOT NULL, usmSessionId INT4 UNSIGNED NOT NULL, ObjectID VARCHAR(64) NOT NULL, SyncKey INT8 NOT NULL, Element LONGTEXT NOT NULL, UNIQUE USMDataStorageUIDX (cid, usmSessionId, objectId, syncKey), FOREIGN KEY(cid, usmSessionId) REFERENCES USMSession(cid, usmSessionId)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin",
-			"CREATE TABLE USMSessionFields (cid INT4 UNSIGNED NOT NULL, usmSessionId INT4 UNSIGNED NOT NULL, field VARCHAR(20) NOT NULL, value VARCHAR(32) NOT NULL, UNIQUE (cid, usmSessionId, field), FOREIGN KEY(cid, usmSessionId) REFERENCES USMSession(cid, usmSessionId)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin" };
+			"CREATE TABLE usmIdMapping(cid INT4 UNSIGNED NOT NULL, usmSessionId INT4 UNSIGNED NOT NULL, longId VARCHAR(255) NOT NULL, shortId INT4 UNSIGNED NOT NULL, PRIMARY KEY (cid, usmSessionId, shortId), FOREIGN KEY(cid, usmSessionId) REFERENCES USMSession(cid, usmSessionId)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin",
+			"CREATE TABLE easUIDMapping(cid INT4 UNSIGNED NOT NULL, usmSessionId INT4 UNSIGNED NOT NULL, folderId VARCHAR(64) NOT NULL, longId VARCHAR(255) NOT NULL, shortId INT4 UNSIGNED NOT NULL, PRIMARY KEY (cid, usmSessionId, folderId, shortId), FOREIGN KEY(cid, usmSessionId) REFERENCES USMSession(cid, usmSessionId)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin",
+			"CREATE TABLE USMDataStorage (cid INT4 UNSIGNED NOT NULL, usmSessionId INT4 UNSIGNED NOT NULL, ObjectID VARCHAR(64) NOT NULL, SyncKey INT8 NOT NULL, Element LONGTEXT NOT NULL, PRIMARY KEY (cid, usmSessionId, objectId, syncKey), FOREIGN KEY(cid, usmSessionId) REFERENCES USMSession(cid, usmSessionId)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin",
+			"CREATE TABLE USMSessionFields (cid INT4 UNSIGNED NOT NULL, usmSessionId INT4 UNSIGNED NOT NULL, field VARCHAR(20) NOT NULL, value VARCHAR(32) NOT NULL, PRIMARY KEY (cid, usmSessionId, field), FOREIGN KEY(cid, usmSessionId) REFERENCES USMSession(cid, usmSessionId)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin" };
 
 	final static String[] UUID_SQL_STATEMENTS = {
 			"CREATE TABLE UUIDHeader(cid INT4 UNSIGNED NOT NULL, header VARCHAR(36) NOT NULL, PRIMARY KEY (header)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin",
