@@ -70,7 +70,7 @@ public class ChangeCollationUpdateTask extends USMBaseUpdateTaskV2 {
 		Connection con = null;
 		Statement stmt = null;
 		try {
-			con = _provider.getDatabaseService().getWritable(contextId);
+			con = _provider.getDatabaseService().getForUpdateTask(contextId);
 			con.setAutoCommit(false);
 			stmt = con.createStatement();
 			for (String table : TableDefinition.TABLE_NAMES)
