@@ -48,51 +48,43 @@
  */
 package com.openexchange.usm.connector.commands;
 
-import static com.openexchange.usm.connector.commands.CommandConstants.AVAILABLE_CONTENT_TYPES;
-import static com.openexchange.usm.connector.commands.CommandConstants.CLIENT_FIELD_PREFIX;
-import static com.openexchange.usm.connector.commands.CommandConstants.CONFLICT_RESOLUTION;
-import static com.openexchange.usm.connector.commands.CommandConstants.CONTEXT_ID;
-import static com.openexchange.usm.connector.commands.CommandConstants.CONTEXT_UUID;
-import static com.openexchange.usm.connector.commands.CommandConstants.CUSTOM_PROPERTIES;
-import static com.openexchange.usm.connector.commands.CommandConstants.DEVICE;
-import static com.openexchange.usm.connector.commands.CommandConstants.END_DATE;
-import static com.openexchange.usm.connector.commands.CommandConstants.FOLDER_TREE;
-import static com.openexchange.usm.connector.commands.CommandConstants.START_DATE;
-import static com.openexchange.usm.connector.commands.CommandConstants.SYNCID;
-import static com.openexchange.usm.connector.commands.CommandConstants.SYNC_CONTENT_TYPES;
-import static com.openexchange.usm.connector.commands.CommandConstants.TIME_ZONE;
-import static com.openexchange.usm.connector.commands.CommandConstants.USER;
-import static com.openexchange.usm.connector.commands.CommandConstants.USERID;
-
+import static com.openexchange.usm.connector.commands.CommandConstants.*;
+import java.io.BufferedReader;
+import java.io.IOException;
+import java.io.InputStreamReader;
+import java.io.UnsupportedEncodingException;
 import java.util.BitSet;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.UUID;
-
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpSession;
-
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
-
 import com.openexchange.usm.api.contenttypes.ContentType;
 import com.openexchange.usm.api.contenttypes.ContentTypeField;
 import com.openexchange.usm.api.contenttypes.ContentTypeManager;
 import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
 import com.openexchange.usm.api.contenttypes.FolderContentType;
+import com.openexchange.usm.api.exceptions.USMAccessDeniedException;
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.session.ChangeState;
 import com.openexchange.usm.api.session.ConflictResolution;
 import com.openexchange.usm.api.session.DataObject;
 import com.openexchange.usm.api.session.Folder;
 import com.openexchange.usm.api.session.Session;
+import com.openexchange.usm.connector.exceptions.DataObjectNotFoundException;
+import com.openexchange.usm.connector.exceptions.InvalidUUIDException;
+import com.openexchange.usm.connector.exceptions.MultipleOperationsOnDataObjectException;
 import com.openexchange.usm.json.ConnectorBundleErrorCodes;
 import com.openexchange.usm.json.JSONSessionInitializer;
+import com.openexchange.usm.json.ServletConstants;
 import com.openexchange.usm.json.USMJSONAPIException;
 import com.openexchange.usm.json.USMJSONAccessDeniedException;
 import com.openexchange.usm.json.USMJSONServlet;
+import com.openexchange.usm.json.USMSessionCredentials;
 import com.openexchange.usm.json.response.ResponseObject;
 import com.openexchange.usm.json.response.ResponseStatusCode;
 import com.openexchange.usm.mimemail.ExternalMailContentTypeFields;
@@ -100,6 +92,8 @@ import com.openexchange.usm.ox_json.OXJSONPropertyNames;
 import com.openexchange.usm.session.dataobject.DataObjectSet;
 import com.openexchange.usm.util.BitSetEncoder;
 import com.openexchange.usm.util.JSONToolkit;
+import com.openexchange.usm.util.TempFileStorage;
+import com.openexchange.usm.util.Toolkit;
 
 /**
  * General Handler for all  USM-JSON-Commands. 
@@ -109,33 +103,112 @@ import com.openexchange.usm.util.JSONToolkit;
  */
 public abstract class BaseCommandHandler implements CommandHandler {
 
-	private static final String DEFAULT_ADDRESS = "defaultAddress";
+	private static final String HOST = "host";
+	private static final String CUSTOM_HOST_HEADER = "X-Host";
+    private static final String DEFAULT_ADDRESS = "defaultAddress";
+    private static final String MAX_INLINE_ATTACHMENT_SIZE = "max_inline_attachment_size";
+    private static final String TEMP_FILES_TIMEOUT = "temp_files_timeout";
+
+    private static JSONObject extractJSONObjectFromRequest(HttpServletRequest request) throws USMJSONAPIException {
+        // read the JSON object from the request
+        BufferedReader reader = null;
+        try {
+            reader = new BufferedReader(new InputStreamReader(request.getInputStream(), ServletConstants.UTF_ENCODING_8));
+            char[] buffer = new char[4096];
+            StringBuilder sb = new StringBuilder(4096);
+            for (int count = reader.read(buffer); count > 0; count = reader.read(buffer)) {
+                sb.append(buffer, 0, count);
+            }
+            JSONObject requestObject = new JSONObject(sb.toString());
+            return requestObject;
+        } catch (UnsupportedEncodingException e) {
+            throw new USMJSONAPIException(
+                ConnectorBundleErrorCodes.UTF_8_NOT_SUPPORTED,
+                ResponseStatusCode.INTERNAL_ERROR,
+                "UTF-8 Encoding is not supported");
+        } catch (IOException e) {
+            throw new USMJSONAPIException(
+                ConnectorBundleErrorCodes.ERROR_ON_READ_REQUEST,
+                ResponseStatusCode.BAD_REQUEST,
+                "Error while reading request",
+                e);
+        } catch (JSONException e) {
+            throw new USMJSONAPIException(
+                ConnectorBundleErrorCodes.BAD_JSONOBJECT_IN_REQUEST,
+                ResponseStatusCode.BAD_REQUEST,
+                "Invalid JSONObject as parameter",
+                e);
+        } finally {
+            Toolkit.close(reader);
+        }
+    }
+
+    public static Session retrieveUSMSession(USMJSONServlet servlet, HttpServletRequest request, USMSessionCredentials creds) throws USMJSONAPIException {
+        return retrieveUSMSession(servlet, request, creds.getUser(), creds.getPassword(), creds.getDevice());
+    }
+
+    public static Session retrieveUSMSession(USMJSONServlet servlet, HttpServletRequest request, String user, String password, String device) throws USMJSONAPIException {
+        try {
+            return servlet.getSession(user, password, device, request.getRemoteAddr(), getXHeadersFromRequest(request));
+        } catch (USMAccessDeniedException e) {
+            throw new USMJSONAPIException(
+                ConnectorBundleErrorCodes.COMMAND_ACCESS_DENIED,
+                ResponseStatusCode.ACCESS_DENIED,
+                e.getErrorMessage());
+        } catch (USMException e) {
+            throw USMJSONAPIException.createInternalError(ConnectorBundleErrorCodes.COMMAND_INTERNAL_ERROR, e);
+        }
+    }
+
+    public static Map<String, String> getXHeadersFromRequest(HttpServletRequest request) {
+        Map<String, String> headers = new HashMap<String, String>();
+        Enumeration<?> headerNames = request.getHeaderNames();
+        while (headerNames.hasMoreElements()) {
+            String headerName = (String) headerNames.nextElement();
+            if (headerName.toLowerCase().startsWith("x-"))
+                headers.put(headerName, request.getHeader(headerName));
+            else if(HOST.equalsIgnoreCase(headerName)) 
+                headers.put(CUSTOM_HOST_HEADER, request.getHeader(headerName));
+        }
+        return headers;
+    }
+
+    public static void initSessionRequestData(USMJSONServlet servlet, Session session) throws USMJSONAccessDeniedException {
+        if (servlet.getAccessLimiter().isUSMAccessDenied(session))
+            throw new USMJSONAccessDeniedException(
+                    ConnectorBundleErrorCodes.COMMAND_REJECTED_TOO_MANY_SYNCHRONIZATIONS,
+                    "Too many unnecessary synchronizations");
+        session.setCustomProperty(OXJSONPropertyNames.NUMBER_OF_CALLS, null);
+        session.setCustomProperty(OXJSONPropertyNames.ACCUMULATED_TIME, null);
+    }
+	
 	protected final USMJSONServlet _servlet;
-	protected final HttpSession _httpSession;
 	protected final JSONObject _parameters;
-	private final HttpServletRequest _currentHttpRequest;
+	protected final HttpServletRequest _currentHttpRequest;
+    
+    private final Map <ContentType, BitSet> _extraFieldsMap = new HashMap<ContentType, BitSet>();
 
 	protected Session _session;
 
-	protected BaseCommandHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters,
-			HttpServletRequest request) throws USMJSONAPIException {
-		_servlet = servlet;
-		_httpSession = httpSession;
-		_parameters = parameters;
-		_currentHttpRequest = request;
-		int requiredCount = 0;
-		for (String key : JSONToolkit.keys(parameters)) {
-			if (checkParameter(key))
-				requiredCount++;
-		}
-		if (requiredCount != getRequiredParameters().length) {
-			for (String key : getRequiredParameters()) {
-				if (!parameters.has(key))
-					throw new USMJSONAPIException(ConnectorBundleErrorCodes.COMMAND_MISSING_REQUIRED_PARAMETER,
-							ResponseStatusCode.WRONG_MISSING_PARAMETERS, "Missing required parameter " + key);
-			}
-		}
-	}
+    protected BaseCommandHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
+        _servlet = servlet;
+        _parameters = extractJSONObjectFromRequest(request);
+        _currentHttpRequest = request;
+        int requiredCount = 0;
+        for (String key : JSONToolkit.keys(_parameters)) {
+            if (checkParameter(key))
+                requiredCount++;
+        }
+        if (requiredCount != getRequiredParameters().length) {
+            for (String key : getRequiredParameters()) {
+                if (!_parameters.has(key))
+                    throw new USMJSONAPIException(
+                        ConnectorBundleErrorCodes.COMMAND_MISSING_REQUIRED_PARAMETER,
+                        ResponseStatusCode.WRONG_MISSING_PARAMETERS,
+                        "Missing required parameter " + key);
+            }
+        }
+    }
 
 	private final boolean checkParameter(String key) throws USMJSONAPIException {
 		for (String requiredKey : getRequiredParameters()) {
@@ -174,6 +247,10 @@ public abstract class BaseCommandHandler implements CommandHandler {
 		return _parameters.has(name) ? getString(_parameters, name) : defaultValue;
 	}
 
+    protected long getLongParameter(String name, long defaultValue) throws USMJSONAPIException {
+        return _parameters.has(name) ? getLong(_parameters, name) : defaultValue;
+    }
+
 	protected boolean hasSyncID() {
 		return _parameters.has(SYNCID);
 	}
@@ -196,16 +273,23 @@ public abstract class BaseCommandHandler implements CommandHandler {
 				ResponseStatusCode.WRONG_MISSING_PARAMETERS, "Bad value for parameter " + CONFLICT_RESOLUTION);
 	}
 
-	protected UUID extractUUIDFromString(String uuid) throws USMJSONAPIException {
-		try {
-			if (uuid != null)
-				return UUID.fromString(uuid);
-		} catch (IllegalArgumentException ignored) {
-			// Fall through to error handling
-		}
-		throw new USMJSONAPIException(ConnectorBundleErrorCodes.COMMAND_INVALID_UUID,
-				ResponseStatusCode.WRONG_MISSING_PARAMETERS, "Not a valid UUID: " + uuid);
-	}
+    protected String getUUIDString(JSONObject object) {
+        return object.optString(UUID_KEY, "<not set>");
+    }
+
+    protected UUID extractUUIDFromJSONObject(JSONObject object) throws InvalidUUIDException {
+        return extractUUIDFromString(getUUIDString(object));
+    }
+
+    protected UUID extractUUIDFromString(String uuid) throws InvalidUUIDException {
+        try {
+            if (uuid != null)
+                return UUID.fromString(uuid);
+        } catch (IllegalArgumentException ignored) {
+            // Fall through to error handling
+        }
+        throw new InvalidUUIDException(uuid);
+    }
 
 	protected JSONArray getJSONArray(JSONObject o, String key) throws USMJSONAPIException {
 		try {
@@ -279,11 +363,15 @@ public abstract class BaseCommandHandler implements CommandHandler {
 		}
 	}
 
-	protected BitSet getExtraFields(ContentType type) {
-		String extraFieldStorage = _session.getPersistentField(JSONSessionInitializer.getTypePropertyKey(type));
-		return (extraFieldStorage == null || extraFieldStorage.length() == 0) ? new BitSet() : BitSetEncoder
-				.fromBase64(extraFieldStorage);
-	}
+    protected BitSet getExtraFields(ContentType type) {
+        BitSet result = _extraFieldsMap.get(type);
+        if (result == null) {
+            String extraFieldStorage = _session.getPersistentField(JSONSessionInitializer.getTypePropertyKey(type));
+            result = (extraFieldStorage == null || extraFieldStorage.length() == 0) ? new BitSet() : BitSetEncoder.fromBase64(extraFieldStorage);
+            _extraFieldsMap.put(type, result);
+        }
+        return result;
+    }
 
 	protected JSONObject getConfiguration() throws USMJSONAPIException {
 		boolean availableContentTypes = getBooleanParameter(AVAILABLE_CONTENT_TYPES);
@@ -299,10 +387,12 @@ public abstract class BaseCommandHandler implements CommandHandler {
 			config.put(END_DATE, String.valueOf(_session.getEndDate()));
 			config.put(FOLDER_TREE, _session.getPersistentField(FolderContentType.FOLDER_TREE));
 			config.put(CONFLICT_RESOLUTION, _session.getConflictResolution());
-			config.put(CONTEXT_UUID, _session.getContextUUID().toString());
+			config.put(CONTEXT_UUID, String.valueOf(_session.getContextUUID()));
 			config.put(CONTEXT_ID, _session.getContextId());
 			config.put(USERID, _session.getUserIdentifier());
 			config.put(DEFAULT_ADDRESS, _session.getDefaultEmailAddress());
+			config.put(MAX_INLINE_ATTACHMENT_SIZE, _servlet.getMaxInlineAttachmentSize());
+			config.put(TEMP_FILES_TIMEOUT, TempFileStorage.getTempFileTimeout());
 
 			config.put(CUSTOM_PROPERTIES, getCustomProperties());
 			//optional
@@ -398,37 +488,30 @@ public abstract class BaseCommandHandler implements CommandHandler {
 		return contentTypesObject;
 	}
 
-	public Session getSession() {
+	@Override
+    public Session getSession() {
 		return _session;
 	}
 
 	protected void setSession(Session session) throws USMJSONAPIException {
-		if (_servlet.getAccessLimiter().isUSMAccessDenied(session))
-			throw new USMJSONAccessDeniedException(
-					ConnectorBundleErrorCodes.COMMAND_REJECTED_TOO_MANY_SYNCHRONIZATIONS,
-					"Too many unnecessary synchronizations");
-		_session = session;
-		_session.setCustomProperty(OXJSONPropertyNames.NUMBER_OF_CALLS, null);
-		_session.setCustomProperty(OXJSONPropertyNames.ACCUMULATED_TIME, null);
+        _session = session;
+		initSessionRequestData(_servlet, session);
 	}
 
 	protected abstract String[] getRequiredParameters();
 
 	protected abstract String[] getOptionalParameters();
 
-	protected DataObject getDataObjectByUUID(DataObjectSet objects, UUID uuid) throws USMJSONAPIException {
-		if (objects != null && objects.contains(uuid)) {
-			DataObject o = objects.get(uuid);
-			if (o.getChangeState() == ChangeState.UNMODIFIED)
-				return o;
-			throw new USMJSONAPIException(ConnectorBundleErrorCodes.COMMAND_MULTIPLE_ACTIONS_ON_DATA_OBJECT,
-					ResponseStatusCode.WRONG_MISSING_PARAMETERS,
-					"More than one action requested on DataObject with uuid " + uuid);
-		}
-		throw new USMJSONAPIException(ConnectorBundleErrorCodes.COMMAND_DATA_OBJECT_NOT_FOUND,
-				ResponseStatusCode.WRONG_MISSING_PARAMETERS, "No DataObject with uuid " + uuid);
-	}
+    protected DataObject getDataObjectByUUID(DataObjectSet objects, UUID uuid) throws DataObjectNotFoundException, MultipleOperationsOnDataObjectException {
+        if (objects == null || !objects.contains(uuid))
+            throw new DataObjectNotFoundException(uuid);
+        DataObject o = objects.get(uuid);
+        if (o.getChangeState() != ChangeState.UNMODIFIED)
+            throw new MultipleOperationsOnDataObjectException(o);
+        return o;
+    }
 
+    // TODO Optimize this
 	protected Folder getFolderByUUID(String folderUUID) throws USMJSONAPIException {
 		try {
 			if (DefaultContentTypes.GROUPS_ID.equals(folderUUID) || DefaultContentTypes.RESOURCES_ID.equals(folderUUID))
@@ -446,19 +529,16 @@ public abstract class BaseCommandHandler implements CommandHandler {
 				ResponseStatusCode.UNKNOWN_UUID, "Unknown folderid " + folderUUID);
 	}
 
-	protected String getClientIP() {
-		return _currentHttpRequest.getRemoteAddr();
-	}
-
 	protected Map<String, String> getXHeadersFromCurrentRequest() {
-		Map<String, String> headers = new HashMap<String, String>();
-		Enumeration<?> headerNames = _currentHttpRequest.getHeaderNames();
-		while (headerNames.hasMoreElements()) {
-			String headerName = (String) headerNames.nextElement();
-			if (headerName.toLowerCase().startsWith("x-"))
-				headers.put(headerName, _currentHttpRequest.getHeader(headerName));
-		}
-		return headers;
-	}
+        return getXHeadersFromRequest(_currentHttpRequest);
+    }
 
+    public HttpSession getHttpSession() {
+        return _currentHttpRequest.getSession();
+    }
+
+    @Override
+    public void disposeResources() {
+        // default: do nothing
+    }
 }
