@@ -70,6 +70,7 @@ public class CommandConstants {
 	public static final String CREATED = "created";
 	public static final String MODIFIED = "modified";
 	public static final String DELETED = "deleted";
+	public static final String DELETED_IF_EXIST = "deletedifexist";
 	public static final String LIMIT = "limit";
 	public static final String MORE_AVAILABLE = "moreAvailable";
 	public static final String CONFIGURATION = "configuration";
@@ -104,7 +105,6 @@ public class CommandConstants {
 	public static final String VERSION = "version";
 	public static final String USERID = "userid";
 	public static final String LOCAL_TIME = "local_time";
-	public static final String REQUESTED_USM_JSON_VERSION = "requestedVersion";
 
 	public static final int KEY_SIZE = 13;
 
@@ -167,4 +167,8 @@ public class CommandConstants {
 	public static final String UID = "uid";
 	public static final String TITLE = "title";
 	public static final String REFRESH = "refresh";
+    public static final String TEMPID = "tempid";
+    public static final String OFFSET = "offset";
+    public static final String LENGTH = "length";
+    public static final String EXACT_SIZE = "exact_size";
 }
