@@ -48,20 +48,12 @@
  */
 package com.openexchange.usm.connector.commands;
 
-import static com.openexchange.usm.connector.commands.CommandConstants.FOLDERID;
-import static com.openexchange.usm.connector.commands.CommandConstants.OBJECTIDS;
-import static com.openexchange.usm.connector.commands.CommandConstants.SESSIONID;
-import static com.openexchange.usm.connector.commands.CommandConstants.SYNCID;
-
+import static com.openexchange.usm.connector.commands.CommandConstants.*;
 import java.util.UUID;
-
 import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpSession;
-
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
-
 import com.openexchange.usm.api.database.DatabaseAccessException;
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.exceptions.USMSQLException;
@@ -86,16 +78,15 @@ public class InvalidateObjectsHandler extends NormalCommandHandler {
 	private static final String[] REQUIRED_PARAMETERS = { SESSIONID, SYNCID };
 	private static final String[] OPTIONAL_PARAMETERS = { FOLDERID, OBJECTIDS };
 
-	public InvalidateObjectsHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters,
-			HttpServletRequest request) throws USMJSONAPIException {
-		super(servlet, httpSession, parameters, request);
-	}
+    public InvalidateObjectsHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
+        super(servlet, request);
+    }
 
 	@Override
 	public ResponseObject handleRequest() throws USMJSONAPIException {
 		long syncID = getSyncID();
 		String folderUUID = getStringParameter(FOLDERID, null);
-		JSONArray objectUUIDs = _parameters.has(OBJECTIDS) ? getJSONArray(_parameters, OBJECTIDS) : null;
+		JSONArray objectUUIDs = _parameters.has(OBJECTIDS) ? getJSONArray(_parameters, OBJECTIDS) : new JSONArray();
 		JSONObject resultData = new JSONObject();
 
 		try {
