@@ -53,11 +53,10 @@ import static com.openexchange.usm.json.response.ResponseObject.*;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.BitSet;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.UUID;
 import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpSession;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
@@ -72,6 +71,9 @@ import com.openexchange.usm.api.session.DataObject;
 import com.openexchange.usm.api.session.Folder;
 import com.openexchange.usm.api.session.SyncResult;
 import com.openexchange.usm.api.session.SynchronizationConflictException;
+import com.openexchange.usm.connector.exceptions.DataObjectNotFoundException;
+import com.openexchange.usm.connector.exceptions.InvalidUUIDException;
+import com.openexchange.usm.connector.exceptions.MultipleOperationsOnDataObjectException;
 import com.openexchange.usm.contenttypes.util.UtilConstants;
 import com.openexchange.usm.json.ConnectorBundleErrorCodes;
 import com.openexchange.usm.json.USMJSONAPIException;
@@ -96,10 +98,9 @@ public class MoveItemsHandler extends SyncCommandHandler {
 
 	private long _originalSyncID;
 
-	public MoveItemsHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters,
-			HttpServletRequest request) throws USMJSONAPIException {
-		super(servlet, httpSession, parameters, request);
-	}
+    public MoveItemsHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
+        super(servlet, request);
+    }
 
 	@Override
     public ResponseObject handleRequest() throws USMJSONAPIException {
@@ -166,12 +167,13 @@ public class MoveItemsHandler extends SyncCommandHandler {
 						ResponseStatusCode.WRONG_MISSING_PARAMETERS, generateErrorMessage(
 								"Source and Destination are not of same type.", sourceFolder, sourceFolderSyncID,
 								destinationFolder, destinationSyncID));
-			DataObjectSet sourceElements = new DataObjectSet(_session.getCachedFolderElements(sourceFolder.getID(),
-					sourceFolder.getElementsContentType(), sourceFolderSyncID));
-			if (sourceElements == null)
-				throw new USMJSONAPIException(ConnectorBundleErrorCodes.MOVE_ITEMS_INVALID_SYNC_ID,
-						ResponseStatusCode.UNKNOWN_SYNCID, generateErrorMessage("Unknown source SyncID", sourceFolder,
-								sourceFolderSyncID, destinationFolder, destinationSyncID));
+			DataObject[] cachedSourceElements = _session.getCachedFolderElements(sourceFolder.getID(),
+					sourceFolder.getElementsContentType(), sourceFolderSyncID);
+            if (cachedSourceElements == null)
+                throw new USMJSONAPIException(ConnectorBundleErrorCodes.MOVE_ITEMS_INVALID_SYNC_ID,
+                        ResponseStatusCode.UNKNOWN_SYNCID, generateErrorMessage("Unknown source SyncID", sourceFolder,
+                                sourceFolderSyncID, destinationFolder, destinationSyncID));
+            DataObjectSet sourceElements = new DataObjectSet(cachedSourceElements);
 			DataObject[] destinationElements = _session.getCachedFolderElements(destinationFolder.getID(),
 					destinationFolder.getElementsContentType(), destinationSyncID);
 			if (destinationElements == null)
@@ -181,18 +183,25 @@ public class MoveItemsHandler extends SyncCommandHandler {
 			_originalSyncID = sourceFolderSyncID;
 			List<DataObject> changesList = new ArrayList<DataObject>();
 			int length = objectsToMove.length();
-			for (int i = 0; i < length; i++) {
-				UUID objectUUID = extractUUIDFromString(getString(objectsToMove, i));
-				try {
-					DataObject object = getDataObjectByUUID(sourceElements, objectUUID);
-					object.setParentFolder(destinationFolder);
-					changesList.add(object);
-				} catch (USMJSONAPIException e) {
-					if (_servlet.isFatalUUIDErrors())
-						throw e;
-					addError(objectUUID, null, e, ErrorStatusCode.UNKNOWN_UUID);
-				}
-			}
+            for (int i = 0; i < length; i++) {
+                String uuidString = getString(objectsToMove, i);
+                try {
+                    DataObject object = getDataObjectByUUID(sourceElements, extractUUIDFromString(uuidString));
+                    object.setParentFolder(destinationFolder);
+                    changesList.add(object);
+                } catch (DataObjectNotFoundException e) {
+                    if (_servlet.isFatalUUIDErrors())
+                        throw USMJSONAPIException.createObjectNotFoundException(e);
+                    addError(uuidString, null, e, ErrorStatusCode.UNKNOWN_UUID);
+                } catch (MultipleOperationsOnDataObjectException e) {
+                    addError(uuidString, e.getDataObject(), e, ErrorStatusCode.MULTIPLE_OPERATIONS_ON_SAME_UUID);
+                    removeDuplicatesAndAddToError(changesList, uuidString, e);
+                } catch (InvalidUUIDException e) {
+                    if (_servlet.isFatalUUIDErrors())
+                        throw USMJSONAPIException.createInvalidUUIDException(e);
+                    addError(uuidString, null, e, ErrorStatusCode.UNKNOWN_UUID);
+                }
+            }
 			changes = new DataObject[changesList.size()];
 			changes = changesList.toArray(changes);
 			SyncResult syncResult = _session.syncChangesWithServer(sourceFolder.getID(), sourceFolderSyncID, -1, null,
@@ -250,14 +259,23 @@ public class MoveItemsHandler extends SyncCommandHandler {
 		return responseData;
 	}
 
-	private void addErrorsToErrorResponse(JSONObject notmoved, int destinationRigths, int sourceRights)
-			throws JSONException {
-		for (Map.Entry<String, SyncErrorData> entry : _errorMap.entrySet()) {
-			String uuid = entry.getKey();
-			SyncErrorData data = entry.getValue();
-			addErrorToNotMovedArray(sourceRights, destinationRigths, notmoved, uuid, data._object, data._error);
-		}
-	}
+	private void removeDuplicatesAndAddToError(List<DataObject> changesList, String uuidString, MultipleOperationsOnDataObjectException e) {
+	    for(Iterator<DataObject> i = changesList.iterator(); i.hasNext();) {
+	        DataObject o = i.next();
+	        if(o.getUUID().toString().equals(uuidString)) {
+	            i.remove();
+                addError(uuidString, o, e, ErrorStatusCode.MULTIPLE_OPERATIONS_ON_SAME_UUID);
+	        }
+	    }
+    }
+
+    private void addErrorsToErrorResponse(JSONObject notmoved, int destinationRigths, int sourceRights) throws JSONException {
+        for (Map.Entry<String, SyncErrorData> entry : _errorMap.entrySet()) {
+            String uuid = entry.getKey();
+            SyncErrorData data = entry.getValue();
+            addErrorToNotMovedArray(sourceRights, destinationRigths, notmoved, uuid, data.getObject(), data._error);
+        }
+    }
 
 	private void addErrorToNotMovedArray(int sourceRights, int destinationRights, JSONObject notmoved, String uuid,
 			DataObject dataObject, USMException exception) throws JSONException {
@@ -300,7 +318,9 @@ public class MoveItemsHandler extends SyncCommandHandler {
 
 	private ErrorStatusCode getErrorStatusCode(int destinationRigths, int sourceRights, DataObject errorDataObject,
 			USMException exception) {
-		if (_errorStatus == ErrorStatusCode.OTHER && errorDataObject != null) {
+	    if(_errorStatus != ErrorStatusCode.OTHER)
+	        return _errorStatus;
+		if (errorDataObject != null) {
 			if (!checkCreatePermissionsAllObjects(destinationRigths))
 				return ErrorStatusCode.DESTINATION_NO_PERMISSION;
 			if (isOwnObject(errorDataObject) && !checkDeletePermissionsOwnObjects(sourceRights))
@@ -308,9 +328,9 @@ public class MoveItemsHandler extends SyncCommandHandler {
 			if (!checkDeletePermissionsAllObjects(sourceRights))
 				return ErrorStatusCode.SOURCE_NO_PERMISSION_ALL_OBJECTS;
 		}
-		if (_errorStatus == ErrorStatusCode.OTHER && isUnknownUuid(exception))
+		if (isUnknownUUIDError(exception))
 			return ErrorStatusCode.UNKNOWN_UUID;
-		return _errorStatus;
+		return ErrorStatusCode.OTHER;
 	}
 
 	private ErrorStatusCode checkFolderExistance(Folder sourceFolder, Folder destinationFolder) {
@@ -357,13 +377,9 @@ public class MoveItemsHandler extends SyncCommandHandler {
 		return resultList.toArray(new DataObject[resultList.size()]);
 	}
 
-	private boolean isUnknownUuid(USMException exception) {
-		if (!(exception instanceof USMJSONAPIException))
-			return false;
-		USMJSONAPIException apiEx = (USMJSONAPIException) exception;
-		return apiEx.getErrorCode() == ConnectorBundleErrorCodes.COMMAND_DATA_OBJECT_NOT_FOUND
-				&& apiEx.getResponseStatusCode() == ResponseStatusCode.WRONG_MISSING_PARAMETERS;
-	}
+    private boolean isUnknownUUIDError(USMException exception) {
+        return (exception instanceof DataObjectNotFoundException) || (exception instanceof InvalidUUIDException);
+    }
 
 	@Override
 	protected String[] getOptionalParameters() {
