@@ -46,6 +46,7 @@
  *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  */
+
 package com.openexchange.usm.connector.commands;
 
 import static com.openexchange.usm.connector.commands.CommandConstants.*;
@@ -53,7 +54,6 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.UUID;
 import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpSession;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
@@ -65,6 +65,7 @@ import com.openexchange.usm.api.session.Folder;
 import com.openexchange.usm.api.session.ObjectChanges;
 import com.openexchange.usm.api.session.Session;
 import com.openexchange.usm.api.session.SyncResult;
+import com.openexchange.usm.connector.exceptions.InvalidUUIDException;
 import com.openexchange.usm.json.ConnectorBundleErrorCodes;
 import com.openexchange.usm.json.USMJSONAPIException;
 import com.openexchange.usm.json.USMJSONServlet;
@@ -74,43 +75,43 @@ import com.openexchange.usm.session.dataobject.DataObjectSet;
 import com.openexchange.usm.util.JSONToolkit;
 
 /**
- * Handler for the USM-Ping Command. 
+ * Handler for the USM-Ping Command.
  * @author ldo
  */
 public class PingHandler extends NormalCommandHandler {
 
     private static final String _DEFAULT_INBOX_FOLDER_KEY = "json.DefaultInboxFolder";
 
-	private static final String INTERVAL = "interval";
-	private static final String[] REQUIRED_PARAMETERS = { SESSIONID };
-	private static final String[] OPTIONAL_PARAMETERS = { OBJECTIDS };
+    private static final String INTERVAL = "interval";
+    private static final String[] REQUIRED_PARAMETERS = { SESSIONID };
+    private static final String[] OPTIONAL_PARAMETERS = { OBJECTIDS };
 
     private DataObjectSet _allCachedFolders;
 
     private boolean _watchFolderHierarchyChanges = true;
 
-    public PingHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters, HttpServletRequest request) throws USMJSONAPIException {
-		super(servlet, httpSession, parameters, request);
-		_servlet.newPingRequestArrived();
-	}
+    public PingHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
+        super(servlet, request);
+        _servlet.newPingRequestArrived();
+    }
 
-	@Override
-	public ResponseObject handleRequest() throws USMJSONAPIException {
+    @Override
+    public ResponseObject handleRequest() throws USMJSONAPIException {
 
-		JSONArray objectUUIDs = _parameters.has(OBJECTIDS) ? getJSONArray(_parameters, OBJECTIDS) : null;
-		try {
+        JSONArray objectUUIDs = _parameters.has(OBJECTIDS) ? getJSONArray(_parameters, OBJECTIDS) : null;
+        try {
             String[] oxIDs = extractOXIdsFromUUIDArray(objectUUIDs);
             ObjectChanges changes = getSession().waitForChanges(_watchFolderHierarchyChanges, oxIDs, 0);
-			JSONArray resultArray = null;
-			if (changes != null) {
-				resultArray = toUUIDsJSONArray(changes.getContentChangeParentIDs());
-				if (changes.hasFolderStructureChanged())
-					resultArray.put(getSession().getContextUUID().toString());
-			}
+            JSONArray resultArray = null;
+            if (changes != null) {
+                resultArray = toUUIDsJSONArray(changes.getContentChangeParentIDs());
+                if (changes.hasFolderStructureChanged())
+                    resultArray.put(String.valueOf(getSession().getContextUUID()));
+            }
             String inboxId = getDefaultInboxId();
             if (inboxId != null && !hasDefaultInboxChanges(changes, inboxId)) {
-				if (resultArray == null)
-					resultArray = new JSONArray();
+                if (resultArray == null)
+                    resultArray = new JSONArray();
                 long newestTimestamp = getSession().getNewestTimestamp(inboxId);
                 if (newestTimestamp > 0 && System.currentTimeMillis() - newestTimestamp >= _servlet.getInboxPollingInterval()) {
                     // clear cached value so that it will be updated from OX configuration
@@ -127,22 +128,24 @@ public class PingHandler extends NormalCommandHandler {
                         if (syncResult.getChanges().length > 0 && defaultInbox.getUUID() != null)
                             resultArray.put(defaultInbox.getUUID());
                     }
-				}
-			}
-			JSONObject data = new JSONObject();
-			if (resultArray != null && resultArray.length() > 0)
-				data.put(OBJECTIDS, resultArray);
-			data.put(INTERVAL, _servlet.getPingInterval());
-			return new ResponseObject(ResponseStatusCode.SUCCESS, data);
-		} catch (USMException e) {
-			throw USMJSONAPIException.createInternalError(ConnectorBundleErrorCodes.PING_INTERNAL_ERROR, e);
-		} catch (InterruptedException e) {
-			throw new USMJSONAPIException(ConnectorBundleErrorCodes.PING_WAIT_FOR_CHANGES_INTERRUPTED_ERROR,
-					ResponseStatusCode.INTERNAL_ERROR, "waiting for changes interrupted", e);
-		} catch (JSONException e) {
-			throw USMJSONAPIException.createJSONError(ConnectorBundleErrorCodes.PING_JSON_ERROR, e);
-		}
-	}
+                }
+            }
+            JSONObject data = new JSONObject();
+            if (resultArray != null && resultArray.length() > 0)
+                data.put(OBJECTIDS, resultArray);
+            data.put(INTERVAL, _servlet.getPingInterval());
+            return new ResponseObject(ResponseStatusCode.SUCCESS, data);
+        } catch (InvalidUUIDException e) {
+            throw USMJSONAPIException.createInvalidUUIDException(e);
+        } catch (USMException e) {
+            throw USMJSONAPIException.createInternalError(ConnectorBundleErrorCodes.PING_INTERNAL_ERROR, e);
+        } catch (InterruptedException e) {
+            throw new USMJSONAPIException(ConnectorBundleErrorCodes.PING_WAIT_FOR_CHANGES_INTERRUPTED_ERROR,
+                    ResponseStatusCode.INTERNAL_ERROR, "waiting for changes interrupted", e);
+        } catch (JSONException e) {
+            throw USMJSONAPIException.createJSONError(ConnectorBundleErrorCodes.PING_JSON_ERROR, e);
+        }
+    }
 
     private DataObjectSet getAllCachedFolders() throws DatabaseAccessException, USMSQLException {
         if (_allCachedFolders == null) {
@@ -158,37 +161,37 @@ public class PingHandler extends NormalCommandHandler {
         String defaultInboxID = JSONToolkit.getString(getSession().getOXUserConfiguration("modules", "mail", "defaultFolder"), "inbox");
         getSession().setCustomProperty(_DEFAULT_INBOX_FOLDER_KEY, defaultInboxID);
         return defaultInboxID;
-	}
+    }
 
     private static boolean hasDefaultInboxChanges(ObjectChanges changes, String inboxId) {
         if (changes == null)
-			return false;
+            return false;
 
-		for (String folderID : changes.getContentChangeParentIDs()) {
+        for (String folderID : changes.getContentChangeParentIDs()) {
             if (folderID.equals(inboxId))
-				return true;
-		}
-		return false;
-	}
+                return true;
+        }
+        return false;
+    }
 
     private UUID getContextUUID() {
-		try {
+        try {
             return getSession().getContextUUID();
-		} catch (DatabaseAccessException e) {
-			USMJSONAPIException.createInternalError(ConnectorBundleErrorCodes.PING_NO_DB_ACCESS, e);
-		} catch (USMSQLException e) {
-			USMJSONAPIException.createInternalError(ConnectorBundleErrorCodes.PING_SQL_ERROR, e);
-		}
+        } catch (DatabaseAccessException e) {
+            USMJSONAPIException.createInternalError(ConnectorBundleErrorCodes.PING_NO_DB_ACCESS, e);
+        } catch (USMSQLException e) {
+            USMJSONAPIException.createInternalError(ConnectorBundleErrorCodes.PING_SQL_ERROR, e);
+        }
         return null; // not reached
-	}
+    }
 
-    private String[] extractOXIdsFromUUIDArray(JSONArray jsonArray) throws USMJSONAPIException, DatabaseAccessException, USMSQLException {
-		if (jsonArray == null)
-			return null;
-		int length = jsonArray.length();
+    private String[] extractOXIdsFromUUIDArray(JSONArray jsonArray) throws USMJSONAPIException, InvalidUUIDException, DatabaseAccessException, USMSQLException {
+        if (jsonArray == null)
+            return null;
+        int length = jsonArray.length();
         List<String> oxIDs = new ArrayList<String>(length);
         UUID contextUUID = getContextUUID();
-		for (int i = 0; i < length; i++) {
+        for (int i = 0; i < length; i++) {
             String id = getString(jsonArray, i);
             if (isSpecialFolderType(id)) {
                 Folder f = (Folder) getAllCachedFolders().get(id);
@@ -203,10 +206,10 @@ public class PingHandler extends NormalCommandHandler {
                     if (f != null)
                         oxIDs.add(f.getID());
                 }
-			}
-		}
+            }
+        }
         return oxIDs.toArray(new String[oxIDs.size()]);
-	}
+    }
 
     private JSONArray toUUIDsJSONArray(String[] idsArray) throws DatabaseAccessException, USMSQLException {
         DataObjectSet allFolders = getAllCachedFolders();
@@ -222,23 +225,22 @@ public class PingHandler extends NormalCommandHandler {
         }
         return result;
     }
-	
+
     private static boolean isSpecialFolderType(String id) {
         return DefaultContentTypes.GROUPS_ID.equals(id) || DefaultContentTypes.RESOURCES_ID.equals(id);
     }
 
     private static boolean isOfDefaultContentType(String contentTypeID) {
         return DefaultContentTypes.FOLDER_ID.equals(contentTypeID) || DefaultContentTypes.CONTACTS_ID.equals(contentTypeID) || DefaultContentTypes.CALENDAR_ID.equals(contentTypeID) || DefaultContentTypes.TASK_ID.equals(contentTypeID) || DefaultContentTypes.MAIL_ID.equals(contentTypeID);
-	}
+    }
 
-	@Override
-	protected String[] getOptionalParameters() {
-		return OPTIONAL_PARAMETERS;
-	}
-
-	@Override
-	protected String[] getRequiredParameters() {
-		return REQUIRED_PARAMETERS;
-	}
+    @Override
+    protected String[] getOptionalParameters() {
+        return OPTIONAL_PARAMETERS;
+    }
 
+    @Override
+    protected String[] getRequiredParameters() {
+        return REQUIRED_PARAMETERS;
+    }
 }
