@@ -49,16 +49,11 @@
 package com.openexchange.usm.connector.commands;
 
 import static com.openexchange.usm.connector.commands.CommandConstants.*;
-
 import java.util.UUID;
-
 import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpSession;
-
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
-
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.session.DataObject;
 import com.openexchange.usm.api.session.Folder;
@@ -82,10 +77,9 @@ public class ResolveIDsHandler extends BaseCommandHandler {
 	private static final String[] REQUIRED_PARAMETERS = { USER, DEVICE, PASSWORD, OBJECT_UUIDS };
 	private static final String[] OPTIONAL_PARAMETERS = { FOLDER_UUID };
 
-	public ResolveIDsHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters,
-			HttpServletRequest request) throws USMJSONAPIException {
-		super(servlet, httpSession, parameters, request);
-	}
+    public ResolveIDsHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
+        super(servlet, request);
+    }
 
 	@Override
 	public ResponseObject handleRequest() throws USMJSONAPIException {
@@ -101,7 +95,7 @@ public class ResolveIDsHandler extends BaseCommandHandler {
 				throw new USMJSONAPIException(ConnectorBundleErrorCodes.RESOLVE_IDS_INVALID_DEVICE,
 						ResponseStatusCode.WRONG_MISSING_PARAMETERS, "Unknown device ID");
 
-			_session = _servlet.getSession(user, password, device, getClientIP(), getXHeadersFromCurrentRequest());
+			_session = _servlet.getSession(user, password, device, _currentHttpRequest.getRemoteAddr(), getXHeadersFromCurrentRequest());
 
 			String folderUUID = getStringParameter(FOLDER_UUID);
 			JSONArray objectUUIDs = getJSONArray(_parameters, OBJECT_UUIDS);
@@ -154,5 +148,4 @@ public class ResolveIDsHandler extends BaseCommandHandler {
 	protected String[] getRequiredParameters() {
 		return REQUIRED_PARAMETERS;
 	}
-
 }
