@@ -48,20 +48,24 @@
  */
 package com.openexchange.usm.connector.commands;
 
-import static com.openexchange.usm.connector.commands.CommandConstants.SESSIONID;
-
-import java.util.*;
-
+import static com.openexchange.usm.connector.commands.CommandConstants.*;
+import java.util.ArrayList;
+import java.util.BitSet;
+import java.util.List;
+import java.util.Map;
 import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpSession;
-
-import org.json.*;
-
-import com.openexchange.usm.api.contenttypes.*;
+import org.json.JSONArray;
+import org.json.JSONException;
+import org.json.JSONObject;
+import com.openexchange.usm.api.contenttypes.ContactContentType;
+import com.openexchange.usm.api.contenttypes.ContentTypeField;
+import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
 import com.openexchange.usm.api.datatypes.DataType;
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.session.DataObject;
-import com.openexchange.usm.json.*;
+import com.openexchange.usm.json.ConnectorBundleErrorCodes;
+import com.openexchange.usm.json.USMJSONAPIException;
+import com.openexchange.usm.json.USMJSONServlet;
 import com.openexchange.usm.json.response.ResponseObject;
 import com.openexchange.usm.json.response.ResponseStatusCode;
 
@@ -78,10 +82,9 @@ public class ResolveRecipientsHandler extends NormalCommandHandler {
 	private static final String[] REQUIRED_PARAMETERS = { SESSIONID, PROPERTIES, NAMES };
 	private static final String[] OPTIONAL_PARAMETERS = {};
 
-	public ResolveRecipientsHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters, HttpServletRequest request)
-			throws USMJSONAPIException {
-		super(servlet, httpSession, parameters, request);
-	}
+    public ResolveRecipientsHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
+        super(servlet, request);
+    }
 
 	@Override
 	public ResponseObject handleRequest() throws USMJSONAPIException {
