@@ -49,19 +49,22 @@
 package com.openexchange.usm.connector.commands;
 
 import static com.openexchange.usm.connector.commands.CommandConstants.*;
-
 import java.util.BitSet;
-
 import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpSession;
-
-import org.json.*;
-
-import com.openexchange.usm.api.contenttypes.*;
+import org.json.JSONArray;
+import org.json.JSONException;
+import org.json.JSONObject;
+import com.openexchange.usm.api.contenttypes.ContentType;
+import com.openexchange.usm.api.contenttypes.ContentTypeField;
+import com.openexchange.usm.api.contenttypes.ContentTypeManager;
+import com.openexchange.usm.api.contenttypes.FolderContentType;
 import com.openexchange.usm.api.database.DatabaseAccessException;
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.exceptions.USMSQLException;
-import com.openexchange.usm.json.*;
+import com.openexchange.usm.json.ConnectorBundleErrorCodes;
+import com.openexchange.usm.json.JSONSessionInitializer;
+import com.openexchange.usm.json.USMJSONAPIException;
+import com.openexchange.usm.json.USMJSONServlet;
 import com.openexchange.usm.json.response.ResponseObject;
 import com.openexchange.usm.json.response.ResponseStatusCode;
 import com.openexchange.usm.mimemail.ExternalMailContentTypeFields;
@@ -84,10 +87,9 @@ public class SetConfigurationHandler extends NormalCommandHandler {
 			CONFLICT_RESOLUTION,
 			CUSTOM_PROPERTIES };
 
-	public SetConfigurationHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters, HttpServletRequest request)
-			throws USMJSONAPIException {
-		super(servlet, httpSession, parameters, request);
-	}
+    public SetConfigurationHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
+        super(servlet, request);
+    }
 
 	@Override
 	public ResponseObject handleRequest() throws USMJSONAPIException {
