@@ -49,16 +49,17 @@
 package com.openexchange.usm.connector.commands;
 
 import static com.openexchange.usm.connector.commands.CommandConstants.*;
-
 import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpSession;
-
 import org.json.JSONException;
 import org.json.JSONObject;
-
 import com.openexchange.usm.api.exceptions.USMException;
-import com.openexchange.usm.api.session.*;
-import com.openexchange.usm.json.*;
+import com.openexchange.usm.api.session.DataObject;
+import com.openexchange.usm.api.session.Session;
+import com.openexchange.usm.api.session.SlowSyncRequiredException;
+import com.openexchange.usm.api.session.SyncResult;
+import com.openexchange.usm.json.ConnectorBundleErrorCodes;
+import com.openexchange.usm.json.USMJSONAPIException;
+import com.openexchange.usm.json.USMJSONServlet;
 import com.openexchange.usm.json.response.ResponseObject;
 import com.openexchange.usm.json.response.ResponseStatusCode;
 
@@ -71,10 +72,9 @@ public class SyncEstimateHandler extends NormalCommandHandler {
 	private static final String[] REQUIRED_PARAMETERS = { SESSIONID };
 	private static final String[] OPTIONAL_PARAMETERS = { FOLDERID, SYNCID };
 
-	public SyncEstimateHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters, HttpServletRequest request)
-			throws USMJSONAPIException {
-		super(servlet, httpSession, parameters, request);
-	}
+    public SyncEstimateHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
+        super(servlet, request);
+    }
 
 	@Override
 	public ResponseObject handleRequest() throws USMJSONAPIException {
