@@ -52,7 +52,6 @@ package com.openexchange.usm.connector.commands;
 import static com.openexchange.usm.connector.commands.CommandConstants.*;
 import java.util.Arrays;
 import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpSession;
 import org.json.JSONObject;
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.session.ConflictResolution;
@@ -77,10 +76,9 @@ public class SyncInitHandler extends SyncCommandHandler {
 	private static final String[] REQUIRED_PARAMETERS = { SESSIONID };
 	private static final String[] OPTIONAL_PARAMETERS = { FOLDERID, CREATED, LIMIT, CONFLICT_RESOLUTION };
 
-	public SyncInitHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters, HttpServletRequest request)
-			throws USMJSONAPIException {
-		super(servlet, httpSession, parameters, request);
-	}
+    public SyncInitHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
+        super(servlet, request);
+    }
 
     @Override
     public ResponseObject handleRequest() throws USMJSONAPIException {
