@@ -58,7 +58,6 @@ import java.util.Map;
 import java.util.Set;
 import java.util.UUID;
 import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpSession;
 import org.json.JSONArray;
 import org.json.JSONObject;
 import com.openexchange.usm.api.contenttypes.ContentType;
@@ -74,6 +73,9 @@ import com.openexchange.usm.api.session.Folder;
 import com.openexchange.usm.api.session.SlowSyncRequiredException;
 import com.openexchange.usm.api.session.SyncResult;
 import com.openexchange.usm.api.session.SynchronizationConflictException;
+import com.openexchange.usm.connector.exceptions.DataObjectNotFoundException;
+import com.openexchange.usm.connector.exceptions.InvalidUUIDException;
+import com.openexchange.usm.connector.exceptions.MultipleOperationsOnDataObjectException;
 import com.openexchange.usm.json.ConnectorBundleErrorCodes;
 import com.openexchange.usm.json.USMJSONAPIException;
 import com.openexchange.usm.json.USMJSONServlet;
@@ -95,6 +97,7 @@ public class SyncUpdateHandler extends SyncCommandHandler {
 			CREATED,
 			MODIFIED,
 			DELETED,
+			DELETED_IF_EXIST,
 			LIMIT,
 			CONFLICT_RESOLUTION,
 			REFRESH };
@@ -120,10 +123,9 @@ public class SyncUpdateHandler extends SyncCommandHandler {
 
 	private final Set<UUID> _refreshUUIDs = new HashSet<UUID>();
 
-	public SyncUpdateHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters,
-			HttpServletRequest request) throws USMJSONAPIException {
-		super(servlet, httpSession, parameters, request);
-	}
+    public SyncUpdateHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
+        super(servlet, request);
+    }
 
     @Override
     public ResponseObject handleRequest() throws USMJSONAPIException {
@@ -132,7 +134,7 @@ public class SyncUpdateHandler extends SyncCommandHandler {
         String acquirer = "syncUpdate:" + lockKey;
         String previousAcquirer = _session.tryLock(lockKey, acquirer);
         if (previousAcquirer != null)
-            return createCannotLockFolderResponse("syncInit", "", lockKey, previousAcquirer);
+            return createCannotLockFolderResponse("syncUpdate", "", lockKey, previousAcquirer);
         try {
             readOptionalLimit();
             ConflictResolution conflictResolution = getOptionalConflictResolution();
@@ -144,6 +146,8 @@ public class SyncUpdateHandler extends SyncCommandHandler {
             }
             readOptionalRefreshUUIDs();
             _syncResult = computeSyncUpdate(folderUUID, _syncID, conflictResolution);
+            if (folderUUID == null) // make sure current folder hierarchy is read again if required for generating response
+                _allFolders = null;
             initializeErrorMap(_syncResult);
             long syncId = _syncResult.getTimestamp();
             Folder folder = null;
@@ -185,22 +189,22 @@ public class SyncUpdateHandler extends SyncCommandHandler {
 		return count;
 	}
 
-	private void readOptionalRefreshUUIDs() throws USMJSONAPIException {
-		if (_parameters.has(REFRESH)) {
-			JSONArray list = getJSONArray(_parameters, REFRESH);
-			int size = list.length();
-			for (int i = 0; i < size; i++) {
-				String uuid = getString(list, i);
-				try {
-					_refreshUUIDs.add(extractUUIDFromString(uuid));
-				} catch (USMJSONAPIException e) {
-					if (_servlet.isFatalUUIDErrors())
-						throw e;
-					addError(uuid, null, e, ErrorStatusCode.UNKNOWN_UUID);
-				}
-			}
-		}
-	}
+    private void readOptionalRefreshUUIDs() throws USMJSONAPIException {
+        if (_parameters.has(REFRESH)) {
+            JSONArray list = getJSONArray(_parameters, REFRESH);
+            int size = list.length();
+            for (int i = 0; i < size; i++) {
+                String uuid = getString(list, i);
+                try {
+                    _refreshUUIDs.add(extractUUIDFromString(uuid));
+                } catch (InvalidUUIDException e) {
+                    if (_servlet.isFatalUUIDErrors())
+                        throw USMJSONAPIException.createInvalidUUIDException(e);
+                    addError(uuid, null, e, ErrorStatusCode.UNKNOWN_UUID);
+                }
+            }
+        }
+    }
 
 	/**
 	 * Modified behavior from default implementation if client sent a non-empty "refresh" array:
@@ -297,7 +301,7 @@ public class SyncUpdateHandler extends SyncCommandHandler {
 			if (folderID == null) {
 				//update folder hierarchy
 				_serverChangesMayBeAvailable = _session.needsSynchronization(null);
-				_folderClientChangesArray = createFolderChangesArray(syncId);
+				_folderClientChangesArray = filterFoldersWithErrors(createFolderChangesArray(syncId));
 				Arrays.sort(_folderClientChangesArray, new FolderHierarchyComparator(_folderClientChangesArray,
 						_servlet.getJournal()));
 				return _session.syncFolderChangesWithServer(syncId, _limit, null, true, conflictResolution,
@@ -305,7 +309,7 @@ public class SyncUpdateHandler extends SyncCommandHandler {
 			}
 			setFolderToSync(folderID);
 			_serverChangesMayBeAvailable = _session.needsSynchronization(_folderToSync.getID());
-			_clientChangesArray = buildClientRequestArray(_folderToSync, syncId, conflictResolution);
+			_clientChangesArray = filterDataObjectsWithErrors(buildClientRequestArray(_folderToSync, syncId, conflictResolution));
 			if (_syncIdForUpdate > 0) {
 				syncId = _syncIdForUpdate;
 				_syncIdForUpdate = 0L;
@@ -368,7 +372,7 @@ public class SyncUpdateHandler extends SyncCommandHandler {
 		} else {
 			modifiedElements = readModifiedFolderElements(folder, originalElements, createdElements);
 		}
-		DataObject[] deletedElements = readDeletedFolderElements(folder, originalElements);
+		DataObject[] deletedElements = readDeletedFolderElements(originalElements);
 		if (createdElements.length == 0 || deletedElements.length == 0 || !isTaskOrAppointmentType(elementsContentType))
 			return buildSimpleClientRequestArray(createdElements, modifiedElements, deletedElements);
 		return buildComplexClientRequestArray(createdElements, modifiedElements, deletedElements);
@@ -445,26 +449,36 @@ public class SyncUpdateHandler extends SyncCommandHandler {
 		}
 	}
 
-	private DataObject[] readModifiedMailDataObjects(Folder folder, ContentType contentType,
-			DataObject[] originalObjects, long syncId) throws USMException {
-		if (!_parameters.has(MODIFIED))
-			return EMPTY_DATAOBJECT_ARRAY;
-		JSONArray list = getJSONArray(_parameters, MODIFIED);
-		int size = list.length();
-		List<DataObject> result = new ArrayList<DataObject>(size);
-		DataObject[] syncStateToStore = new DataObject[originalObjects.length];
-		syncStateToStore = originalObjects;
-		for (int i = 0; i < size; i++) {
-			JSONObject data = getJSONObject(list, i);
-			DataObject object;
-			try {
-				object = getDataObjectFromJSONObjectAndOriginalObjects(contentType, data, originalObjects);
-			} catch (USMJSONAPIException e) {
-				if (_servlet.isFatalUUIDErrors())
-					throw e;
-				addError(getString(data, UUID_KEY), null, e, ErrorStatusCode.UNKNOWN_UUID);
-				continue;
-			}
+    private DataObject[] readModifiedMailDataObjects(Folder folder, ContentType contentType, DataObject[] originalObjects, long syncId) throws USMException {
+        if (!_parameters.has(MODIFIED))
+            return EMPTY_DATAOBJECT_ARRAY;
+        JSONArray list = getJSONArray(_parameters, MODIFIED);
+        int size = list.length();
+        List<DataObject> result = new ArrayList<DataObject>(size);
+        DataObject[] syncStateToStore = new DataObject[originalObjects.length];
+        syncStateToStore = originalObjects;
+        for (int i = 0; i < size; i++) {
+            JSONObject data = getJSONObject(list, i);
+            String uuidString = getUUIDString(data);
+            DataObject object;
+            try {
+                object = getDataObjectFromJSONObjectAndOriginalObjects(contentType, data, originalObjects);
+            } catch (DataObjectNotFoundException e) {
+                if (_servlet.isFatalUUIDErrors())
+                    throw USMJSONAPIException.createObjectNotFoundException(e);
+                addError(uuidString, null, e, ErrorStatusCode.UNKNOWN_UUID);
+                continue;
+            } catch (MultipleOperationsOnDataObjectException e) {
+                if (_servlet.isFatalUUIDErrors())
+                    throw USMJSONAPIException.createMultipleOperationsOnDataObjectException(e);
+                addError(uuidString, e.getDataObject(), e, ErrorStatusCode.MULTIPLE_OPERATIONS_ON_SAME_UUID);
+                continue;
+            } catch (InvalidUUIDException e) {
+                if (_servlet.isFatalUUIDErrors())
+                    throw USMJSONAPIException.createInvalidUUIDException(e);
+                addError(uuidString, null, e, ErrorStatusCode.UNKNOWN_UUID);
+                continue;
+            }
 			if (hasNoError(object)) {
 				try {
 					_syncIdForUpdate = updateMailObject(folder, originalObjects, syncId, result, data, object,
@@ -474,26 +488,35 @@ public class SyncUpdateHandler extends SyncCommandHandler {
 				}
 			}
 		}
-		//if syncStateToStore contains objects from other folder, reset syncStateToStore to originalObjects
-		if (containsForeignObjects(syncStateToStore, folder.getID()))
-			syncStateToStore = originalObjects;
+        // if syncStateToStore contains objects from other folder, reset syncStateToStore to originalObjects
+        if (containsForeignObjects(syncStateToStore, folder.getID()))
+            syncStateToStore = originalObjects;
 
-		_syncIdForUpdate = _session.storeSyncState(getOriginalSyncID(), syncId, folder.getID(), syncStateToStore);
-		DataObject[] savedElements = _session.getCachedFolderElements(folder.getID(), contentType, _syncIdForUpdate);
-		for (int i = 0; i < size; i++) {
-			JSONObject data = getJSONObject(list, i);
-			try {
-				DataObject object = getDataObjectFromJSONObjectAndOriginalObjects(contentType, data, savedElements);
-				if (hasNoError(object))
-					result.add(object);
-			} catch (USMJSONAPIException e) {
-				if (_servlet.isFatalUUIDErrors())
-					throw e;
-				addError(getString(data, UUID_KEY), null, e, ErrorStatusCode.UNKNOWN_UUID);
-			}
-		}
-		return result.toArray(new DataObject[result.size()]);
-	}
+        _syncIdForUpdate = _session.storeSyncState(getOriginalSyncID(), syncId, folder.getID(), syncStateToStore);
+        DataObject[] savedElements = _session.getCachedFolderElements(folder.getID(), contentType, _syncIdForUpdate);
+        for (int i = 0; i < size; i++) {
+            JSONObject data = getJSONObject(list, i);
+            String uuidString = getUUIDString(data);
+            try {
+                DataObject object = getDataObjectFromJSONObjectAndOriginalObjects(contentType, data, savedElements);
+                if (hasNoError(object))
+                    result.add(object);
+            } catch (DataObjectNotFoundException e) {
+                if (_servlet.isFatalUUIDErrors())
+                    throw USMJSONAPIException.createObjectNotFoundException(e);
+                addError(uuidString, null, e, ErrorStatusCode.UNKNOWN_UUID);
+            } catch (MultipleOperationsOnDataObjectException e) {
+                if (_servlet.isFatalUUIDErrors())
+                    throw USMJSONAPIException.createMultipleOperationsOnDataObjectException(e);
+                addError(uuidString, e.getDataObject(), e, ErrorStatusCode.MULTIPLE_OPERATIONS_ON_SAME_UUID);
+            } catch (InvalidUUIDException e) {
+                if (_servlet.isFatalUUIDErrors())
+                    throw USMJSONAPIException.createInvalidUUIDException(e);
+                addError(uuidString, null, e, ErrorStatusCode.UNKNOWN_UUID);
+            }
+        }
+        return result.toArray(new DataObject[result.size()]);
+    }
 
 	private boolean containsForeignObjects(DataObject[] syncStateToStore, String folderId) {
 		for (DataObject dataObject : syncStateToStore) {
