@@ -50,11 +50,8 @@
 package com.openexchange.usm.json;
 
 import static com.openexchange.usm.json.ServletConstants.*;
-import java.io.BufferedReader;
 import java.io.IOException;
-import java.io.InputStreamReader;
 import java.io.OutputStreamWriter;
-import java.io.UnsupportedEncodingException;
 import java.io.Writer;
 import java.util.Map;
 import javax.servlet.ServletException;
@@ -82,11 +79,10 @@ import com.openexchange.usm.json.response.ResponseObject;
 import com.openexchange.usm.json.response.ResponseStatusCode;
 import com.openexchange.usm.ox_json.OXJSONPropertyNames;
 import com.openexchange.usm.util.AccessLimiter;
-import com.openexchange.usm.util.Toolkit;
 
 public class USMJSONServlet extends HttpServlet {
 
-    private static final int ONE_SECOND = 1000;
+    private static final long ONE_SECOND = 1000L;
 
     private static final String COMMAND = "action";
 
@@ -113,6 +109,8 @@ public class USMJSONServlet extends HttpServlet {
 
     private transient int _pingTimestampsIndex = -1;
 
+    private final transient Object _pingSynchronizeObject = new Object();
+
     private transient int _minPingInterval;
 
     private transient int _maxPingInterval;
@@ -137,6 +135,8 @@ public class USMJSONServlet extends HttpServlet {
 
     private transient int _maxEmailSize;
 
+    private transient int _maxInlineAttachmentSize;
+
     private transient AccessLimiter _accessLimiter;
 
     /**
@@ -205,6 +205,10 @@ public class USMJSONServlet extends HttpServlet {
                 ConfigurationProperties.JSON_SERVLET_MAX_EMAIL_SIZE_PROPERTY,
                 ConfigurationProperties.JSON_SERVLET_MAX_EMAIL_SIZE_DEFAULT,
                 false);
+            _maxInlineAttachmentSize = _configurationManager.getProperty(
+                ConfigurationProperties.JSON_SERVLET_MAX_INLINE_ATTACHMENT_SIZE_PROPERTY,
+                ConfigurationProperties.JSON_SERVLET_MAX_INLINE_ATTACHMENT_SIZE_DEFAULT,
+                false);
             int testInterval = _configurationManager.getProperty(
                 ConfigurationProperties.JSON_SERVLET_ACCESS_CHECK_TEST_INTERVAL_PROPERTY,
                 ConfigurationProperties.JSON_SERVLET_ACCESS_CHECK_TEST_INTERVAL_DEFAULT,
@@ -221,7 +225,7 @@ public class USMJSONServlet extends HttpServlet {
         } catch (USMInvalidConfigurationException e1) {
             logAndThrowStartupError(
                 ConnectorBundleErrorCodes.INVALID_ALIAS_FOR_SERVLET,
-                ConfigurationProperties.JSON_SERVLET_ALIAS_PROPERTY,
+                e1.getMessage(),
                 e1);
         }
 
@@ -266,95 +270,114 @@ public class USMJSONServlet extends HttpServlet {
     protected void doPut(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
         long startTime = System.currentTimeMillis();
         String command = getCommandName(request);
+        Session session = null;
         if (_journal.isDebugEnabled()) {
             _journal.debug("HTTP PUT " + command);
         }
         try {
-            ResponseObject result;
-            CommandHandler handler = null;
-            boolean printedError = false;
-            try {
-                JSONObject parameters = extractJSONObjectFromRequest(request);
-                handler = CommandDelegateFactory.createCommandHandler(this, request.getSession(), command, parameters, request);
-                result = handler.handleRequest();
-            } catch (USMJSONAPIException e) {
-                result = new ResponseObject(e);
-            } catch (USMResourceAccessException e) {
-                JSONObject errorDetailsObject = new JSONObject();
-                USMException.appendStacktrace(e, errorDetailsObject);
-                result = new ResponseObject(
-                    ResponseStatusCode.INTERNAL_ERROR,
-                    ConnectorBundleErrorCodes.EXCEPTION_IN_SERVLET_2,
-                    e.getMessage(),
-                    errorDetailsObject,
-                    null);
-                _journal.error(computeLogMessage(command, handler, result), e);
-                printedError = true;
-            } catch (Exception e) {
-                JSONObject errorDetailsObject = new JSONObject();
-                USMException.appendStacktrace(e, errorDetailsObject);
-                result = new ResponseObject(
-                    ResponseStatusCode.INTERNAL_ERROR,
-                    ConnectorBundleErrorCodes.EXCEPTION_IN_SERVLET,
-                    e.getMessage(),
-                    errorDetailsObject,
-                    null);
-                _journal.error(computeLogMessage(command, handler, result), e);
-                printedError = true;
-            }
-            switch (result.getStatusCode()) {
-            case SUCCESS:
-            case ACCESS_DENIED:
-            case BAD_REQUEST:
-            case SYNC_FAILED:
-            case UNKNOWN_SESSION:
-            case UNKNOWN_SYNCID:
-            case UNKNOWN_UUID:
-            case WRONG_MISSING_PARAMETERS:
-            case UNSUPPORTED_VERSION:
-                if (_journal.isDebugEnabled()) {
-                    String message = computeLogMessage(command, handler, result);
-                    USMJSONException e = result.getCause();
-                    if (e instanceof Throwable) {
-                        _journal.debug(message, (Throwable) e);
-                    } else {
-                        _journal.debug(message);
-                    }
-                }
-                break;
-            case DATABASE_ERROR:
-            case INTERNAL_ERROR:
-            case OX_SERVER_ERROR:
-            default:
-                if (!printedError) {
-                    String message = computeLogMessage(command, handler, result);
-                    USMJSONException e = result.getCause();
-                    if (e instanceof USMJSONAccessDeniedException) {
-                        _journal.info(result.getErrorMessage());
-                    } else if (e instanceof Throwable) {
-                        _journal.error(message, (Throwable) e);
-                    } else {
-                        _journal.error(result.toString(3));
-                        _journal.error(message);
-                    }
-                }
-                break;
-            }
-            response.setContentType(ServletConstants.USM_JSON_CONTENT_TYPE);
-            response.setContentLength(computeContentLength(result));
-            Writer w = response.getWriter();
-            if (w != null) {
-                result.write(w);
-                w.flush();
-                w.close();
-            }
-            if (_journal.isDebugEnabled()) {
-                _journal.debug(generateTimeLog(handler, command, System.currentTimeMillis() - startTime));
-            }
+            session = handleCommand(request, response, command);
         } catch (Exception error) {
             _journal.error("Open-Xchange USM/JSON: Internal server error in " + command, error);
             sendError(response, HttpStatus.SC_INTERNAL_SERVER_ERROR);
         }
+        if (_journal.isDebugEnabled()) {
+            _journal.debug(generateTimeLog(session, command, System.currentTimeMillis() - startTime));
+        }
+    }
+
+    protected Session handleCommand(HttpServletRequest request, HttpServletResponse response, String command) throws JSONException, IOException {
+        ResponseObject result;
+        CommandHandler handler = null;
+        boolean printedError = false;
+        try {
+            verifyContentType(request.getContentType());
+            handler = CommandDelegateFactory.createCommandHandler(this, command, request, response);
+            result = handler.handleRequest();
+        } catch (USMJSONAPIException e) {
+            result = new ResponseObject(e);
+        } catch (USMResourceAccessException e) {
+            JSONObject errorDetailsObject = new JSONObject();
+            USMException.appendStacktrace(e, errorDetailsObject);
+            result = new ResponseObject(
+                ResponseStatusCode.INTERNAL_ERROR,
+                ConnectorBundleErrorCodes.EXCEPTION_IN_SERVLET_2,
+                e.getMessage(),
+                errorDetailsObject,
+                null);
+            _journal.error(computeLogMessage(command, handler, result), e);
+            printedError = true;
+        } catch (Exception e) {
+            JSONObject errorDetailsObject = new JSONObject();
+            USMException.appendStacktrace(e, errorDetailsObject);
+            result = new ResponseObject(
+                ResponseStatusCode.INTERNAL_ERROR,
+                ConnectorBundleErrorCodes.EXCEPTION_IN_SERVLET,
+                e.getMessage(),
+                errorDetailsObject,
+                null);
+            _journal.error(computeLogMessage(command, handler, result), e);
+            printedError = true;
+        }
+        if (result != null) {
+            logResult(command, result, handler, printedError);
+            if (handler != null)
+                handler.disposeResources();
+            sendResponse(response, result);
+        }
+        return (handler == null) ? null : handler.getSession();
+    }
+
+    private static void sendResponse(HttpServletResponse response, ResponseObject result) throws JSONException, IOException {
+        response.setContentType(ServletConstants.USM_JSON_CONTENT_TYPE);
+        response.setContentLength(computeContentLength(result));
+        Writer w = response.getWriter();
+        if (w != null) {
+            result.write(w);
+            w.flush();
+            w.close();
+        }
+    }
+
+    protected void logResult(String command, ResponseObject result, CommandHandler handler, boolean printedError) throws JSONException {
+        switch (result.getStatusCode()) {
+        case SUCCESS:
+        case ACCESS_DENIED:
+        case BAD_REQUEST:
+        case SYNC_FAILED:
+        case UNKNOWN_SESSION:
+        case UNKNOWN_SYNCID:
+        case UNKNOWN_UUID:
+        case WRONG_MISSING_PARAMETERS:
+        case UNSUPPORTED_VERSION:
+        case SUCCESS_FILTERED_OUT_LARGE_OBJECTS:
+            if (_journal.isDebugEnabled()) {
+                String message = computeLogMessage(command, handler, result);
+                USMJSONException e = result.getCause();
+                if (e instanceof Throwable) {
+                    _journal.debug(message, (Throwable) e);
+                } else {
+                    _journal.debug(message);
+                }
+            }
+            break;
+        case DATABASE_ERROR:
+        case INTERNAL_ERROR:
+        case OX_SERVER_ERROR:
+        default:
+            if (!printedError) {
+                String message = computeLogMessage(command, handler, result);
+                USMJSONException e = result.getCause();
+                if (e instanceof USMJSONAccessDeniedException) {
+                    _journal.info(result.getErrorMessage());
+                } else if (e instanceof Throwable) {
+                    _journal.error(message, (Throwable) e);
+                } else {
+                    _journal.error(result.toString(3));
+                    _journal.error(message);
+                }
+            }
+            break;
+        }
     }
 
     private static int computeContentLength(ResponseObject result) throws JSONException, IOException {
@@ -365,7 +388,7 @@ public class USMJSONServlet extends HttpServlet {
         return counter.getByteCount();
     }
 
-    private void sendError(HttpServletResponse response, int status) throws IOException {
+    private static void sendError(HttpServletResponse response, int status) throws IOException {
         if (!response.isCommitted()) {
             try {
                 response.sendError(status);
@@ -375,7 +398,7 @@ public class USMJSONServlet extends HttpServlet {
         }
     }
 
-    private String computeLogMessage(String command, CommandHandler handler, ResponseObject result) {
+    private static String computeLogMessage(String command, CommandHandler handler, ResponseObject result) {
         Session session = (handler == null) ? null : handler.getSession();
         StringBuilder sb = new StringBuilder(256);
         sb.append(session).append(' ').append(command).append(' ').append(result.getStatusCode());
@@ -388,13 +411,13 @@ public class USMJSONServlet extends HttpServlet {
         return sb.toString();
     }
 
-    private String getCommandName(HttpServletRequest request) {
+    private static String getCommandName(HttpServletRequest request) {
         String[] result = request.getParameterValues(COMMAND);
         if (result != null && result.length == 1) {
             return result[0];
         }
         String command = request.getPathInfo();
-        // Remove all leading '/' in the sub-path (not just the first, in case the client is uses a wrong configuration)
+        // Remove all leading '/' in the sub-path (not just the first, in case the client uses a wrong configuration)
         int i = 0;
         int length = command.length();
         while (length > i && command.charAt(i) == '/') {
@@ -403,19 +426,14 @@ public class USMJSONServlet extends HttpServlet {
         if (i > 0) {
             command = command.substring(i);
         }
-        // XXX This is only a temporary workaround for a bug in the OX ServletContainer
-        // int index = command.lastIndexOf('/');
-        // if (index >= 0)
-        // command = command.substring(index + 1);
         return command;
     }
 
-    private JSONObject extractJSONObjectFromRequest(HttpServletRequest request) throws USMJSONAPIException {
-        // Only use the content-type before a ";", ignore any extra information
-        String contentType = request.getContentType();
+    protected static void verifyContentType(String contentType) throws USMJSONAPIException {
         if (contentType != null) {
             int semiColon = contentType.indexOf(';');
             if (semiColon >= 0) {
+                // Only use the content-type before a ";", ignore any extra information
                 contentType = contentType.substring(0, semiColon);
             }
         }
@@ -425,37 +443,6 @@ public class USMJSONServlet extends HttpServlet {
                 ResponseStatusCode.BAD_REQUEST,
                 "Request type not \"text/javascript\"");
         }
-        // read the JSON object from the request
-        BufferedReader reader = null;
-        try {
-            reader = new BufferedReader(new InputStreamReader(request.getInputStream(), ServletConstants.UTF_ENCODING_8));
-            char[] buffer = new char[4096];
-            StringBuilder sb = new StringBuilder(4096);
-            for (int count = reader.read(buffer); count > 0; count = reader.read(buffer)) {
-                sb.append(buffer, 0, count);
-            }
-            JSONObject requestObject = new JSONObject(sb.toString());
-            return requestObject;
-        } catch (UnsupportedEncodingException e) {
-            throw new USMJSONAPIException(
-                ConnectorBundleErrorCodes.UTF_8_NOT_SUPPORTED,
-                ResponseStatusCode.INTERNAL_ERROR,
-                "UTF-8 Encoding is not supported");
-        } catch (IOException e) {
-            throw new USMJSONAPIException(
-                ConnectorBundleErrorCodes.ERROR_ON_READ_REQUEST,
-                ResponseStatusCode.BAD_REQUEST,
-                "Error while reading request",
-                e);
-        } catch (JSONException e) {
-            throw new USMJSONAPIException(
-                ConnectorBundleErrorCodes.BAD_JSONOBJECT_IN_REQUEST,
-                ResponseStatusCode.BAD_REQUEST,
-                "Invalid JSONObject as parameter",
-                e);
-        } finally {
-            Toolkit.close(reader);
-        }
     }
 
     public SessionManager getSessionManager() {
@@ -480,40 +467,33 @@ public class USMJSONServlet extends HttpServlet {
     }
 
     public long getPingInterval() {
-        int nextIndex = (_pingTimestampsIndex + 1 >= _pingTimestamps.length) ? 0 : _pingTimestampsIndex + 1;
-        if (_pingTimestamps[nextIndex] == 0) {
-            return _minPingInterval;
-        }
-        long deltaT = _pingTimestamps[_pingTimestampsIndex] - _pingTimestamps[nextIndex];
-        if (deltaT < 1) {
-            deltaT = 1;
-        }
-        if (deltaT >= ONE_SECOND * _pingSampleSize) {
-            return _minPingInterval;
-        } else {
-            long newInterval = _minPingInterval * ONE_SECOND / deltaT;
-            return (newInterval > _maxPingInterval) ? _maxPingInterval : newInterval;
+        synchronized (_pingSynchronizeObject) {
+            int nextIndex = (_pingTimestampsIndex + 1 >= _pingTimestamps.length) ? 0 : _pingTimestampsIndex + 1;
+            if (_pingTimestamps[nextIndex] == 0L)
+                return _minPingInterval;
+            long deltaT = Math.max(_pingTimestamps[_pingTimestampsIndex] - _pingTimestamps[nextIndex], 1L);
+            if (deltaT >= ONE_SECOND * _pingSampleSize)
+                return _minPingInterval;
+            return Math.min(_maxPingInterval, _minPingInterval * ONE_SECOND / deltaT);
         }
     }
 
     public void newPingRequestArrived() {
-        _pingTimestampsIndex++;
-        if (_pingTimestampsIndex >= _pingTimestamps.length) {
-            _pingTimestampsIndex = 0;
+        synchronized (_pingSynchronizeObject) {
+            _pingTimestampsIndex++;
+            if (_pingTimestampsIndex >= _pingTimestamps.length)
+                _pingTimestampsIndex = 0;
+            _pingTimestamps[_pingTimestampsIndex] = System.currentTimeMillis();
         }
-        _pingTimestamps[_pingTimestampsIndex] = System.currentTimeMillis();
     }
 
-    private String generateTimeLog(CommandHandler handler, String command, long duration) {
-        Session session = (handler == null) ? null : handler.getSession();
-        if (session == null) {
+    private static String generateTimeLog(Session session, String command, long duration) {
+        if (session == null)
             return "No session: " + command + " executed in " + duration + " ms.";
-        }
         Object numberOfCalls = session.getCustomProperty(OXJSONPropertyNames.NUMBER_OF_CALLS);
         Object accumulatedTime = session.getCustomProperty(OXJSONPropertyNames.ACCUMULATED_TIME);
-        if (numberOfCalls == null || accumulatedTime == null) {
+        if (numberOfCalls == null || accumulatedTime == null)
             return session + ": " + command + " executed in " + duration + " ms, no OX calls made.";
-        }
         return session + ": " + command + " executed in " + duration + " ms, " + numberOfCalls + " OX calls took " + accumulatedTime + " ms.";
     }
 
@@ -552,4 +532,8 @@ public class USMJSONServlet extends HttpServlet {
     public int getMaxEmailSize() {
         return _maxEmailSize;
     }
+
+    public int getMaxInlineAttachmentSize() {
+        return _maxInlineAttachmentSize;
+    }
 }
