@@ -59,15 +59,14 @@ import javax.mail.internet.MimeMessage;
  * @author ibr
  *
  */
-public class USMMimeMessage extends MimeMessage {
+public class USMMimeMessage extends MimeMessage implements OutputStreamWriter {
 
 	public USMMimeMessage(Session session) {
 		super(session);
 		
 	}
 
-    @Override
-	protected void updateMessageID() throws MessagingException {
+	protected void updateMessageID() {
 		//The superclass MimeMessage updates always the Header Message-ID 
     	//with own generated Unique - identifier. It will be not different, if the Message-Id Header was already set or not.
 	}	
