@@ -48,13 +48,13 @@
  */
 package com.openexchange.usm.ox_event.impl;
 
-import java.util.*;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.CopyOnWriteArrayList;
-
 import org.apache.commons.logging.Log;
 import org.osgi.service.event.Event;
 import org.osgi.service.event.EventHandler;
-
 import com.openexchange.event.CommonEvent;
 import com.openexchange.event.RemoteEvent;
 import com.openexchange.groupware.Types;
@@ -71,15 +71,31 @@ import com.openexchange.resource.*;
  *
  */
 public class OXEventManagerImpl implements OXEventManager, EventHandler {
-	private static final boolean USE_NEW_AFFECTED_USERS_METHOD = true;
+
+    private static final boolean USE_NEW_AFFECTED_USERS_METHOD = true;
 	
 	private static final String GROUP_TOPICS_PREFIX = "com/openexchange/groupware/group/";
 	private static final String RESOURCES_TOPIC_PREFIX = "com/openexchange/groupware/resource/";
 
-	/*
-	 * The constants were copied from com.openexchange.push.PushEventConstants
-	 * to avoid a requirement on that package which would need the appriopriate push-bundle.
-	 */
+    /*
+     * The constants were copied from com.openexchange.resource.ResourceEventConstants
+     * and com.openexchange.group.GroupEventConstants to avoid a requirement on that
+     * package which would need the appropriate push-bundle.
+     * TODO Replace this with the real constant references once the constants are available in all relevant OX sever branches.
+     */
+    // private static final String RESOURCE_PROPERTY_USER_ID = ResourceEventConstants.PROPERTY_USER_ID;
+    // private static final String RESOURCE_PROPERTY_CONTEXT_ID = ResourceEventConstants.PROPERTY_CONTEXT_ID;
+    // private static final String GROUP_PROPERTY_USER_ID = GroupEventConstants.PROPERTY_USER_ID;
+    // private static final String GROUP_PROPERTY_CONTEXT_ID = GroupEventConstants.PROPERTY_CONTEXT_ID;
+
+    private static final String RESOURCE_PROPERTY_USER_ID = "userId";
+
+    private static final String RESOURCE_PROPERTY_CONTEXT_ID = "contextId";
+
+    private static final String GROUP_PROPERTY_USER_ID = "userId";
+
+    private static final String GROUP_PROPERTY_CONTEXT_ID = "contextId";
+	
 
 	private static final class PushEventConstants {
 
@@ -159,7 +175,7 @@ public class OXEventManagerImpl implements OXEventManager, EventHandler {
             } else if (event.getTopic().startsWith(RESOURCES_TOPIC_PREFIX)) {
                 handleResourceEvent(event);
             } else if (oxEventData instanceof CommonEvent) {
-                handleCommonEvent((CommonEvent) oxEventData);
+                handleCommonEvent(event, (CommonEvent) oxEventData);
             } else {
                 oxEventData = event.getProperty(RemoteEvent.EVENT_KEY);
                 if (oxEventData instanceof RemoteEvent) {
@@ -173,7 +189,7 @@ public class OXEventManagerImpl implements OXEventManager, EventHandler {
         }
     }
 
-	private void handlePushEvent(Event event) {
+	private boolean handlePushEvent(Event event) {
 		String folder = (String) event.getProperty(PushEventConstants.PROPERTY_FOLDER);
 		int contextId = getIntProperty(event, PushEventConstants.PROPERTY_CONTEXT);
 		int userId = getIntProperty(event, PushEventConstants.PROPERTY_USER);
@@ -181,15 +197,16 @@ public class OXEventManagerImpl implements OXEventManager, EventHandler {
 		boolean folderChange = contentRelated != null && !contentRelated.booleanValue();
 		if (_journal.isDebugEnabled())
 			_journal.debug("PushEvent " + folderChange + ":" + contextId + ":" + userId + ":" + folder);
-		if (folder != null && contextId != 0 && userId != 0)
-			notifyListeners(folderChange, contextId, userId, folder);
+		if (folder == null || contextId == 0 || userId == 0)
+		    return false;
+		notifyListeners(folderChange, contextId, userId, folder);
+		return true;
 	}
 	
 	
-	@SuppressWarnings("restriction")
     private void handleGroupEvent(Event event) {
-        int contextId = getIntProperty(event, GroupEventConstants.PROPERTY_CONTEXT_ID);
-        int userId = getIntProperty(event, GroupEventConstants.PROPERTY_USER_ID);
+        int contextId = getIntProperty(event, GROUP_PROPERTY_CONTEXT_ID);
+        int userId = getIntProperty(event, GROUP_PROPERTY_USER_ID);
         String folderID = DefaultContentTypes.GROUPS_ID;
         boolean folderChange = false;
         if (_journal.isDebugEnabled())
@@ -199,10 +216,9 @@ public class OXEventManagerImpl implements OXEventManager, EventHandler {
     }
 	
 	
-	@SuppressWarnings("restriction")
     private void handleResourceEvent(Event event) {
-	    int contextId = getIntProperty(event, ResourceEventConstants.PROPERTY_CONTEXT_ID);
-        int userId = getIntProperty(event, ResourceEventConstants.PROPERTY_USER_ID);
+	    int contextId = getIntProperty(event, RESOURCE_PROPERTY_CONTEXT_ID);
+        int userId = getIntProperty(event, RESOURCE_PROPERTY_USER_ID);
         String folderID = DefaultContentTypes.RESOURCES_ID;
         boolean folderChange = false;
         if (_journal.isDebugEnabled())
@@ -224,7 +240,7 @@ public class OXEventManagerImpl implements OXEventManager, EventHandler {
 		return null;
 	}
 
-	private void handleCommonEvent(CommonEvent oxEvent) {
+	private void handleCommonEvent(Event event, CommonEvent oxEvent) {
 		int userId = oxEvent.getUserId();
 		int contextId = oxEvent.getContextId();
 		int module = oxEvent.getModule();
@@ -267,7 +283,8 @@ public class OXEventManagerImpl implements OXEventManager, EventHandler {
 			} else if (destFolderID != null) {
 				notifyListeners(isFolderChange, contextId, userId, destFolderID);
 			} else {
-				notifyListeners(isFolderChange, contextId, userId, null);
+			    if(!handlePushEvent(event))
+			        notifyListeners(isFolderChange, contextId, userId, null);
 			}
 		}
 		if (oxEvent.getAction() == CommonEvent.DELETE) {
