@@ -75,11 +75,13 @@ import org.apache.commons.httpclient.methods.ByteArrayRequestEntity;
 import org.apache.commons.httpclient.methods.GetMethod;
 import org.apache.commons.httpclient.methods.PostMethod;
 import org.apache.commons.httpclient.methods.PutMethod;
+import org.apache.commons.httpclient.methods.RequestEntity;
 import org.apache.commons.httpclient.methods.StringRequestEntity;
 import org.apache.commons.httpclient.methods.multipart.ByteArrayPartSource;
 import org.apache.commons.httpclient.methods.multipart.FilePart;
 import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
 import org.apache.commons.httpclient.methods.multipart.Part;
+import org.apache.commons.httpclient.methods.multipart.PartSource;
 import org.apache.commons.httpclient.methods.multipart.StringPart;
 import org.apache.commons.httpclient.params.HttpConnectionManagerParams;
 import org.apache.commons.httpclient.params.HttpMethodParams;
@@ -88,12 +90,14 @@ import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
 import com.openexchange.usm.api.USMVersion;
+import com.openexchange.usm.api.contenttypes.ResourceInputStream;
 import com.openexchange.usm.api.exceptions.AuthenticationFailedException;
 import com.openexchange.usm.api.exceptions.ConflictingChangeException;
 import com.openexchange.usm.api.exceptions.OXCommunicationException;
 import com.openexchange.usm.api.exceptions.TemporaryDownOrBusyException;
 import com.openexchange.usm.api.exceptions.USMIllegalStateException;
 import com.openexchange.usm.api.exceptions.USMStartupException;
+import com.openexchange.usm.api.io.InputStreamProvider;
 import com.openexchange.usm.api.session.OXConnectionInformation;
 import com.openexchange.usm.api.session.Session;
 import com.openexchange.usm.ox_json.JSONResult;
@@ -108,6 +112,8 @@ import com.openexchange.usm.ox_json.OXResource;
 public class OXJSONAccessImpl implements OXJSONAccess {
 
     private static final String OX_REQUEST_INFORMATION_FIELD = "OXJSONAccessImpl.OXRequestInformation";
+    
+    private static final String CUSTOM_HOST_HEADER = "X-Host";
 
     private static final int MAX_RETRY = 10;
 
@@ -125,10 +131,6 @@ public class OXJSONAccessImpl implements OXJSONAccess {
 
     private final static int RESOURCE_BUFFER_DEFAULT_SIZE = 40000;
 
-    private final static String ACTION_KEY = "**action**";
-
-    private final static String JSON_KEY = "**json**";
-
     private final static String FILE_PART_NAME = "file";
 
     private final static String STRING_PART_NAME = "json";
@@ -145,13 +147,13 @@ public class OXJSONAccessImpl implements OXJSONAccess {
 
     private final static String HTTPS_PREFIX = "https:";
 
-    // private final static String UDATE_ACTION = "update";
-
     // Check after 5 minutes if redirect still active, restricts number of log messages and redirect tests
     private static final long REDIRECT_RETEST_INTERVAL = 300000L;
 
     private static final long LOCK_TIMEOUT = 100;
 
+    private static final long _STREAM_INACTIVITY_TIMEOUT = 20000L;
+
     private static final String USM_USER_AGENT = "Open-Xchange USM HTTP Client";
 
     private Log _journal;
@@ -166,13 +168,9 @@ public class OXJSONAccessImpl implements OXJSONAccess {
 
     private HttpURL _oxJSONBaseURL;
 
-    private String _postResultPattern;
-
-    private int _actionIndex;
-
-    private int _jsonIndex;
-
     private long _lastRedirect = 0L;
+    
+    private boolean _passClientHostHeader = false;
 
     public void login(Session session) throws AuthenticationFailedException, OXCommunicationException {
         logout(session, false);
@@ -248,7 +246,10 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         method.addParameter("name", user);
         method.addParameter("password", password);
         for (String header : session.getXRequestHeaders().keySet()) {
-            method.setRequestHeader(header, session.getXRequestHeaders().get(header));
+            if (CUSTOM_HOST_HEADER.equals(header) && _passClientHostHeader) {
+                    method.setRequestHeader(header, session.getXRequestHeaders().get(header));
+            } else
+                method.setRequestHeader(header, session.getXRequestHeaders().get(header));
         }
         JSONResult result = executeMethodOnce(session, client, method);
         if (result.getResultType() == JSONResultType.Error)
@@ -274,7 +275,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         }
     }
 
-    private String createClientIPParameter(Session session) {
+    private static String createClientIPParameter(Session session) {
         return ("127.0.0.1".equals(session.getClientIp()) ? "" : ("&clientIP=" + session.getClientIp()));
     }
 
@@ -313,7 +314,9 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         try {
             executeMethod(data.getHttpClient(), method);
         } catch (HttpException ignored) {
+            // do nothing on failed logout
         } catch (IOException ignored) {
+            // do nothing on failed logout
         } finally {
             method.releaseConnection();
         }
@@ -356,18 +359,26 @@ public class OXJSONAccessImpl implements OXJSONAccess {
     }
 
     public JSONResult doPut(String path, String action, Session session, Map<String, String> parameters, byte[] bytes) throws AuthenticationFailedException, OXCommunicationException {
+        return doPut(path, action, session, parameters, "byte array", new ByteArrayRequestEntity(bytes));
+    }
+
+    public JSONResult doPut(String path, String action, Session session, Map<String, String> parameters, InputStreamProvider streamProvider) throws AuthenticationFailedException, OXCommunicationException {
+        return doPut(path, action, session, parameters, "InputStream", new InputStreamProviderRequestEntity(streamProvider));
+    }
+
+    private JSONResult doPut(String path, String action, Session session, Map<String, String> parameters, String type, RequestEntity entity) throws AuthenticationFailedException, OXCommunicationException {
         OXConnectionData data = obtainConnectionData(session);
         String url = getBaseURL() + path;
         if (_journal.isDebugEnabled())
-            _journal.debug(session + " PUT " + url + " with byte array");
+            _journal.debug(session + " PUT " + url + " with " + type);
         PutMethod method = new PutMethod(url);
         method.setQueryString(generateQueryParameters(action, data.getSessionID(), parameters));
-        method.setRequestEntity(new ByteArrayRequestEntity(bytes));
+        method.setRequestEntity(entity);
         return executeMethodAndExtractResult(session, data, method, action, parameters);
     }
 
-    public void activate(Log log, String oxJSONURL, String postResultPattern, int maxConnections, int connectionTimeout, boolean performStaleChecking) {
-        String error = doActivate(log, oxJSONURL, postResultPattern, maxConnections, connectionTimeout, performStaleChecking);
+    public void activate(Log log, String oxJSONURL, int maxConnections, int connectionTimeout, boolean performStaleChecking, boolean passHostHeaderFromClient) {
+        String error = doActivate(log, oxJSONURL, maxConnections, connectionTimeout, performStaleChecking, passHostHeaderFromClient);
         if (error != null) {
             error = "OXJSONAccess not activated, reason: " + error;
             log.error(error);
@@ -380,21 +391,15 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         deactivate("USM OXJSONAccess deactivated");
     }
 
-    private String doActivate(Log log, String oxJSONURL, String postResultPattern, int maxConnections, int connectionTimeout, boolean performStaleChecking) {
+    private String doActivate(Log log, String oxJSONURL, int maxConnections, int connectionTimeout, boolean performStaleChecking, boolean passHostHeaderFromClient) {
         _journal = log;
         _oxJSONBaseURLasString = oxJSONURL;
+        _passClientHostHeader = passHostHeaderFromClient;
         try {
             _oxJSONBaseURL = getURLFromString(oxJSONURL);
         } catch (URIException e) {
             return "OXJSONAccess base URL is no valid http(s) URL: " + _oxJSONBaseURLasString;
         }
-        _postResultPattern = postResultPattern.trim();
-        _actionIndex = _postResultPattern.indexOf(ACTION_KEY);
-        if (_actionIndex < 0)
-            return ACTION_KEY + " not found in post result pattern";
-        _jsonIndex = _postResultPattern.indexOf(JSON_KEY);
-        if (_jsonIndex < 0)
-            return JSON_KEY + " not found in post result pattern";
         _connectionTimeout = connectionTimeout;
         _httpManager = new MultiThreadedHttpConnectionManager();
         HttpConnectionManagerParams httpParams = _httpManager.getParams();
@@ -408,8 +413,8 @@ public class OXJSONAccessImpl implements OXJSONAccess {
             public void run() {
                 try {
                     while (!isInterrupted()) {
-                        Thread.sleep(_connectionTimeout);
-                        _httpManager.closeIdleConnections(_connectionTimeout);
+                        Thread.sleep(getConnectionTimeout());
+                        getHttpConnectionManager().closeIdleConnections(getConnectionTimeout());
                     }
                 } catch (InterruptedException ignored) {
                     // We get interrupted to indicate that this thread has to stop, so simply stop
@@ -423,7 +428,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         return null;
     }
 
-    private HttpURL getURLFromString(String urlString) throws URIException {
+    private static HttpURL getURLFromString(String urlString) throws URIException {
         return (urlString.toLowerCase().startsWith(HTTPS_PREFIX)) ? new HttpsURL(urlString, CHARSET_UTF8) : new HttpURL(
             urlString,
             CHARSET_UTF8);
@@ -435,7 +440,6 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         _backgroundConnectionCloser = null;
         _oxJSONBaseURL = null;
         _oxJSONBaseURLasString = null;
-        _postResultPattern = null;
         if (message != null)
             _journal.info(message);
         _journal = null;
@@ -444,15 +448,20 @@ public class OXJSONAccessImpl implements OXJSONAccess {
     private OXConnectionData obtainConnectionData(Session session) throws AuthenticationFailedException, OXCommunicationException {
         obtainRequiredLock(session, "<obtain OX connection data>");
         try {
-            OXConnectionInformation o = session.getOXConnectionData();
+            OXConnectionData o = getCurrentOXConnectionData(session);
             _journal.debug(session + " Retrieving OX session with ID: " + o);
-            return (o instanceof OXConnectionData) ? ((OXConnectionData) o) : performLogin(session);
+            return (o == null) ? performLogin(session) : o;
         } finally {
             releaseLock(session);
         }
     }
 
-    private NameValuePair[] generateQueryParameters(String action, String sessionID, Map<String, String> params) {
+    private static OXConnectionData getCurrentOXConnectionData(Session session) {
+        OXConnectionInformation o = session.getOXConnectionData();
+        return (o instanceof OXConnectionData) ? ((OXConnectionData) o) : null;
+    }
+
+    private static NameValuePair[] generateQueryParameters(String action, String sessionID, Map<String, String> params) {
         List<NameValuePair> list = new ArrayList<NameValuePair>();
         if (action != null)
             list.add(new NameValuePair("action", action));
@@ -470,7 +479,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         return _oxJSONBaseURLasString;
     }
 
-    private String urlEncode(String s) {
+    private static String urlEncode(String s) {
         try {
             return URLEncoder.encode(s, CHARSET_UTF8);
         } catch (UnsupportedEncodingException e) {
@@ -542,23 +551,23 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         }
     }
 
-    private long getSessionProperty(Session session, String name) {
+    private static long getSessionProperty(Session session, String name) {
         Object o = session.getCustomProperty(name);
         return (o instanceof Long) ? ((Long) o).longValue() : 0L;
     }
 
-    private boolean isSynchronizationConflictError(JSONResult result) {
+    private static boolean isSynchronizationConflictError(JSONResult result) {
         return result.getResultType() == JSONResultType.Error && OXJSONSupport.ERROR_CODE_OBJECT_CHANGED.equals(OXJSONSupport.getJSONErrorCode(result.getJSONObject()));
     }
 
-    private boolean shouldPerformNewLogin(JSONResult result) {
+    private static boolean shouldPerformNewLogin(JSONResult result) {
         if (result.getResultType() != JSONResultType.Error)
             return false;
         String errorCode = OXJSONSupport.getJSONErrorCode(result.getJSONObject());
         return OXJSONSupport.ERROR_CODE_UNKNOWN_SESSION.equals(errorCode) || OXJSONSupport.ERROR_CODE_EXPIRED_SESSION.equals(errorCode);
     }
 
-    private boolean shouldRetryRequest(JSONResult result) {
+    private static boolean shouldRetryRequest(JSONResult result) {
         if (result.getResultType() != JSONResultType.Error)
             return false;
         String errorCategory = OXJSONSupport.getJSONErrorCategory(result.getJSONObject());
@@ -567,8 +576,9 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         || OXJSONSupport.CATEGORY_SOCKET_CONNECTION_CORRUPT.equals(errorCategory) || OXJSONSupport.CATEGORY_SUBSYSTEM_DOWN.equals(errorCategory);
     }
 
-    public OXResource getResource(Session session, String path, Map<String, String> parameters) throws AuthenticationFailedException, OXCommunicationException {
+    private ResourceInputStreamImpl accessResource(Session session, String path, Map<String, String> parameters) throws AuthenticationFailedException, OXCommunicationException {
         GetMethod method = null;
+        boolean freeMethod = true;
         try {
             if (_journal.isDebugEnabled())
                 _journal.debug(session + " GET " + _oxJSONBaseURLasString + " + " + path);
@@ -596,28 +606,27 @@ public class OXJSONAccessImpl implements OXJSONAccess {
                     OXJSONErrorCode.ACCESSIMPL_NO_CONTENT_TYPE_RECEIVED,
                     "No content type in response header for path: " + method.getURI().getPathQuery());
             }
-            String contentType = responseHeader.getValue();
-            if (APPLICATION_OCTET_STREAM.equals(contentType) || APPLICATION_OCTET_STREAM_UTF_8.equals(contentType))
-                return new OXResource(contentType, extractResponseBody(method));
-            else if (contentType != null && contentType.startsWith(IMAGE))
-                return new OXResource(contentType, extractResponseBody(method));
-            else if (contentType != null && contentType.startsWith(TEXT_HTML)) {
-                String httpResult = extractResponseBodyAsString(method);
-                int startIndex = httpResult.indexOf("({");
-                int endIndex = httpResult.lastIndexOf("})");
-                JSONResult json = new JSONErrorResult(new JSONObject());
-                if (startIndex > 0 && endIndex > startIndex) {
-                    json = checkPostResult(httpResult.substring(startIndex + 1, endIndex + 1));
-                }
-                throw new OXCommunicationException(
-                    OXJSONErrorCode.ACCESSIMPL_ERROR_ON_READING_RESOURCE,
-                    "Error on getting resource from Server",
-                    json.getJSONObject());
-            } else {
+            String contentType = responseHeader.getValue().toLowerCase();
+            if (isValidResourceContentType(contentType)) {
+                freeMethod = false;
+                ResourceInputStreamImpl ris = new ResourceInputStreamImpl(this, session, connectionData, method, contentType);
+                return ris;
+            }
+            if (contentType == null || !contentType.startsWith(TEXT_HTML))
                 throw new OXCommunicationException(
                     OXJSONErrorCode.ACCESSIMPL_INVALID_CONTENT_TYPE_RECEIVED,
                     "Invalid content type in response: " + contentType);
+            String httpResult = extractResponseBodyAsString(method);
+            int startIndex = httpResult.indexOf("({");
+            int endIndex = httpResult.lastIndexOf("})");
+            JSONResult json = new JSONErrorResult(new JSONObject());
+            if (startIndex > 0 && endIndex > startIndex) {
+                json = checkPostResult(httpResult.substring(startIndex + 1, endIndex + 1));
             }
+            throw new OXCommunicationException(
+                OXJSONErrorCode.ACCESSIMPL_ERROR_ON_READING_RESOURCE,
+                "Error on getting resource from Server",
+                json.getJSONObject());
         } catch (URIException e) {
             throw new OXCommunicationException(OXJSONErrorCode.ACCESSIMPL_INVALID_URL_NUMBER_16, "Invalid URL specified: " + path, e);
         } catch (HttpException e) {
@@ -625,16 +634,68 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         } catch (IOException e) {
             throw new OXCommunicationException(OXJSONErrorCode.ACCESSIMPL_GETRESOURCE_ERROR_NUMBER_18, e);
         } finally {
-            if (method != null)
+            if (freeMethod && method != null)
                 method.releaseConnection();
         }
     }
 
+    public OXResource getResource(Session session, String path, Map<String, String> parameters) throws AuthenticationFailedException, OXCommunicationException {
+        obtainRequiredLock(session, "GET " + path + ' ' + parameters);
+        try {
+            ResourceInputStreamImpl data = accessResource(session, path, parameters);
+            try {
+                return new OXResource(data.getContentType(), extractResponseBody(data.getMethod()));
+            } catch (IOException e) {
+                throw new OXCommunicationException(OXJSONErrorCode.ACCESSIMPL_CAN_NOT_READ_RESOURCE, e);
+            } finally {
+                data.getMethod().releaseConnection();
+            }
+        } finally {
+            releaseLock(session);
+        }
+    }
+
+    public ResourceInputStream getResourceAsStream(Session session, String path, Map<String, String> parameters) throws AuthenticationFailedException, OXCommunicationException {
+        obtainRequiredLock(session, "(as stream) GET " + path + ' ' + parameters);
+        try {
+            return accessResource(session, path, parameters);
+        } catch (AuthenticationFailedException e) {
+            releaseLock(session);
+            throw e;
+        } catch (OXCommunicationException e) {
+            releaseLock(session);
+            throw e;
+        }
+    }
+
     public JSONResult storeResource(String path, String action, Session session, Map<String, String> parameters, JSONObject jsonData, byte[] imageByteData, String fileContentType) throws AuthenticationFailedException, OXCommunicationException {
         return storeResource(path, action, session, parameters, jsonData, imageByteData, fileContentType, STRING_PART_NAME, FILE_PART_NAME);
     }
 
     public JSONResult storeResource(String path, String action, Session session, Map<String, String> parameters, JSONObject jsonData, byte[] imageByteData, String fileContentType, String jsonPartName, String filePartName) throws AuthenticationFailedException, OXCommunicationException {
+        ByteArrayPartSource source = new ByteArrayPartSource(filePartName, imageByteData);
+        return storeResource(path, action, session, parameters, jsonData, fileContentType, jsonPartName, filePartName, source);
+    }
+
+    public JSONResult storeResourceFromStream(String path, String action, Session session, Map<String, String> parameters, JSONObject jsonData, String fileContentType, String jsonPartName, final String filePartName, final long size, final InputStreamProvider streamProvider) throws AuthenticationFailedException, OXCommunicationException {
+        PartSource source = new PartSource() {
+
+            public long getLength() {
+                return size;
+            }
+
+            public String getFileName() {
+                return filePartName;
+            }
+
+            public InputStream createInputStream() throws IOException {
+                return streamProvider.getInputStream();
+            }
+        };
+        return storeResource(path, action, session, parameters, jsonData, fileContentType, jsonPartName, filePartName, source);
+    }
+
+    private JSONResult storeResource(String path, String action, Session session, Map<String, String> parameters, JSONObject jsonData, String fileContentType, String jsonPartName, String filePartName, PartSource dataSource) throws AuthenticationFailedException, OXCommunicationException {
         PostMethod multiPartPostMethod = null;
         try {
             OXConnectionData connectionData = obtainConnectionData(session);
@@ -655,11 +716,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
                 parts.add(jsonPart);
             }
 
-            FilePart filePart = new FilePart(
-                filePartName,
-                new ByteArrayPartSource(filePartName, imageByteData),
-                fileContentType,
-                BINARY_ENCODING);
+            FilePart filePart = new FilePart(filePartName, dataSource, fileContentType, BINARY_ENCODING);
             filePart.setTransferEncoding(BINARY_ENCODING);
             parts.add(filePart);
 
@@ -684,7 +741,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         }
     }
 
-    private JSONResult checkAndExtractResult(String httpResult) throws OXCommunicationException {
+    private static JSONResult checkAndExtractResult(String httpResult) throws OXCommunicationException {
         httpResult = httpResult.trim();
         int startIndex = httpResult.indexOf("({");
         int endIndex = httpResult.lastIndexOf("})");
@@ -698,7 +755,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
             httpResult);
     }
 
-    private JSONResult checkPostResult(String json) throws OXCommunicationException {
+    private static JSONResult checkPostResult(String json) throws OXCommunicationException {
         try {
             return OXJSONSupport.generateJSONResultForJSONObject(new JSONObject(json), JSONResultType.JSONObject);
         } catch (JSONException e) {
@@ -709,7 +766,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         }
     }
 
-    private byte[] extractResponseBody(HttpMethod method) throws IOException {
+    private static byte[] extractResponseBody(HttpMethod method) throws IOException {
         ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream(RESOURCE_BUFFER_DEFAULT_SIZE);
         InputStream inputStream = method.getResponseBodyAsStream();
         byte[] data = new byte[4096];
@@ -719,7 +776,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         return byteArrayStream.toByteArray();
     }
 
-    private String extractResponseBodyAsString(HttpMethodBase method) throws IOException {
+    private static String extractResponseBodyAsString(HttpMethodBase method) throws IOException {
         return new String(extractResponseBody(method), method.getResponseCharSet());
     }
 
@@ -758,7 +815,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         return status;
     }
 
-    private Header getRedirection(HttpMethod method, int status) {
+    private static Header getRedirection(HttpMethod method, int status) {
         return (status == HttpStatus.SC_MOVED_PERMANENTLY || status == HttpStatus.SC_SEE_OTHER || status == HttpStatus.SC_TEMPORARY_REDIRECT) ? method.getResponseHeader("location") : null;
     }
 
@@ -799,23 +856,71 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         }
         throw new OXCommunicationException(
             OXJSONErrorCode.FAILED_TO_GAIN_EXCLUSIVE_OX_ACCESS,
-            "Timeout while waiting for exclusive access to OX server, locked by " + session + '/' + session.getCustomProperty(OX_REQUEST_INFORMATION_FIELD),
+            "Timeout while waiting for exclusive access to OX server, locked by " + session + '/' + getRequestInformation(session),
             (String) null);
     }
 
     private boolean tryLock(Session session, String requestInformation) throws InterruptedException {
-        boolean success = session.getOXConnectionLock().tryLock(LOCK_TIMEOUT, TimeUnit.SECONDS);
+        for (int i = 0; i < LOCK_TIMEOUT; i += 10) {
+            if (tryLock(session, requestInformation, 10))
+                return true;
+        }
+        return false;
+    }
+
+    private boolean tryLock(Session session, String requestInformation, long timeout) throws InterruptedException {
+        OXConnectionData data = getCurrentOXConnectionData(session);
+        if (data != null) {
+            ResourceInputStreamImpl currentRIS = data.getCurrentResourceInputStream();
+            if (currentRIS != null && currentRIS.getLastActivity() < System.currentTimeMillis() - _STREAM_INACTIVITY_TIMEOUT) {
+                _journal.info("Closing ResourceInputStream due to long inactivity (was locked by " + getRequestInformation(session) + ')');
+                try {
+                    currentRIS.close();
+                } catch (IOException ignored) {
+                    // do not log error on auto-close, other request that took too long will log an error due to the closed InputStream
+                }
+            }
+        }
+        boolean success = session.getOXConnectionLock().tryLock(timeout, TimeUnit.SECONDS);
         if (success)
             setRequestInformation(session, requestInformation);
         return success;
     }
 
-    private void releaseLock(Session session) {
+    public void releaseLock(Session session) {
         setRequestInformation(session, null);
-        session.getOXConnectionLock().unlock();
+        try {
+            // Probably not necessary, just to be sure to avoid a deadlock in case one of the access methods fails to cleanup properly
+            session.getOXConnectionLock().unlock();
+        } catch (IllegalMonitorStateException ignored) {
+            // If the lock was acquired in another thread, the unlock() call will fail.
+            // This can only happen if OX was accessed using one of the streaming methods.
+            // Since in that case we additionally close the ResourceInputStream, that other
+            // thread will get notified and release the lock itself.
+        }
     }
 
-    private void setRequestInformation(Session session, String requestInformation) {
+    private static void setRequestInformation(Session session, String requestInformation) {
         session.setCustomProperty(OX_REQUEST_INFORMATION_FIELD, requestInformation);
     }
+
+    private static Object getRequestInformation(Session session) {
+        return session.getCustomProperty(OX_REQUEST_INFORMATION_FIELD);
+    }
+
+    private static boolean isValidResourceContentType(String contentType) {
+        return (contentType != null) && (APPLICATION_OCTET_STREAM.equals(contentType) || APPLICATION_OCTET_STREAM_UTF_8.equals(contentType) || contentType.startsWith(IMAGE));
+    }
+
+    public MultiThreadedHttpConnectionManager getHttpConnectionManager() {
+        return _httpManager;
+    }
+
+    public int getConnectionTimeout() {
+        return _connectionTimeout;
+    }
+
+    public MultiThreadedHttpConnectionManager getHttpManager() {
+        return _httpManager;
+    }
 }
