@@ -52,11 +52,9 @@ import java.io.Serializable;
 import java.util.BitSet;
 import java.util.Map;
 import java.util.UUID;
-
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
-
 import com.openexchange.usm.api.USMDebugConstants;
 import com.openexchange.usm.api.contenttypes.ContentType;
 import com.openexchange.usm.api.contenttypes.ContentTypeField;
@@ -490,7 +488,7 @@ public class SimpleDataObject implements DataObject {
 		result[1] = _source.getUUID();
 		for (int i = 0; i < _fieldContent.length; i++) {
 			DataType<?> fieldType = fields[i].getFieldType();
-			result[i + 2] = fieldType.createCopy(_fieldContent[i]);
+			result[i + 2] = fieldType.serialize(_fieldContent[i]);
 		}
 		return result;
 	}
