@@ -752,6 +752,32 @@ public class PersistentSessionData implements Serializable {
     }
 
     private void insertOrUpdateDBField(String key, String value) throws DatabaseAccessException, USMSQLException {
+        if (_session.getSessionManager().getDatabaseAccess().supportsOnDuplicateKeyUpdateStatement()) {
+            EncapsulatedConnection con = _session.getWritableDBConnection();
+            PreparedStatement statement = null;
+            try {
+                statement = con.prepareStatement("insert into USMSessionFields (cid, usmSessionId, Field, Value) values(?,?,?,?) on duplicate key update value = ?");
+                addSessionIdenifierParamsToStatement(statement);
+                statement.setString(NUMBER_OF_SESSION_ID_FIELDS + 1, key);
+                statement.setString(NUMBER_OF_SESSION_ID_FIELDS + 2, value);
+                statement.setString(NUMBER_OF_SESSION_ID_FIELDS + 3, value);
+                if (statement.executeUpdate() == 0)
+                    _session.getSessionManager().getJournal().warn(_session + " Setting field " + key + " in DB did not change any rows");
+            } catch (SQLException e) {
+                String errorMessage = _session + " SQL error while updating field " + key;
+                _session.getSessionManager().getJournal().error(errorMessage, e);
+                throw new USMSQLException(USMSessionManagementErrorCodes.PERSISTENT_SESSION_ERROR_NUMBER3, errorMessage, e);
+            } finally {
+                if (con != null) {
+                    Toolkit.close(statement);
+                    Toolkit.close(con);
+                }
+            }
+        } else
+            insertOrUpdateDBFieldInMoreSteps(key, value);
+    }
+
+    private void insertOrUpdateDBFieldInMoreSteps(String key, String value) throws DatabaseAccessException, USMSQLException {
         EncapsulatedConnection con = null;
         PreparedStatement statement = null;
         try {
