@@ -57,6 +57,8 @@ import java.util.Comparator;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.PriorityQueue;
+import java.util.Queue;
 import java.util.TimeZone;
 import java.util.UUID;
 import java.util.concurrent.ConcurrentHashMap;
@@ -82,6 +84,7 @@ import com.openexchange.usm.api.exceptions.TimestampMismatchException;
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.exceptions.USMResourceAccessException;
 import com.openexchange.usm.api.exceptions.USMSQLException;
+import com.openexchange.usm.api.session.CompleteSessionID;
 import com.openexchange.usm.api.session.ConflictResolution;
 import com.openexchange.usm.api.session.DataObject;
 import com.openexchange.usm.api.session.DataObjectFilter;
@@ -90,8 +93,10 @@ import com.openexchange.usm.api.session.OXConnectionInformation;
 import com.openexchange.usm.api.session.ObjectChanges;
 import com.openexchange.usm.api.session.Session;
 import com.openexchange.usm.api.session.SessionChangeListener;
+import com.openexchange.usm.api.session.SessionID;
 import com.openexchange.usm.api.session.SessionInitializer;
 import com.openexchange.usm.api.session.SyncResult;
+import com.openexchange.usm.api.session.UserID;
 import com.openexchange.usm.mapping.FolderIdMapping;
 import com.openexchange.usm.ox_json.JSONResult;
 import com.openexchange.usm.ox_json.JSONResultType;
@@ -135,14 +140,18 @@ public class SessionImpl implements Session, Serializable {
         0.75f,
         1);
 
+    private final Queue<CustomPropertyLimit> _customPropertyLimits = new PriorityQueue<CustomPropertyLimit>();
+
     private final Object _waitObject = new Object(); // Used to wait on folder changes and to notify about folder changes
 
-    private final DataObjectStorage _dataObjectCache;
+    private final SyncStateStorage _syncStateStorage;
 
     private final OXDataCacheID _folderHierarchyCacheID;
 
     private final CompleteSessionID _completeSessionID;
 
+    private final SessionInitializer _sessionInitializer;
+    
     private int _uniqueSessionID;
 
     private String _defaultInboxId;
@@ -183,16 +192,17 @@ public class SessionImpl implements Session, Serializable {
 
     private Map<String, String> _xHeaders = new HashMap<String, String>();
 
-    public SessionImpl(SessionManagerImpl sessionManager, String user, String password, String protocol, String device) {
+    public SessionImpl(SessionManagerImpl sessionManager, String user, String password, String protocol, String device, SessionInitializer initializer) {
         _sessionManager = sessionManager;
         _password = password;
         _completeSessionID = new CompleteSessionID(new SessionID(user, protocol, device), new UserID(-1, -1), null);
         _persistentData = new PersistentSessionData(this);
-        _dataObjectCache = new DataObjectStorage(this);
+        _syncStateStorage = new SyncStateStorage(this);
         _folderHierarchyCacheID = new OXDataCacheID(this);
+        _sessionInitializer = initializer;
     }
 
-    public void initialize(SessionInitializer initializer, JSONObject configuration) throws USMException {
+    public void initialize(JSONObject configuration) throws USMException {
         int contextID;
         int userIdentifier;
         try {
@@ -216,7 +226,7 @@ public class SessionImpl implements Session, Serializable {
         _completeSessionID.setUserID(new UserID(contextID, userIdentifier));
         _defaultInboxId = JSONToolkit.getString(configuration, "modules", "mail", "defaultFolder", "inbox");
         _defaultAddress = JSONToolkit.getString(configuration, "modules", "mail", "defaultaddress");
-        _persistentData.initialize(initializer, configuration);
+        _persistentData.initialize(_sessionInitializer, configuration);
     }
 
     private USMException generateInitializationException(String fieldName, Throwable cause) {
@@ -399,10 +409,12 @@ public class SessionImpl implements Session, Serializable {
     }
 
     public SyncResult syncFolderChangesWithServer(long timestamp, int resultLimit, DataObjectFilter filter, boolean storeResult, ConflictResolution conflictResolution, Folder... folderChanges) throws USMException {
+        Folder[] cachedFolders = getCachedFolders(timestamp);
+        removeUnusedFoldersFromSyncStateStorage(cachedFolders);
         SyncResult result = _sessionManager.getIncrementalSyncer().syncChangesWithServer(
             createFolderHierachyStorage(filter, storeResult, timestamp),
             getConflictResolution(conflictResolution),
-            getCachedFolders(timestamp),
+            cachedFolders,
             folderChanges,
             timestamp,
             resultLimit,
@@ -452,11 +464,11 @@ public class SessionImpl implements Session, Serializable {
     }
 
     public Folder[] getCachedFolders() throws DatabaseAccessException, USMSQLException {
-        return getCachedFolders(_dataObjectCache.getNewestTimestamp(FOLDER_HIERARCHY_ID));
+        return getCachedFolders(_syncStateStorage.getNewestTimestamp(FOLDER_HIERARCHY_ID));
     }
 
     public Folder[] getCachedFolders(long timestamp) throws DatabaseAccessException, USMSQLException {
-        return getCachedFolders(_dataObjectCache.get(FOLDER_HIERARCHY_ID, timestamp), timestamp);
+        return getCachedFolders(_syncStateStorage.get(FOLDER_HIERARCHY_ID, timestamp), timestamp);
     }
 
     private Folder[] getCachedFolders(Serializable[][] values, long timestamp) {
@@ -478,15 +490,15 @@ public class SessionImpl implements Session, Serializable {
     }
 
     public Folder getCachedFolder(String folderID, long timestamp) throws DatabaseAccessException, USMSQLException {
-        return getCachedFolder(_dataObjectCache.get(FOLDER_HIERARCHY_ID, timestamp), folderID, timestamp);
+        return getCachedFolder(_syncStateStorage.get(FOLDER_HIERARCHY_ID, timestamp), folderID, timestamp);
     }
 
     public Folder getCachedFolder(String folderID) throws DatabaseAccessException, USMSQLException {
         Folder f = _sessionManager.getSpecialFolder(this, folderID);
         if (f != null)
             return f;
-        long newestTS = _dataObjectCache.getNewestTimestamp(FOLDER_HIERARCHY_ID);
-        return getCachedFolder(_dataObjectCache.get(FOLDER_HIERARCHY_ID, newestTS), folderID, newestTS);
+        long newestTS = _syncStateStorage.getNewestTimestamp(FOLDER_HIERARCHY_ID);
+        return getCachedFolder(_syncStateStorage.get(FOLDER_HIERARCHY_ID, newestTS), folderID, newestTS);
     }
 
     private Folder getCachedFolder(Serializable[][] values, String folderID, long timestamp) {
@@ -512,13 +524,13 @@ public class SessionImpl implements Session, Serializable {
     }
 
     public DataObject[] getCachedFolderElements(String folderId, ContentType contentType, long timestamp) throws DatabaseAccessException, USMSQLException {
-        return getCachedFolderElements(folderId, contentType, timestamp, _dataObjectCache.get(getShortFolderID(folderId), timestamp));
+        return getCachedFolderElements(folderId, contentType, timestamp, _syncStateStorage.get(getShortFolderID(folderId), timestamp));
     }
 
     public DataObject[] getCachedFolderElements(String folderId, ContentType contentType) throws DatabaseAccessException, USMSQLException {
         String shortID = getShortFolderID(folderId);
-        long newestTS = _dataObjectCache.getNewestTimestamp(shortID);
-        return getCachedFolderElements(folderId, contentType, newestTS, _dataObjectCache.get(shortID, newestTS));
+        long newestTS = _syncStateStorage.getNewestTimestamp(shortID);
+        return getCachedFolderElements(folderId, contentType, newestTS, _syncStateStorage.get(shortID, newestTS));
     }
 
     private DataObject[] getCachedFolderElements(String folderId, ContentType contentType, long timestamp, Serializable[][] values) {
@@ -550,14 +562,14 @@ public class SessionImpl implements Session, Serializable {
             folderId,
             contentType,
             objectID,
-            _dataObjectCache.get(getShortFolderID(folderId), timestamp),
+            _syncStateStorage.get(getShortFolderID(folderId), timestamp),
             timestamp);
     }
 
     public DataObject getCachedFolderElement(String folderId, ContentType contentType, String objectID) throws DatabaseAccessException, USMSQLException {
         String shortID = getShortFolderID(folderId);
-        long ts = _dataObjectCache.getNewestTimestamp(shortID);
-        return getCachedFolderElement(folderId, contentType, objectID, _dataObjectCache.get(shortID, ts), ts);
+        long ts = _syncStateStorage.getNewestTimestamp(shortID);
+        return getCachedFolderElement(folderId, contentType, objectID, _syncStateStorage.get(shortID, ts), ts);
     }
 
     private DataObject getCachedFolderElement(String folderId, ContentType contentType, String objectID, Serializable[][] values, long timestamp) {
@@ -581,22 +593,11 @@ public class SessionImpl implements Session, Serializable {
         long result = internalStoreCachedDataObjects(FOLDER_HIERARCHY_ID, oldTimestamp, timestamp, objects);
         if (!updatesPending)
             _lastFolderSync = result;
-
-        List<String> shortIDs = new ArrayList<String>();
-        for (DataObject o : objects) {
-            String id = o.getID();
-            shortIDs.add(getShortFolderID(id));
-        }
- 
-        shortIDs.add(FOLDER_HIERARCHY_ID);
-        for (String specialFolder : _sessionManager.getSpecialFolders())
-            shortIDs.add(getShortFolderID(specialFolder));
-        _dataObjectCache.retain(shortIDs.toArray(new String[shortIDs.size()]));
-         //the short ids mapping should not be removed for non existing folders - because a deletion should be reported to the client first with the correct short id (see bug 26252)
         return result;
     }
-
-    public long storeCachedDataObjects(String folderID, long oldTimestamp, long timestamp, DataObjectSet objects, boolean updatesPending) throws DatabaseAccessException, USMSQLException {
+    
+  
+    public long storeCachedDataObjects(String folderID, long oldTimestamp, long timestamp, DataObjectSet objects,  boolean updatesPending) throws DatabaseAccessException, USMSQLException {
         long result = internalStoreCachedDataObjects(getShortFolderID(folderID), oldTimestamp, timestamp, objects);
         if (updatesPending)
             _syncedFolderContent.remove(folderID);
@@ -608,7 +609,7 @@ public class SessionImpl implements Session, Serializable {
     private long internalStoreCachedDataObjects(String folderID, long oldTimestamp, long timestamp, DataObjectSet objects) throws DatabaseAccessException, USMSQLException {
         if (!_sessionManager.isKeepLastSentSyncStateInDB())
             oldTimestamp = 0L;
-        long resultingTimestamp = _dataObjectCache.put(folderID, timestamp, objects, oldTimestamp, this);
+        long resultingTimestamp = _syncStateStorage.put(folderID, timestamp, objects, oldTimestamp, this);
         if (resultingTimestamp != timestamp) {
             for (DataObject o : objects) {
                 o.setTimestamp(resultingTimestamp);
@@ -896,14 +897,16 @@ public class SessionImpl implements Session, Serializable {
             }
         }
         for (String id : folderIds) {
-            journal.debug("Removing folder from sync folder content, " + id);
-            _syncedFolderContent.remove(id);
-            _sessionManager.getOXDataCache().removeCachedData(new OXDataCacheID(this, id));
-            if ("1".equals(id)) { // for Remote Events when folderID == 1, it means that the default inbox has been changed
-                if (journal.isDebugEnabled())
-                    journal.debug("Removing default inbox from sync folder content, " + _defaultInboxId);
-                _syncedFolderContent.remove(_defaultInboxId);
-                _sessionManager.getOXDataCache().removeCachedData(new OXDataCacheID(this, _defaultInboxId));
+            if (null != id) {
+             journal.debug("Removing folder from sync folder content, " + id);
+             _syncedFolderContent.remove(id);
+             _sessionManager.getOXDataCache().removeCachedData(new OXDataCacheID(this, id));
+             if ("1".equals(id)) { // for Remote Events when folderID == 1, it means that the default inbox has been changed
+                 if (journal.isDebugEnabled())
+                     journal.debug("Removing default inbox from sync folder content, " + _defaultInboxId);
+                 _syncedFolderContent.remove(_defaultInboxId);
+                 _sessionManager.getOXDataCache().removeCachedData(new OXDataCacheID(this, _defaultInboxId));
+             }
             }
         }
         synchronized (_waitObject) {
@@ -944,12 +947,8 @@ public class SessionImpl implements Session, Serializable {
         synchronized (_waitObject) {
             result = extractFolderChanges(watchFolderChanges, contentChangeParentIDsToWatch);
             if (result != null) {
-                // TODO Make "multiple ping mark as synchronized" workaround configurable, let protocol configure it, remove reference to
-                // EAS here
-                if (++_directPingResponses > DIRECT_PING_RESPONSE_LIMIT && "EAS".equals(getProtocol())) { // fix for bug 22741 - enable the
-                                                                                                          // ping
-                    // response limit ONLY for EAS
-                    // protocol
+                if (!_sessionInitializer.allowsUnlimitedWaitForChangesOnModifiedFolders() && ++_directPingResponses > DIRECT_PING_RESPONSE_LIMIT && "EAS".equals(getProtocol())) {
+                    // fix for bug 22741 - enable the ping response limit ONLY for protocols that require it (e.g. EAS)
                     if (journal.isDebugEnabled())
                         _sessionManager.getJournal().debug(
                             this + " Too many pings without sync, marked as synchronized: " + Arrays.toString(contentChangeParentIDsToWatch));
@@ -1053,7 +1052,33 @@ public class SessionImpl implements Session, Serializable {
     }
 
     public Object setCustomProperty(String key, Object value) {
-        return (value == null) ? _customProperties.remove(key) : _customProperties.put(key, value);
+        synchronized (_customPropertyLimits) {
+            _customPropertyLimits.remove(new CustomPropertyLimit(key));
+        }
+        Object oldValue = (value == null) ? _customProperties.remove(key) : _customProperties.put(key, value);
+        return oldValue;
+    }
+
+    public Object setCustomProperty(String key, Object value, long timeout) {
+        synchronized (_customPropertyLimits) {
+            Object oldValue = (value == null) ? _customProperties.remove(key) : _customProperties.put(key, value);
+            if (oldValue != null)
+                _customPropertyLimits.remove(new CustomPropertyLimit(key));
+            _customPropertyLimits.add(new CustomPropertyLimit(key, timeout));
+            return oldValue;
+        }
+    }
+
+    public int removeOutdatedCustomProperties(long now) {
+        int count = 0;
+        synchronized (_customPropertyLimits) {
+            for (CustomPropertyLimit limit = _customPropertyLimits.peek(); limit != null && limit.getStorageLimit() < now; limit = _customPropertyLimits.peek()) {
+                if (_customProperties.remove(limit.getKey()) != null)
+                    count++;
+                _customPropertyLimits.poll();
+            }
+        }
+        return count;
     }
 
     public DataObjectFilter getSyncServerObjectFilter() {
@@ -1064,53 +1089,53 @@ public class SessionImpl implements Session, Serializable {
         _syncServerObjectFilter = filter;
     }
 
-    public static void main(String[] args) {
-        Thread[] writeThreads = new Thread[100];
-        final SessionImpl mySession = new SessionImpl(null, "user", "password", "protocol", "device");
-        for (int i = 0; i < writeThreads.length; i++) {
-            writeThreads[i] = new Thread("Thread " + i) {
-
-                @Override
-                public void run() {
-                    try {
-                        for (;;) {
-                            mySession.foldersChanged(0L);
-                            Thread.sleep(100L);
-                            mySession.folderContentChanged(0L, getName());
-                            Thread.sleep(100L);
-                        }
-                    } catch (InterruptedException ignored) {
-                        // exit on interrupt
-                    }
-                }
-            };
-            writeThreads[i].start();
-        }
-        Thread readThread = new Thread("Reader") {
-
-            @Override
-            public void run() {
-                try {
-                    for (;;) {
-                        ObjectChanges result = mySession.waitForChanges(true, null, 1000L);
-                        System.out.println("Result: " + result);
-                    }
-                } catch (USMException e) {
-                    e.printStackTrace();
-                } catch (InterruptedException e) {
-                    e.printStackTrace();
-                }
-            }
-        };
-        readThread.start();
-        // try {
-        // Thread.sleep(10000L);
-        // } catch (InterruptedException e) {
-        // }
-        // for (Thread t : writeThreads)
-        // t.interrupt();
-        // readThread.interrupt();
-    }
+//    public static void main(String[] args) {
+//        Thread[] writeThreads = new Thread[100];
+//        final SessionImpl mySession = new SessionImpl(null, "user", "password", "protocol", "device");
+//        for (int i = 0; i < writeThreads.length; i++) {
+//            writeThreads[i] = new Thread("Thread " + i) {
+//
+//                @Override
+//                public void run() {
+//                    try {
+//                        for (;;) {
+//                            mySession.foldersChanged(0L);
+//                            Thread.sleep(100L);
+//                            mySession.folderContentChanged(0L, getName());
+//                            Thread.sleep(100L);
+//                        }
+//                    } catch (InterruptedException ignored) {
+//                        // exit on interrupt
+//                    }
+//                }
+//            };
+//            writeThreads[i].start();
+//        }
+//        Thread readThread = new Thread("Reader") {
+//
+//            @Override
+//            public void run() {
+//                try {
+//                    for (;;) {
+//                        ObjectChanges result = mySession.waitForChanges(true, null, 1000L);
+//                        System.out.println("Result: " + result);
+//                    }
+//                } catch (USMException e) {
+//                    e.printStackTrace();
+//                } catch (InterruptedException e) {
+//                    e.printStackTrace();
+//                }
+//            }
+//        };
+//        readThread.start();
+//        // try {
+//        // Thread.sleep(10000L);
+//        // } catch (InterruptedException e) {
+//        // }
+//        // for (Thread t : writeThreads)
+//        // t.interrupt();
+//        // readThread.interrupt();
+//    }
 
     public TimeZone getUserTimeZone() {
         return _userTimeZone;
@@ -1139,15 +1164,19 @@ public class SessionImpl implements Session, Serializable {
     public void setMailLimit(int limit) {
         _mailLimit = limit;
     }
-    
-    public void setObjectsLimit(int limit) {
-        _sessionManager.setMaxObjectsPerFolder(limit);
-    }
 
     public int getObjectsLimit() {
         return _sessionManager.getMaxObjectsPerFolder();
     }
 
+    public int getPIMAttachmentsCountLimit() {
+        return _sessionManager.getAttachmentCountLimitPerObject();
+    }
+
+    public long getPIMAttachmentsSizeLimit() {
+        return _sessionManager.getAttachmentSizeLimitPerObject();
+    }
+
     public String getFolderID(String shortFolderID) throws USMSQLException {
         if (shortFolderID.length() > 0) {
             switch (shortFolderID.charAt(0)) {
@@ -1211,9 +1240,9 @@ public class SessionImpl implements Session, Serializable {
     }
 
     public void endSyncronization() throws USMSQLException, DatabaseAccessException {
-        _dataObjectCache.removeAllObjectsForSession();
+        _syncStateStorage.removeAllObjectsForSession();
         _persistentData.removeAllDBFieldsForSession();
-        _sessionManager.removeSessionFromStorage(this);
+        _sessionManager.getSessionStorage().removeSession(this);
     }
 
     public void storeUUID(ContentType contentType, int objectId, UUID uuid, int folderOwner) throws USMException {
@@ -1249,7 +1278,7 @@ public class SessionImpl implements Session, Serializable {
     }
 
     public void remapCachedData(String oldObjectID, String newObjectID) throws DatabaseAccessException, USMSQLException {
-        _dataObjectCache.remapStates(oldObjectID, newObjectID);
+        _syncStateStorage.remapStates(oldObjectID, newObjectID);
     }
 
     public void setLastAccessCheck(long lastAccessCheck) {
@@ -1335,7 +1364,7 @@ public class SessionImpl implements Session, Serializable {
         Log journal = _sessionManager.getJournal();
         try {
             String shortID = getShortFolderID(folderID);
-            return _dataObjectCache.getNewestTimestamp(shortID);
+            return _syncStateStorage.getNewestTimestamp(shortID);
         } catch (DatabaseAccessException e) {
             if (journal.isDebugEnabled())
                 journal.debug(this + " DatabaseAccessException on getting the newest timestamp for folder: " + folderID);
@@ -1362,8 +1391,8 @@ public class SessionImpl implements Session, Serializable {
             markFolderHierarchyDirty();
     }
 
-    public DataObjectStorage getDataObjectCache() {
-        return _dataObjectCache;
+    public SyncStateStorage getDataObjectCache() {
+        return _syncStateStorage;
     }
 
     public Lock getOXConnectionLock() {
@@ -1381,4 +1410,26 @@ public class SessionImpl implements Session, Serializable {
     public void unlock(String id) {
         SynchronizationLock.unlock(this, id);
     }
+    
+    public SessionInitializer getSessionInitializer() {
+        return _sessionInitializer;
+    }
+
+    private void removeUnusedFoldersFromSyncStateStorage(Folder[] cachedFolders) throws DatabaseAccessException, USMSQLException {
+        if (cachedFolders == null)
+            return;
+        List<String> shortIDs = new ArrayList<String>();
+        List<String> folderIDs = new ArrayList<String>();
+        for (DataObject o : cachedFolders) {
+            String id = o.getID();
+            folderIDs.add(id);
+            shortIDs.add(getShortFolderID(id));
+        }
+        shortIDs.add(FOLDER_HIERARCHY_ID);
+        for (String specialFolder : _sessionManager.getSpecialFolders())
+            shortIDs.add(getShortFolderID(specialFolder));
+        _syncStateStorage.retain(shortIDs.toArray(new String[shortIDs.size()]));
+        if (_folderIdMapping != null)
+            _folderIdMapping.retain(folderIDs.toArray(new String[folderIDs.size()]));
+    }
 }
