@@ -61,6 +61,7 @@ import java.util.UUID;
 import javax.management.MalformedObjectNameException;
 import javax.management.NotCompliantMBeanException;
 import javax.management.ObjectName;
+import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
 import org.apache.commons.logging.Log;
 import org.json.JSONObject;
 import com.openexchange.exception.OXException;
@@ -68,6 +69,7 @@ import com.openexchange.management.ManagementService;
 import com.openexchange.timer.ScheduledTimerTask;
 import com.openexchange.timer.TimerService;
 import com.openexchange.usm.api.USMVersion;
+import com.openexchange.usm.api.cache.SyncStateCacheProvider;
 import com.openexchange.usm.api.contenttypes.ContentType;
 import com.openexchange.usm.api.contenttypes.ContentTypeManager;
 import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
@@ -79,13 +81,16 @@ import com.openexchange.usm.api.exceptions.AuthenticationFailedException;
 import com.openexchange.usm.api.exceptions.OXCommunicationException;
 import com.openexchange.usm.api.exceptions.USMAccessDeniedException;
 import com.openexchange.usm.api.exceptions.USMException;
+import com.openexchange.usm.api.exceptions.USMResourceAccessException;
 import com.openexchange.usm.api.exceptions.USMSQLException;
 import com.openexchange.usm.api.exceptions.USMStartupException;
 import com.openexchange.usm.api.session.DataObject;
 import com.openexchange.usm.api.session.Folder;
 import com.openexchange.usm.api.session.Session;
+import com.openexchange.usm.api.session.SessionID;
 import com.openexchange.usm.api.session.SessionInitializer;
 import com.openexchange.usm.api.session.SessionManager;
+import com.openexchange.usm.api.session.UserID;
 import com.openexchange.usm.configuration.ConfigurationManager;
 import com.openexchange.usm.configuration.ConfigurationProperties;
 import com.openexchange.usm.configuration.USMInvalidConfigurationException;
@@ -95,10 +100,12 @@ import com.openexchange.usm.ox_event.OXEventManager;
 import com.openexchange.usm.ox_json.OXJSONAccess;
 import com.openexchange.usm.session.dataobject.DataObjectSet;
 import com.openexchange.usm.session.dataobject.FolderImpl;
+import com.openexchange.usm.session.impl.cache.MemorySyncStateCacheProvider;
 import com.openexchange.usm.session.sync.IncrementalContentSyncer;
 import com.openexchange.usm.session.sync.OXDataCache;
 import com.openexchange.usm.session.sync.SlowContentSyncer;
 import com.openexchange.usm.util.JSONToolkit;
+import com.openexchange.usm.util.TempFileStorage;
 import com.openexchange.usm.util.Toolkit;
 import com.openexchange.usm.uuid.OXObjectID;
 import com.openexchange.usm.uuid.UUIDMappingService;
@@ -111,7 +118,7 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
     // LATER we might want to make this constant configurable
     private final static long ACCESS_CHECK_INTERVAL = 300000L;
 
-    //private final static long SESSION_CACHE_CHECK_INTERVAL = 5000L;
+    // private final static long SESSION_CACHE_CHECK_INTERVAL = 5000L;
 
     private final static long ONE_DAY_IN_MILLISECONDS = 24L * 60L * 60L * 1000L;
 
@@ -162,7 +169,7 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
     private int _waitForChangesEmailMinPullDelay = -1;
 
     private int _maxSyncStatesInDB = ConfigurationProperties.SYNC_MAX_STATES_IN_DB_DEFAULT;
-    
+
     private int _maxObjectsPerFolder = ConfigurationProperties.OBJECTS_IN_FOLDER_SYNC_LIMIT_DEFAULT;
 
     private boolean _keepLastSentSyncStateInDB = ConfigurationProperties.SYNC_KEEP_LAST_SENT_STATE_IN_DB_DEFAULT;
@@ -174,19 +181,25 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
     private ScheduledTimerTask _cleanupTask;
 
     private ScheduledTimerTask _sessionCacheCleanupTask;
-    
+
     private ScheduledTimerTask _sessionCacheCleanupLogInfoTask;
 
     private int _syncRetryCount;
 
-    private boolean _ignoreClientDataAtSlowSync;
-
     private boolean _forceNewSyncKeyOnEmptySync;
 
     private int _sessionCacheTimeout;
 
     private int _syncStateCacheTimeout;
 
+    private int _attachmentCountLimitPerObject;
+
+    private long _attachmentSizeLimitPerObject;
+
+    private final SyncStateCacheProvider _defaultCacheProvider = new MemorySyncStateCacheProvider(this);
+
+    private SyncStateCacheProvider _syncStateCacheProvider;
+
     public SessionManagerImpl() {
     }
 
@@ -200,7 +213,7 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
         _dataCache = new OXDataCache(0, 0);
     }
 
-    public void activate(Log log, OXJSONAccess ajaxAccess, ContentTypeManager contentTypeManager, FolderContentType folderContentType, OXEventManager oxEventManager, ConfigurationManager configurationManager, DatabaseAccess dbAccess, ObjectIdMappingService mappingService, UUIDMappingService uuidService, TimerService timerService) {
+    public void activate(Log log, OXJSONAccess ajaxAccess, ContentTypeManager contentTypeManager, FolderContentType folderContentType, OXEventManager oxEventManager, ConfigurationManager configurationManager, DatabaseAccess dbAccess, ObjectIdMappingService mappingService, UUIDMappingService uuidService) {
         _journal = log;
         _oxAjaxAccess = ajaxAccess;
         _contentTypeManager = contentTypeManager;
@@ -210,7 +223,6 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
         _databaseAccess = dbAccess;
         _mappingService = mappingService;
         _uuidService = uuidService;
-        _timerService = timerService;
         try {
             _syncRetryCount = _configurationManager.getProperty(
                 ConfigurationProperties.SYNC_CONCURRENT_MODIFICATION_MAX_RETRIES_PROPERTY,
@@ -300,18 +312,6 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
                 e);
         }
 
-        try {
-            _ignoreClientDataAtSlowSync = _configurationManager.getProperty(
-                ConfigurationProperties.USM_IGNORE_CLIENT_DATA_AT_SLOW_SYNC,
-                ConfigurationProperties.USM_IGNORE_CLIENT_DATA_AT_SLOW_SYNC_DEFAULT,
-                false);
-        } catch (USMInvalidConfigurationException e) {
-            throw new USMStartupException(
-                USMSessionManagementErrorCodes.CONFIGURATION_ERROR_12,
-                "Invalid configuration for " + ConfigurationProperties.USM_IGNORE_CLIENT_DATA_AT_SLOW_SYNC,
-                e);
-        }
-
         try {
             _forceNewSyncKeyOnEmptySync = _configurationManager.getProperty(
                 ConfigurationProperties.USM_FORCE_NEW_SYNC_KEY_ON_EMPTY_SYNC,
@@ -345,7 +345,7 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
                 "Invalid configuration for " + ConfigurationProperties.USM_CACHE_SYNC_STATE_TIMEOUT_PROPERTY,
                 e);
         }
-        
+
         try {
             _maxObjectsPerFolder = _configurationManager.getProperty(
                 ConfigurationProperties.OBJECTS_IN_FOLDER_SYNC_LIMIT_PROPERTY,
@@ -354,7 +354,43 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
         } catch (USMInvalidConfigurationException e) {
             throw new USMStartupException(
                 USMSessionManagementErrorCodes.CONFIGURATION_ERROR_16,
-                "Configuration for max_objects_per_folder is invalid",
+                "Invalid configuration for " + ConfigurationProperties.OBJECTS_IN_FOLDER_SYNC_LIMIT_PROPERTY,
+                e);
+        }
+
+        try {
+            _attachmentCountLimitPerObject = _configurationManager.getProperty(
+                ConfigurationProperties.ATTACHMENT_COUNT_LIMIT_PER_OBJECT_PROPERTY,
+                ConfigurationProperties.ATTACHMENT_COUNT_LIMIT_PER_OBJECT_DEFAULT,
+                false);
+        } catch (USMInvalidConfigurationException e) {
+            throw new USMStartupException(
+                USMSessionManagementErrorCodes.CONFIGURATION_ERROR_17,
+                "Invalid configuration for " + ConfigurationProperties.ATTACHMENT_COUNT_LIMIT_PER_OBJECT_PROPERTY,
+                e);
+        }
+
+        try {
+            _attachmentSizeLimitPerObject = (long) _configurationManager.getProperty(
+                ConfigurationProperties.ATTACHMENT_SIZE_LIMIT_PER_OBJECT_PROPERTY,
+                ConfigurationProperties.ATTACHMENT_SIZE_LIMIT_PER_OBJECT_DEFAULT,
+                false);
+        } catch (USMInvalidConfigurationException e) {
+            throw new USMStartupException(
+                USMSessionManagementErrorCodes.CONFIGURATION_ERROR_18,
+                "Invalid configuration for " + ConfigurationProperties.ATTACHMENT_SIZE_LIMIT_PER_OBJECT_PROPERTY,
+                e);
+        }
+
+        try {
+            TempFileStorage.setTempFileTimeout(_configurationManager.getProperty(
+                ConfigurationProperties.TEMP_FILES_TIMEOUT_PROPERTY,
+                ConfigurationProperties.TEMP_FILES_TIMEOUT_DEFAULT,
+                false));
+        } catch (USMInvalidConfigurationException e) {
+            throw new USMStartupException(
+                USMSessionManagementErrorCodes.CONFIGURATION_ERROR_19,
+                "Invalid configuration for " + ConfigurationProperties.TEMP_FILES_TIMEOUT_PROPERTY,
                 e);
         }
 
@@ -368,7 +404,7 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
     }
 
     public UUID getContextUUID(int cid) throws DatabaseAccessException, USMSQLException {
-        return _uuidService.getContextUUID(cid);
+        return _uuidService == null ? null : _uuidService.getContextUUID(cid);
     }
 
     public void deactivate() {
@@ -393,15 +429,17 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
         return getSession(user, password, protocol, device, null, "127.0.0.1", new HashMap<String, String>());
     }
 
-    // FIXME This code must be modified to use a more efficient approach. Iterating over all context ids takes way too long for large installations (e.g. 2 mio contexts)
-    // FIXME E.g. first check if USM session exists in memory and determine context id from it. if it doesn't exist in memory, try to login to OX server, and determine
+    // FIXME This code must be modified to use a more efficient approach. Iterating over all context ids takes way too long for large
+    // installations (e.g. 2 mio contexts)
+    // FIXME E.g. first check if USM session exists in memory and determine context id from it. if it doesn't exist in memory, try to login
+    // to OX server, and determine
     // FIXME context id after login was successful. Finally, perform database check only on previously determined context id
     public boolean existsSessionInDB(String user, String protocol, String device) throws USMException {
         SessionID id = new SessionID(user, protocol, device);
         SessionImpl session = _sessionStorage.getSession(id);
         if (session != null)
             return session.checkProperDBStorage();
-        
+
         List<Integer> contextIds = _databaseAccess.getAllContextIds();
         for (Integer contextId : contextIds) {
             EncapsulatedConnection con = _databaseAccess.getReadOnly(contextId);
@@ -447,14 +485,14 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
             // Cached Session is no longer present in DB, start from scratch
             _sessionStorage.removeSession(session);
         }
-        session = new SessionImpl(this, user, password, protocol, device);
+        session = new SessionImpl(this, user, password, protocol, device, initializer);
         session.setClientIp(clientIP);
         session.setXRequestHeaders(xHeaders);
         _oxAjaxAccess.login(session);
         JSONObject configuration = _oxAjaxAccess.getConfiguration(session);
         checkUSMAccess(session, configuration);
         session.setLastAccessCheck(System.currentTimeMillis());
-        session.initialize(initializer, configuration);
+        session.initialize(configuration);
         _sessionStorage.storeSession(session);
         _usmCacheInformation.newSessionCreated();
         session.setPersistentField(LAST_ACCESS_FIELD_NAME, String.valueOf(System.currentTimeMillis()));
@@ -475,14 +513,14 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
         if (_usmInterface == null || _usmInterface.length() == 0) {
             JSONObject usmModule = getUSMModule(session, configuration, "com.openexchange.usm");
             if (usmModule != null && usmModule.optBoolean("active"))
-                return;
+              return;
         } else {
             JSONObject oxInterfaces = getUSMModule(session, configuration, "interfaces");
             if (oxInterfaces != null && oxInterfaces.optBoolean(_usmInterface))
                 return;
         }
         _oxAjaxAccess.logout(session);
-        throw new USMAccessDeniedException(USMSessionManagementErrorCodes.USM_ACCESS_DISABLED, "USM access disabled for user");
+        throw new USMAccessDeniedException(USMSessionManagementErrorCodes.USM_ACCESS_DISABLED, "USM access disabled for user " + session.getUserIdentifier() + " in context " + session.getContextId());
     }
 
     private JSONObject getUSMModule(SessionImpl session, JSONObject configuration, String module) throws AuthenticationFailedException {
@@ -527,14 +565,6 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
         return _keepLastSentSyncStateInDB;
     }
 
-    public void removeSessionFromStorage(SessionImpl session) {
-        _sessionStorage.removeSession(session);
-    }
-
-    public boolean isIgnoreClientDataAtSlowSync() {
-        return _ignoreClientDataAtSlowSync;
-    }
-
     // --- Interface OXEventListener ---
 
     public void folderChanged(int contextId, int userId, String folderId, long timestamp) {
@@ -628,10 +658,12 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
                 int folderOwner = o.getParentFolderOwnerID();
                 if (folderOwner == o.getSession().getUserIdentifier())
                     folderOwner = 0;
-                _uuidService.storeMapping(getModifedUUID(o.getUUID(), folderOwner), new OXObjectID(
-                    o.getSession().getContextId(),
-                    contentTypeCode,
-                    objectId));
+                if (_uuidService != null) {
+                    _uuidService.storeMapping(getModifedUUID(o.getUUID(), folderOwner), new OXObjectID(
+                        o.getSession().getContextId(),
+                        contentTypeCode,
+                        objectId));
+                }
             } catch (NumberFormatException ignored) {
                 // For all cases in which we have no integer OX object id (e.g. mail folders), only store UUIDs locally in sync states
             }
@@ -705,8 +737,16 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
             _uuidService.removeMapping(contextID, oxObjectId);
         } catch (DatabaseAccessException e) {
             _journal.error("Database could not be accessed to delete UUID mapping for " + oxObjectId);
+            throw new USMResourceAccessException(
+                USMSessionManagementErrorCodes.OX_EVENT_DB_ERROR_3,
+                "Database could not be accessed to delete UUID mapping for " + oxObjectId,
+                e);
         } catch (USMSQLException e) {
             _journal.error("SQL error occurred while deleting UUID mapping for " + oxObjectId);
+            throw new USMResourceAccessException(
+                USMSessionManagementErrorCodes.OX_EVENT_DB_ERROR_4,
+                "SQL error occurred while deleting UUID mapping for " + oxObjectId,
+                e);
         }
     }
 
@@ -796,33 +836,31 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
             _sessionCacheCleanupTask.cancel();
             _sessionCacheCleanupTask = null;
         }
-        if (_sessionCacheCleanupLogInfoTask != null) {
-            _sessionCacheCleanupLogInfoTask.cancel();
-            _sessionCacheCleanupLogInfoTask = null;
-        }
     }
-    
+
     public void setTimerService(TimerService timerService) {
         deactivateTimers();
-        if (_timerService != null && _configurationManager != null) { // Only start new tasks if bundle is activated and TimerService is available
+        _timerService = timerService;
+        if (_timerService != null && _configurationManager != null) { // Only start new tasks if bundle is activated and TimerService is
+                                                                      // available
             _cleanupTask = _timerService.scheduleAtFixedRate(
                 new USMSessionCleanupTask(this),
                 USMSessionCleanupTask.INITIAL_DELAY,
                 USMSessionCleanupTask.EXECUTION_INTERVAL);
-            
+
             long sessionCacheCheckInterval = ConfigurationProperties.USM_SESSION_CACHE_CHECK_INTERVAL_DEFAULT;
             long sessionCacheCheckIntervalLogInfo = ConfigurationProperties.USM_SESSION_CACHE_CHECK_INTERVAL_LOG_INFO_DEFAULT;
-            
+
             try {
                 sessionCacheCheckInterval = _configurationManager.getProperty(
-                                ConfigurationProperties.USM_SESSION_CACHE_CHECK_INTERVAL_PROPERTY, 
-                                ConfigurationProperties.USM_SESSION_CACHE_CHECK_INTERVAL_DEFAULT,
-                                true);
-                
+                    ConfigurationProperties.USM_SESSION_CACHE_CHECK_INTERVAL_PROPERTY,
+                    ConfigurationProperties.USM_SESSION_CACHE_CHECK_INTERVAL_DEFAULT,
+                    true);
+
                 sessionCacheCheckIntervalLogInfo = _configurationManager.getProperty(
-                                ConfigurationProperties.USM_SESSION_CACHE_CHECK_INTERVAL_LOG_INFO_PROPERTY, 
-                                ConfigurationProperties.USM_SESSION_CACHE_CHECK_INTERVAL_LOG_INFO_DEFAULT,
-                                true);
+                    ConfigurationProperties.USM_SESSION_CACHE_CHECK_INTERVAL_LOG_INFO_PROPERTY,
+                    ConfigurationProperties.USM_SESSION_CACHE_CHECK_INTERVAL_LOG_INFO_DEFAULT,
+                    true);
 
             } catch (USMInvalidConfigurationException e) {
                 _journal.error(e.getStackTrace(), e);
@@ -831,7 +869,7 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
                 new USMSessionCacheCleanupTask(this),
                 sessionCacheCheckInterval,
                 sessionCacheCheckInterval);
-            
+
             _sessionCacheCleanupLogInfoTask = _timerService.scheduleAtFixedRate(
                 new USMSessionCacheCleanupLogInfoTask(this),
                 sessionCacheCheckIntervalLogInfo,
@@ -898,15 +936,21 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
     public void setSyncStateCacheTimeout(int timeout) {
         _syncStateCacheTimeout = timeout;
     }
-    
-    
+
     public int getMaxObjectsPerFolder() {
         return _maxObjectsPerFolder;
     }
 
-    
+    public int getAttachmentCountLimitPerObject() {
+        return _attachmentCountLimitPerObject;
+    }
+
+    public long getAttachmentSizeLimitPerObject() {
+        return _attachmentSizeLimitPerObject;
+    }
+
     public void setMaxObjectsPerFolder(int maxObjectsPerFolder) {
-       _maxObjectsPerFolder = maxObjectsPerFolder;
+        _maxObjectsPerFolder = maxObjectsPerFolder;
     }
 
     public void updateCacheRemovals(int removedSessions, int jvmRemovals, int removedSyncStates) {
@@ -914,7 +958,7 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
         _usmCacheInformation.sessionsRemovedByJVM(jvmRemovals);
         _usmCacheInformation.syncStatesRemovedDueToInactivity(removedSyncStates);
     }
-    
+
     public USMCacheInformation getUSMCacheInformation() {
         return _usmCacheInformation;
     }
@@ -926,4 +970,17 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
     public void syncStatesLoadedFromDatabase(int count) {
         _usmCacheInformation.syncStatesLoadedFromDatabase(count);
     }
+
+    public MultiThreadedHttpConnectionManager getHttpConnectionManager() {
+        return _oxAjaxAccess.getHttpConnectionManager();
+    }
+
+    public void setSyncStateCacheProvider(SyncStateCacheProvider provider) {
+        _syncStateCacheProvider = provider;
+    }
+
+    public SyncStateCacheProvider getSyncStateCacheProvider() {
+        SyncStateCacheProvider provider = _syncStateCacheProvider;
+        return (provider == null) ? _defaultCacheProvider : provider;
+    }
 }
