@@ -58,6 +58,9 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.ConcurrentHashMap;
+import com.openexchange.usm.api.session.CompleteSessionID;
+import com.openexchange.usm.api.session.SessionID;
+import com.openexchange.usm.api.session.UserID;
 import com.openexchange.usm.ox_json.OXJSONAccess;
 
 public class SessionStorage {
@@ -97,6 +100,7 @@ public class SessionStorage {
             if (completeSessionId != null)
                 removeFromUserDeviceMap(completeSessionId.getUserID(), completeSessionId.getSessionID());
         }
+        _sessionManager.getSyncStateCacheProvider().freeCache(session.getCompleteSessionID());
         _sessionManager.getOXAjaxAccess().logout(session);
     }
 
@@ -174,6 +178,7 @@ public class SessionStorage {
             if (id != null) {
                 removeFromUserDeviceMap(id.getUserID(), id.getSessionID());
                 _map.remove(id.getSessionID());
+                _sessionManager.getSyncStateCacheProvider().freeCache(id);
                 _sessionManager.getOXAjaxAccess().logout(id.getOXConnectionInformation());
                 // SoftReference<SessionImpl> oldRef = _map.remove(id.getSessionID());
                 // assert (ref == oldRef);
