@@ -51,16 +51,15 @@ package com.openexchange.usm.session.impl;
 
 import org.apache.commons.logging.Log;
 
-
 /**
  * {@link USMSessionCacheCleanupLogInfoTask}
- *
+ * 
  * @author <a href="mailto:ioannis.chouklis@open-xchange.com">Ioannis Chouklis</a>
  */
 public class USMSessionCacheCleanupLogInfoTask implements Runnable {
-    
+
     private final SessionManagerImpl _sessionManager;
-    
+
     public USMSessionCacheCleanupLogInfoTask(SessionManagerImpl manager) {
         _sessionManager = manager;
     }
@@ -74,8 +73,7 @@ public class USMSessionCacheCleanupLogInfoTask implements Runnable {
         int removedSessions = _sessionManager.getUSMCacheInformation().getSessionsRemovedSinceLastLogging();
         int jvmRemovals = _sessionManager.getUSMCacheInformation().getSessionsRemovedByJVMSinceLastLogging();
         int removedSyncStates = _sessionManager.getUSMCacheInformation().getSyncStatesRemovedSinceLastLogging();
-        log.info(removedSessions + " USM sessions (and " + 
-                 jvmRemovals + " by JVM) and " + 
-                 removedSyncStates + " SyncStates have been removed from memory cache since the last cleanup task.");
+        log.info(removedSessions + " USM sessions (and " + jvmRemovals + " by JVM) and " + removedSyncStates + " SyncStates have been removed from memory cache since the last cleanup task.");
+        //        SyncStateStorageStatistics.reportStatistics(log);
     }
 }
