@@ -67,7 +67,7 @@ public class USMSessionInformation extends StandardMBean implements USMSessionIn
     private final static Map<String, String[]> PARAMETER_NAMES = new HashMap<String, String[]>();
 
     private final static Map<String, String[]> PARAMETER_DESCRIPTIONS = new HashMap<String, String[]>();
-
+    
     static {
         DESCRIPTIONS.put(
             "WaitForChangesEmailPullDelay",
@@ -119,8 +119,14 @@ public class USMSessionInformation extends StandardMBean implements USMSessionIn
             "SyncStatesLoadedFromDatabase",
             "Number of SyncStates that have been restored from the database (since the last counter reset)");
         DESCRIPTIONS.put(
-            "getSyncStatesRemovedDueToInactivity",
-            "Number of SyncStates that have been removed from the memory cache due to inactivity (since the last counter reset)");
+            "SyncStatesRemovedDueToInactivity",
+            "Number of SyncStates that have been removed from the memory cache due to inactivity (since the last counter reset)");       
+        DESCRIPTIONS.put(
+            "MaxTotalConnections",
+            "The overall maximum number of (concurrent) connections the USM may use for requests to the OX server.");
+        DESCRIPTIONS.put(
+            "ConnectionsInUse",
+            "Overall number of http connections in use (USM -> OX server)");
 
         PARAMETER_NAMES.put("getPropertiesOfActiveSessions", new String[] { "cid", "id", "user", "protocol", "device", "field" });
         PARAMETER_DESCRIPTIONS.put("getPropertiesOfActiveSessions", new String[] {
@@ -346,4 +352,13 @@ public class USMSessionInformation extends StandardMBean implements USMSessionIn
     public int getSyncStatesRemovedDueToInactivity() {
         return _usmCacheInformation.getSyncStatesRemovedDueToInactivity();
     }
+
+    @SuppressWarnings("deprecation")
+    public int getMaxTotalConnections() {
+        return _sessionManager.getHttpConnectionManager().getMaxTotalConnections();
+    }
+
+    public int getConnectionsInUse() {
+        return _sessionManager.getHttpConnectionManager().getConnectionsInPool();
+    }
 }
