@@ -106,4 +106,8 @@ public interface USMSessionInformationMBean {
     int getSyncStatesLoadedFromDatabase();
 
     int getSyncStatesRemovedDueToInactivity();
+    
+    int getMaxTotalConnections();
+    
+    int getConnectionsInUse();
 }
