@@ -123,8 +123,6 @@ public class USMSessionManagementErrorCodes {
 
     public static final int CONFIGURATION_ERROR_11 = 0x300023;
 
-    public static final int CONFIGURATION_ERROR_12 = 0x300024;
-
     public static final int CONFIGURATION_ERROR_13 = 0x300025;
 
     public static final int CONFIGURATION_ERROR_14 = 0x300026;
@@ -132,8 +130,14 @@ public class USMSessionManagementErrorCodes {
     public static final int CONFIGURATION_ERROR_15 = 0x300027;
     
     public static final int CONFIGURATION_ERROR_16 = 0x300028;
-    
 
+    public static final int CONFIGURATION_ERROR_17 = 0x300029;
+
+    public static final int CONFIGURATION_ERROR_18 = 0x30002A;
+
+    public static final int CONFIGURATION_ERROR_19 = 0x30002B;
+    
+    
     public static final int UUID_GENERATE_OBJECT_ID_ERROR = 0x300031;
 
     public static final int CONFLICTS_SLOW_SYNC_UUID_ERRORS = 0x300033;
