@@ -52,6 +52,7 @@ package com.openexchange.usm.session.sync;
 import com.openexchange.usm.api.database.DatabaseAccessException;
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.exceptions.USMSQLException;
+import com.openexchange.usm.api.session.DataObject;
 import com.openexchange.usm.session.dataobject.DataObjectSet;
 
 public interface ContentSyncerStorage {
@@ -83,4 +84,15 @@ public interface ContentSyncerStorage {
      * @return {@link OXDataCacheID} for the OX server storage (a specific folder or the folder hierarchy for a session)
      */
     OXDataCacheID getCacheID();
+
+    /**
+     * Tries to find a matching client object for a given server object. This method is called if no client object with the same ID as the
+     * given server object could be found. The method must either return one of the objects in clientDataObjects or null if none of the
+     * objects matches based on other fields (e.g. standard_folder_type).
+     * 
+     * @param clientDataObjects set of available client objects to match
+     * @param serverObject server object to match against
+     * @return matching client object or null if none matches
+     */
+    DataObject findMatchingClientObjectForServerObject(DataObjectSet clientDataObjects, DataObject serverObject);
 }
