@@ -69,7 +69,9 @@ import com.openexchange.usm.session.dataobject.FolderHierarchyComparator;
 import com.openexchange.usm.session.impl.SessionManagerImpl;
 
 public class ContentSyncerSupport {
-	private static final String ID = "id";
+	private static final String TITLE = "title";
+
+    private static final String ID = "id";
 
 	private static final boolean READ_SERVER_CHANGES_ON_CREATION = true;
 
@@ -146,6 +148,14 @@ public class ContentSyncerSupport {
 		executeServerUpdatesInternal(sessionManager, serverObjects, creations, changes, deletions, resultMap, true,
 				serverChanges);
 	}
+	
+	private static String getLatestMappedID(String originalID, Map<String,String> folderIdMapping) {
+            String newMappedID = folderIdMapping.get(originalID);
+            if(!folderIdMapping.containsKey(newMappedID))
+                return newMappedID;
+            
+            return  getLatestMappedID(newMappedID, folderIdMapping);
+	}
 
 	private static void executeServerUpdatesInternal(final SessionManagerImpl sessionManager,
 			final DataObjectSet serverObjects, List<DataObject> creations, List<DataObject> changes,
@@ -158,26 +168,41 @@ public class ContentSyncerSupport {
 				if (folderIdMapping.containsKey(change.getOriginalParentFolderID())
 						|| folderIdMapping.containsKey(change.getOriginalID())) {
 					DataObject serverObjectCopy = serverObject.createCopy(true);
-					if (folderIdMapping.containsKey(change.getOriginalParentFolderID()))
-						serverObjectCopy.setParentFolderID(folderIdMapping.get(change.getOriginalParentFolderID()));
-					if (folderIdMapping.containsKey(change.getOriginalID()))
-						serverObjectCopy.setID(folderIdMapping.get(change.getOriginalID()));
-					serverObjectCopy.commitChanges();
 					DataObjectUtil.copyModifiedFields(serverObjectCopy, change);
+					if (folderIdMapping.containsKey(serverObjectCopy.getParentFolderID()))
+						serverObjectCopy.setParentFolderID(getLatestMappedID(serverObjectCopy.getParentFolderID(), folderIdMapping)); 
+					if (folderIdMapping.containsKey(serverObjectCopy.getID()))
+						serverObjectCopy.setID(getLatestMappedID(serverObjectCopy.getID(), folderIdMapping)); 
+					
 					serverObjectCopy.getContentType().getTransferHandler().writeUpdatedDataObject(serverObjectCopy,
 							serverObjectCopy.getTimestamp());
+					if (READ_SERVER_CHANGES_ON_MODIFICATION && isFolder) {
+					    readServerChanges(sessionManager, serverChanges, serverObjectCopy, (Folder) serverObjectCopy, serverObjectCopy.getContentType());
+	                }
+					 DataObjectUtil.copyModifiedFields(change, serverObjectCopy);
+					 DataObjectUtil.copyModifiedFields(serverObject, serverObjectCopy);
+                     serverObject.commitChanges();
 				} else {
 					change.getContentType().getTransferHandler().writeUpdatedDataObject(change, change.getTimestamp());
-				}
-				if (serverObject != null) {
-					DataObjectUtil.copyModifiedFields(serverObject, change);
-					serverObject.commitChanges();
+					if (READ_SERVER_CHANGES_ON_MODIFICATION && isFolder) {
+	                    readServerChanges(sessionManager, serverChanges, change, (Folder) change, change.getContentType());
+	                }
+					if (serverObject != null) {
+					    DataObjectUtil.copyModifiedFields(serverObject, change);
+					    serverObject.commitChanges();
+					}
 				}
 				if (isFolder && serverObject != null && change.isFieldModified(change.getFieldIndex(ID))) {
-					//this means that we have renamed/moved a mail folder and the id is changed 
+					//this means that we have renamed/moved a mail folder and the id is changed and we have to remap all cached data to the new id
 				    String originalID = change.getOriginalID();
 				    String newID = serverObject.getID();
-					folderIdMapping.put(originalID, newID); //old to new
+				    
+				    String newOriginalID = getLatestMappedID(originalID, folderIdMapping); //if the parent folder has also been renamed in the same update and we already have a new id mapped
+				    if(newOriginalID != null) 
+				        originalID = newOriginalID;
+				    folderIdMapping.put(originalID, newID); //old to new
+				    
+				   
 					Session session = serverObject.getSession();
 					session.remapCachedData(session.getShortFolderID(originalID), session
 							.getShortFolderID(newID));
@@ -195,20 +220,17 @@ public class ContentSyncerSupport {
 							    newParentFolderID += "/" + childSubPath.substring(0, childParentDelimiterIndex);
 							}
 							String newChildID = newID + "/" + childSubPath;
-							folderIdMapping.put(originalID, newChildID); //old to new
+							folderIdMapping.put(childFolderID, newChildID); //old to new
 							childFolder.setParentFolderID(newParentFolderID);
 							childFolder.setID(newChildID);
 							session = childFolder.getSession();
-							session.remapCachedData(session.getShortFolderID(originalID), session
+							session.remapCachedData(session.getShortFolderID(childFolderID), session
 									.getShortFolderID(newChildID));
 							childFolder.commitChanges();
 						}
 					}
 				}
-
-				if (READ_SERVER_CHANGES_ON_MODIFICATION && isFolder) {
-					readServerChanges(sessionManager, serverChanges, change, (Folder) change, change.getContentType());
-				}
+				
 			}
 		});
 		// perform deletions of already existing objects
@@ -330,4 +352,4 @@ public class ContentSyncerSupport {
 			resultMap.put(update, e);
 		}
 	}
-}
+}
\ No newline at end of file
