@@ -124,4 +124,8 @@ public class FolderElementsStorage implements ContentSyncerStorage {
     public String toString() {
         return String.valueOf(_session) + ' ' + _folder.getID() + '(' + _storeResult + ')';
     }
+
+    public DataObject findMatchingClientObjectForServerObject(DataObjectSet clientDataObjects, DataObject serverObject) {
+        return null;
+    }
 }
