@@ -50,12 +50,15 @@
 package com.openexchange.usm.session.sync;
 
 import java.util.BitSet;
+import java.util.UUID;
 import com.openexchange.usm.api.contenttypes.ContentType;
+import com.openexchange.usm.api.contenttypes.FolderConstants;
 import com.openexchange.usm.api.contenttypes.FolderContentType;
 import com.openexchange.usm.api.contenttypes.FolderContentTypeTransferHandler;
 import com.openexchange.usm.api.database.DatabaseAccessException;
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.exceptions.USMSQLException;
+import com.openexchange.usm.api.session.DataObject;
 import com.openexchange.usm.api.session.DataObjectFilter;
 import com.openexchange.usm.api.session.Folder;
 import com.openexchange.usm.session.dataobject.DataObjectSet;
@@ -149,4 +152,19 @@ public class FolderHierarchyStorage implements ContentSyncerStorage {
         }
         f.commitChanges();
     }
+
+    public DataObject findMatchingClientObjectForServerObject(DataObjectSet clientDataObjects, DataObject serverObject) {
+        if (!_session.getSessionInitializer().objectsSynchronizedByUUID())
+            return null;
+        Object v = serverObject.getFieldContent(FolderConstants.STANDARD_FOLDER_TYPE);
+        if (!(v instanceof Number) || ((Number) v).intValue() == 0)
+            return null;
+        for (DataObject i : clientDataObjects) {
+            if (v.equals(i.getFieldContent(FolderConstants.STANDARD_FOLDER_TYPE)))
+                _session.getSessionManager().getJournal().warn(
+                    _session + " Standard folder (type " + v + ") changed OX id: '" + i.getID() + "' -> '" + serverObject.getID() + '\'');
+            return i;
+        }
+        return null;
+    }
 }
