@@ -49,7 +49,7 @@
 package com.openexchange.usm.session.sync;
 
 import java.io.Serializable;
-import com.openexchange.usm.session.impl.SessionID;
+import com.openexchange.usm.api.session.SessionID;
 import com.openexchange.usm.session.impl.SessionImpl;
 
 /**
