@@ -74,9 +74,6 @@ public class SlowContentSyncer extends BaseContentSyncer {
 		if (_journal.isDebugEnabled())
 			_journal.debug(storage + " Starting slow sync, resolution=" + conflictResolution);
 		
-		if(_sessionManager.isIgnoreClientDataAtSlowSync())
-			clientData = EMPTY_DATA_OBJECT_ARRAY;
-		
 		setParentFolderOwnerIDField(clientData);
 		for (int count = 0; count <= _retryCount; count++) {
 			if (filter != null)
